<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 31.01.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-12-21 17:06:47 +0100 (Fr, 21. Dez 2012) $
 * last changed by: $LastChangedBy: lazee $
 * revision       : $LastChangedRevision: 9852 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.libs/Hmt/Log.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
require_once 'Zend/Registry.php';
require_once 'Zend/Db/Adapter/Abstract.php';
require_once 'Hpls/Utils/Logusing.php';
require_once 'Hpls/Utils/Db.php';
require_once 'Hmt/Db/Query.php';
require_once 'Hmt/Db/Query/Servers.php';
require_once 'Hpls/Mailtrace/Consts.php';
require_once 'Hmt/Db/Query/Constraints.php';

/**
 *
 * @todo comment
 *       Zentrale Instanz zur Datenbank-Ansprache.
 *       <p>Sie beachtet, dass die Datenbank nach Tagen aufgeteilt ist und teilt
 *       entsprechend die Anfragen auf.
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9852 $
 * @since 1.0.9
 */
class Hmt_Log extends Hpls_Utils_Logusing {
	
	public static $DATE_FIELDS = array(
			Hpls_Mailtrace_Consts::SRCH_C_TIME,
			Hpls_Mailtrace_Consts::SRCH_D_TIME,
			Hpls_Mailtrace_Consts::SRCH_C_UTIME,
			Hpls_Mailtrace_Consts::SRCH_D_UTIME );
	
	public static $DELIVER_FIELD_MAP = array(
			Hpls_Mailtrace_Consts::SRCH_D_ID => Hmt_Db_Consts::F_D_ID,
			Hpls_Mailtrace_Consts::SRCH_D_DBID => Hmt_Db_Consts::F_D_REF_C_ID,
			// Hpls_Mailtrace_Consts::SRCH_MAILTO => Hmt_Db_Consts::F_D_MAILTO,
			Hpls_Mailtrace_Consts::SRCH_D_MAILTO_ADR => Hmt_Db_Consts::F_D_MAILTO_ADR,
			Hpls_Mailtrace_Consts::SRCH_D_MAILTO_DMN => Hmt_Db_Consts::F_D_MAILTO_DOMAIN,
			Hpls_Mailtrace_Consts::SRCH_D_ORIGINAL_MAILTO_ADR => Hmt_Db_Consts::F_D_ORIGTO_ADR,
			Hpls_Mailtrace_Consts::SRCH_D_ORIGINAL_MAILTO_DMN => Hmt_Db_Consts::F_D_ORIGTO_DOMAIN,
			Hpls_Mailtrace_Consts::SRCH_STATUS => Hmt_Db_Consts::F_D_STATUS,
			Hpls_Mailtrace_Consts::SRCH_REASON => Hmt_Db_Consts::F_D_REASON,
			Hpls_Mailtrace_Consts::SRCH_RELAY_HOST => Hmt_Db_Consts::F_D_RELAYHOST,
			Hpls_Mailtrace_Consts::SRCH_D_QUEUE_ID => Hmt_Db_Consts::F_D_QUEUE_ID,
			Hpls_Mailtrace_Consts::SRCH_D_TIME => Hmt_Db_Consts::F_D_TIME,
			Hpls_Mailtrace_Consts::SRCH_D_UTIME => Hmt_Db_Consts::F_D_CREATE_TIME );
	public static $CONNECT_FIELD_MAP = array(
			Hpls_Mailtrace_Consts::SRCH_C_ID => Hmt_Db_Consts::F_C_ID,
			Hpls_Mailtrace_Consts::SRCH_CLIENT => Hmt_Db_Consts::F_C_CLIENT,
			Hpls_Mailtrace_Consts::SRCH_C_CLIENT_HOST => Hmt_Db_Consts::F_C_CLIENT_HOST,
			Hpls_Mailtrace_Consts::SRCH_C_CLIENT_IP => Hmt_Db_Consts::F_C_CLIENT_IP,
			// Hpls_Mailtrace_Consts::SRCH_MAILFROM =>
			// Hmt_Db_Consts::F_C_MAILFROM,
			Hpls_Mailtrace_Consts::SRCH_C_MAIL_FROM_ADR => Hmt_Db_Consts::F_C_MAILFROM_ADR,
			Hpls_Mailtrace_Consts::SRCH_C_MAIL_FROM_DMN => Hmt_Db_Consts::F_C_MAILFROM_DOMAIN,
			Hpls_Mailtrace_Consts::SRCH_MSG_ID => Hmt_Db_Consts::F_C_MSG_ID,
			Hpls_Mailtrace_Consts::SRCH_SIZE => Hmt_Db_Consts::F_C_SIZE,
			Hpls_Mailtrace_Consts::SRCH_C_QUEUE_ID => Hmt_Db_Consts::F_C_QUEUE_ID,
			Hpls_Mailtrace_Consts::SRCH_MX => Hmt_Db_Consts::F_C_MXNAME,
			Hpls_Mailtrace_Consts::SRCH_C_TIME => Hmt_Db_Consts::F_C_TIME,
			Hpls_Mailtrace_Consts::SRCH_C_UTIME => Hmt_Db_Consts::F_C_CREATE_TIME );
	public static $CONNECT_SEARCH_FIELDS = array(
			Hpls_Mailtrace_Consts::SRCH_C_ID,
			Hpls_Mailtrace_Consts::SRCH_CLIENT,
			Hpls_Mailtrace_Consts::SRCH_MAILFROM,
			Hpls_Mailtrace_Consts::SRCH_C_MAIL_FROM_ADR,
			Hpls_Mailtrace_Consts::SRCH_C_MAIL_FROM_DMN,
			Hpls_Mailtrace_Consts::SRCH_MSG_ID,
			Hpls_Mailtrace_Consts::SRCH_SIZE,
			Hpls_Mailtrace_Consts::SRCH_C_QUEUE_ID,
			Hpls_Mailtrace_Consts::SRCH_MX,
			Hpls_Mailtrace_Consts::SRCH_C_TIME,
			Hpls_Mailtrace_Consts::SRCH_C_UTIME );
	
	/*
	 * deliver
	 */
	public static $DELIVER_SEARCH_FIELDS = array(
			Hpls_Mailtrace_Consts::SRCH_D_ID,
			Hpls_Mailtrace_Consts::SRCH_D_DBID,
			Hpls_Mailtrace_Consts::SRCH_MAILTO,
			Hpls_Mailtrace_Consts::SRCH_D_MAILTO_ADR,
			Hpls_Mailtrace_Consts::SRCH_D_MAILTO_DMN,
			Hpls_Mailtrace_Consts::SRCH_D_ORIGINAL_MAILTO_ADR,
			Hpls_Mailtrace_Consts::SRCH_D_ORIGINAL_MAILTO_DMN,
			Hpls_Mailtrace_Consts::SRCH_STATUS,
			Hpls_Mailtrace_Consts::SRCH_REASON,
			Hpls_Mailtrace_Consts::SRCH_RELAY_HOST,
			Hpls_Mailtrace_Consts::SRCH_D_QUEUE_ID,
			Hpls_Mailtrace_Consts::SRCH_D_TIME,
			Hpls_Mailtrace_Consts::SRCH_D_UTIME );
	
	/*
	 * server
	 */
	public static $SERVER_FIELD_MAP = array(
			Hpls_Mailtrace_Consts::SRCH_S_ID => Hmt_Db_Consts::F_S_ID,
			Hpls_Mailtrace_Consts::SRCH_S_NAME => Hmt_Db_Consts::F_S_NAME );
	
	public static $SERVER_SEARCH_FIELDS = array(
			Hmt_Db_Consts::F_S_ID,
			Hmt_Db_Consts::F_S_NAME );
	
	/**
	 * Felder, die beim Umschreiben der Hpls_Var_Clause in Hmt_Db_Query in
	 * mehrere Felder aufgespalten werden
	 * 
	 * @var array:string
	 */
	public static $REPLACEABLE_FIELDS = array(
			Hpls_Mailtrace_Consts::SRCH_MAILFROM => array(
					0 => Hpls_Mailtrace_Consts::SRCH_C_MAIL_FROM_ADR,
					1 => Hpls_Mailtrace_Consts::SRCH_C_MAIL_FROM_DMN ),
			Hpls_Mailtrace_Consts::SRCH_D_ORIGINAL_MAILTO => array(
					0 => Hpls_Mailtrace_Consts::SRCH_D_ORIGINAL_MAILTO_ADR,
					1 => Hpls_Mailtrace_Consts::SRCH_D_ORIGINAL_MAILTO_DMN ),
			Hpls_Mailtrace_Consts::SRCH_MAILTO => array(
					0 => Hpls_Mailtrace_Consts::SRCH_D_MAILTO_ADR,
					1 => Hpls_Mailtrace_Consts::SRCH_D_MAILTO_DMN ),
			Hpls_Mailtrace_Consts::SRCH_CLIENT => array(
					0 => Hpls_Mailtrace_Consts::SRCH_C_CLIENT_HOST,
					1 => Hpls_Mailtrace_Consts::SRCH_C_CLIENT_IP ) );
	
	protected static $_instance = null;
	/**
	 *
	 * @var Zend_Db_Adapter_Abstract
	 */
	protected $_db = NULL;
	protected $_intervals = null;
	protected $_type = null;

	/**
	 *
	 * @return Hmt_Log
	 */
	public static function i() {
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	protected function __construct() {
	}

	public static function isDeliverKey($key) {
		return (strncasecmp('d_', $key, 2) == 0) || (strncasecmp('d.', $key, 2) == 0);
	}

	public static function isConnectKey($key) {
		return (strncasecmp('c_', $key, 2) == 0) || (strcasecmp('dbid', $key) == 0) || (strncasecmp('d.', $key, 2) == 0);
	}

	public static function isServerKey($key) {
		return (strncasecmp('s_', $key, 2) == 0) || (strncasecmp('s.', $key, 2) == 0);
	}

	/**
	 *
	 * @param $query Hpls_Mailtrace_Db_Query
	 * @return Hpls_Mailtrace_Db_Query
	 */
	public function query(Hpls_Var_Clause $clause, Hmt_Db_Query_Constraints $constraints) {
		if (is_null($constraints)) {
			require_once 'Hmt/Db/Query/Exception.php';
			throw new Hmt_Db_Query_Exception('$constraints must not be empty.', 1);
		}
		$query = null;
		$isConQuery = $this->isConnectQuery($clause, $constraints);
		$isDeliverQuery = $this->isDeliverQuery($clause, $constraints);
		if ($isConQuery && $isDeliverQuery) {
			// both connect and deliver tables
			$query = new Hmt_Db_Query_LogSearch($clause, $constraints);
		} elseif ($isConQuery) {
			// only connect table
			$query = new Hmt_Db_Query_Connects($clause, $constraints);
		} elseif ($isDeliverQuery) {
			// only deliver table
			$query = new Hmt_Db_Query_Delivers($clause, $constraints);
		} else {
			return null;
		}
		
		$query->setDb($this->getDb());
		return $query;
	}

	public function connect(Hmt_Db_Query $query) {
		$query->setDb($this->getDb());
		return $query;
	}

	/**
	 *
	 * @param $query unknown_type
	 */
	public function getServerEntry($query) {
	
	}

	/**
	 *
	 * @return NULL array:Hmt_Log_ServerEntry
	 * @throws
	 *
	 *
	 *
	 *
	 */
	public function getServers() {
		$qry = $this->connect(new Hmt_Db_Query_Servers());
		$qry->execute();
		$r = $qry->getResults();

		#$this->log()->debug(' .::..::..: SORT :..::..::..::.. BEFORE: '.print_r($r, TRUE));
		$r = $this->serverSort($r);
		#$this->log()->debug(' .::..::..: SORT :..::..::..::.. AFTER: '.print_r($r, TRUE));
		return $r;
	}

	/**
	 *
	 * @todo comment
	 * @param $id unknown_type
	 * @return NULL Hmt_Log_ServerEntry
	 */
	public function getServer($id) {
		$qry = $this->connect(new Hmt_Db_Query_Servers(array(
				$id )));
		$qry->execute();
		$ret = $qry->getResults();
		if (count($ret) === 0) {
			return null;
		}
		return array_shift($ret);
	}
	
	public function serverSort($array_of_serverobjects) {
		if (!empty($array_of_serverobjects)) {
			usort($array_of_serverobjects, array('self','callbackServerSort'));
		}
		return $array_of_serverobjects;
	}
	
	public function callbackServerSort($a, $b) {
		$tmp_cmp_a = $a->s_name;
		$tmp_cmp_b = $b->s_name;
		if($tmp_cmp_a == $tmp_cmp_b) return 0;
		return ($tmp_cmp_a < $tmp_cmp_b) ? -1 : 1;
	}
	
	/**
	 *
	 * @return Zend_Db_Adapter_Abstract
	 */
	public function getDb() {
		if (is_null($this->_db)) {
			$config = Hpls_Settings::i()->getConfig('hmt.mtlog.storage.db');
			if (empty($config)) {
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration('No configuration for hmt.mtlog.storage.db in Settings', Hpls_Err::ERR_CONFIGURATION);
			}
			$dbConfig = $config->toArray();
			$this->_type = strtolower($dbConfig['type']);
			
			$this->_db = Hpls_Utils_Db::mapDriverType2ZendDb($dbConfig);
		}
		return $this->_db;
	}

	public function getCountServers() {
		$db = $this->getDb();
		$sql = $db->select();
		$sql->from(Hmt_Db_Consts::TABLE_SERVER, 'COUNT(*)');
		$stm = $db->query($sql);
		$this->log()->info(__METHOD__ . " SQL: " . $sql->assemble());
		
		return $stm->fetchColumn(0);
	}

	public function getCountDayTables() {
		$db = $this->getDb();
		$tables = $db->listTables();
		return $tables;
	}

	public function getConnectTables() {
		return Hpls_Utils::array_values_startWith($this->getTables(), Hmt_Db_Consts::TABLE_CONNECT);
	}

	public function getCountConnects($dates = NULL) {
		$tables = null;
		if (is_null($dates)) {
			$tables = $this->getConnectTables();
		} elseif (FALSE == is_array($dates)) {
			// @todo
		}
		$count = 0;
		foreach ($tables as $table) {
			$select = $this->getDb()->select();
			$select->from($table, 'COUNT(*)');
			$stm = $select->query();
			$count += (int) $stm->fetchColumn(0);
		}
		return $count;
	}

	/**
	 *
	 * @todo caching
	 */
	public function getDeliverTables() {
		return Hpls_Utils::array_values_startWith($this->getTables(), Hmt_Db_Consts::TABLE_DELIVER);
	}

	public function getCountDelivers($dates = NULL) {
		$tables = null;
		if (is_null($dates)) {
			$tables = $this->getDeliverTables();
		} elseif (FALSE == is_array($dates)) {
			// @todo
		}
		$count = 0;
		foreach ($tables as $table) {
			$select = $this->getDb()->select();
			$select->from($table, 'COUNT(*)');
			$stm = $select->query();
			$count += (int) $stm->fetchColumn(0);
		}
		return $count;
	}

	public function getProperty($key) {
	
	}

	protected function getDbDateFormat() {
		return 'Y-m-d\TH:i:s\%\sO';
	}

	public function getDefaultLimit() {
		return 30;
	}

	/**
	 *
	 * @todo comment
	 * @todo abfrage in DB statt statischer werte
	 */
	public function getDeliverStates() {
		return array(
				'sent',
				'deferred',
				'rejected',
				'bounced',
				'expired',
				// 'greylisted',
// 				'undeliverable',
// 				'deliverable',
				'unknown' );
	}

	/**
	 * Gibt die in der DB vorliegenden Intervalle sortiert von ältestem '[0]' zu
	 * jüngstem '[n]'.
	 * 
	 * @return NULL array:string
	 */
	public function getIntervals() {
		if (is_null($this->_intervals)) {
			$tables = $this->getConnectTables();
			$this->_intervals = array();
			$prefixLen = strlen(Hmt_Db_Consts::TABLE_CONNECT);
			foreach ($tables as $table) {
				$iv = substr($table, $prefixLen);
				$this->_intervals[] = str_pad($iv, 14, '0');
			}
			sort($this->_intervals);
		}
		return $this->_intervals;
	}

	public function isValidInterval($iv) {
		return in_array($iv, $this->getIntervals());
	}

	/**
	 * Gibt das zeitlich jüngste Interval zurück.
	 * 
	 * @return string
	 */
	public function getNewestInterval() {
		$intervals = $this->getIntervals();
		return end($intervals);
	}

	/**
	 * Gibt das zeitlich älteste Interval zurück.
	 * 
	 * @return string
	 */
	public function getOldestInterval() {
		$intervals = $this->getIntervals();
		return reset($intervals);
	}

	public function getContainingInterval($datetime) {
		if (intval($datetime) < 10) {
			$datetime = Hpls_Utils_Time::isoToTimestamp($datetime);
		}
		$interval = date('YmdHis', $datetime);
		$ivals = $this->getIntervals();
		$ret = $ivals[0];
		foreach ($this->getIntervals() as $ival) {
			if ($ival > $interval) {
				return $ret;
			}
			$ret = $ival;
		}
		return $ret;
	}

	public function getNextInterval($iv) {
		if ($this->isValidInterval($iv)) {
			$this->log()->debug(__METHOD__ . '## IV ' . $iv);
			$next = null;
			foreach (array_reverse($this->getIntervals()) as $interval) {
				$this->log()->debug(__METHOD__ . '## TRYING IV ' . $interval);
				if ($interval > $iv) {
					$this->log()->debug(__METHOD__ . '## GREATER IV ' . $interval);
					$next = $interval;
				} else {
					return $next;
				}
			}
			return $next;
		}
		$this->log()->debug(__METHOD__ . '## RECURSING ' . $iv);
		return $this->getNextInterval($this->getContainingInterval($iv));
	}

	public function getPreviousInterval($iv) {
		if ($this->isValidInterval($iv)) {
			$this->log()->debug(__METHOD__ . '## IV ' . $iv);
			$next = null;
			foreach ($this->getIntervals() as $interval) {
				$this->log()->debug(__METHOD__ . '## TRYING IV ' . $interval);
				if ($interval < $iv) {
					$this->log()->debug(__METHOD__ . '## GREATER IV ' . $interval);
					$next = $interval;
				} else {
					return $next;
				}
			}
			return $next;
		}
		$this->log()->debug(__METHOD__ . '## RECURSING ' . $iv);
		return $this->getNextInterval($this->getContainingInterval($iv));
	}

	/**
	 * Sucht den ältesten Eintrag in allen vorhanden Intervallen von je
	 * connectlog_ und deliverlog_ .
	 * 
	 * @return Hmt_Log_DeliverEntry NULL [connect] -> ältester Connect-Eintrag,
	 *         [deliver] -> ältester Deliver-Eintrag.
	 */
	public function getOldestEntries() {
		require_once 'Hmt/Log/ConnectEntry.php';
		require_once 'Hmt/Log/DeliverEntry.php';
		$ivs = $this->getIntervals();
		if (!empty($ivs)) {
			$db = $this->getDb();
			$dbname = $db->quoteIdentifier(Hmt_Db_Consts::TABLE_CONNECT . $ivs[0]);
			$sql = "select * from $dbname where " . $db->quoteIdentifier(Hmt_Db_Consts::F_C_TIME) . " = (select min(" . $db->quoteIdentifier(Hmt_Db_Consts::F_C_TIME) . ") from $dbname) order by " . $db->quoteIdentifier(Hmt_Db_Consts::F_C_ID) . " desc limit 1";
			$stm = $db->query($sql);
			$res['connect'] = $stm->fetch(PDO::FETCH_ASSOC);
			if (!empty($res['connect'])) {
				$res['connect']['pt'] = $ivs[0];
				$res['connect'] = new Hmt_Log_ConnectEntry($res['connect']);
			}
			
			$dbname = $db->quoteIdentifier(Hmt_Db_Consts::TABLE_DELIVER . $ivs[0]);
			$sql = "select * from $dbname where " . $db->quoteIdentifier(Hmt_Db_Consts::F_D_TIME) . " = (select min(" . $db->quoteIdentifier(Hmt_Db_Consts::F_D_TIME) . ") from $dbname) order by " . $db->quoteIdentifier(Hmt_Db_Consts::F_D_ID) . " desc limit 1";
			$stm = $db->query($sql);
			$res['deliver'] = $stm->fetch(PDO::FETCH_ASSOC);
			if (!empty($res['deliver'])) {
				$res['deliver']['pt'] = $ivs[0];
				$res['deliver'] = new Hmt_Log_DeliverEntry($res['deliver']);
			}
			return $res;
		} else {
			return null;
		}
	}

	/**
	 * Sucht den jüngsten Eintrag in allen vorhanden Intervallen von je
	 * connectlog_ und deliverlog_ .
	 * 
	 * @return Hmt_Log_DeliverEntry NULL [connect] -> jüngster Connect-Eintrag,
	 *         [deliver] -> jüngster Deliver-Eintrag.
	 */
	public function getNewestEntries() {
		require_once 'Hmt/Log/ConnectEntry.php';
		require_once 'Hmt/Log/DeliverEntry.php';
		$ivs = $this->getIntervals();
		if (!empty($ivs)) {
			$db = $this->getDb();
			$dbname = $db->quoteIdentifier(Hmt_Db_Consts::TABLE_CONNECT . $ivs[count($ivs) - 1]);
			$sql = "select * from $dbname where " . $db->quoteIdentifier(Hmt_Db_Consts::F_C_TIME) . " = (select max(" . $db->quoteIdentifier(Hmt_Db_Consts::F_C_TIME) . ") from $dbname) order by " . $db->quoteIdentifier(Hmt_Db_Consts::F_C_ID) . " desc limit 1";
			$stm = $db->query($sql);
			$res['connect'] = $stm->fetch(PDO::FETCH_ASSOC);
			if (!empty($res['connect'])) {
				$res['connect']['pt'] = $ivs[0];
				$res['connect'] = new Hmt_Log_ConnectEntry($res['connect']);
			}
			
			$dbname = $db->quoteIdentifier(Hmt_Db_Consts::TABLE_DELIVER . $ivs[count($ivs) - 1]);
			$sql = "select * from $dbname where " . $db->quoteIdentifier(Hmt_Db_Consts::F_D_TIME) . " = (select max(" . $db->quoteIdentifier(Hmt_Db_Consts::F_D_TIME) . ") from $dbname) order by " . $db->quoteIdentifier(Hmt_Db_Consts::F_D_ID) . " desc limit 1";
			$stm = $db->query($sql);
			$res['deliver'] = $stm->fetch(PDO::FETCH_ASSOC);
			if (!empty($res['deliver'])) {
				$res['deliver']['pt'] = $ivs[0];
				$res['deliver'] = new Hmt_Log_DeliverEntry($res['deliver']);
			}
			return $res;
		} else {
			return null;
		}
	}

	/**
	 *
	 * @todo comment
	 * @return Hmt_Db_Statistics
	 */
	public function getStatistics() {
		$db = $this->getDb();
		if ($db instanceof Zend_Db_Adapter_Pdo_Pgsql) {
			require_once 'Hmt/Db/Statistics/Pgsql.php';
			return new Hmt_Db_Statistics_Pgsql($db);
		} elseif ($db instanceof Zend_Db_Adapter_Pdo_Mysql) {
			require_once 'Hmt/Db/Statistics/Mysql.php';
			return new Hmt_Db_Statistics_Mysql($db);
		}
		return null;
	}

	public function getServerTime() {
		$db = $this->getDb();
		$stm = $db->query("select now() as server_time;");
		return $this->toTimestamp($stm->fetchColumn());
	}

	/**
	 * Wandelt einen Timestamp-String, wie ihn die DBs im Ergebnis zurückliefern
	 * in einen nummerischen Timestamp um.
	 * 
	 * @param $dbTime string
	 * @return number 13stelliger Timestamp (als millisekunden angabe)
	 */
	public function toTimestamp($dbTimeStr) {
		$dbTimeStr = explode('.', $dbTimeStr);
		if (count($dbTimeStr) > 1) {
			$dbTimeStr[1] = str_pad($dbTimeStr[1], 3, "0", STR_PAD_LEFT);
			$dbTimeStr[1] = substr($dbTimeStr[1], 0, 3);
			return (strtotime($dbTimeStr[0]) * 1000) + intval($dbTimeStr[1]); // date(Hpls_Consts0::DATE_FORMAT_JS,
			// strtotime($timesring));
		}
		return strtotime($dbTimeStr[0]) * 1000; // date(Hpls_Consts0::DATE_FORMAT_JS,
		// strtotime($timesring));
	}

	public function refreshConfig() {
		$this->_db = null;
		$this->_intervals = null;
		$this->_type = null;
	}

	public function hasData() {
		return count($this->getConnectTables()) > 0;
	}

	/**
	 *
	 * @todo comment
	 * @return array <ul><li><b>[connection]</b></li>
	 *         <li><b>[structure]</b></li>
	 *         <li><b>[data]</b></li>
	 *         </ul>
	 */
	public function checkReadyness() {
		$ret = array(
				'connection' => array(
						'result' => 'check_passed' ),
				'structure' => array(
						'result' => 'check_passed' ),
				'data' => array(
						'result' => 'check_passed' ) );
		try {
			$this->getDb()->beginTransaction();
			$this->getDb()->commit();
		} catch (Exception $ex) {
			$ret['connection'] = array(
					'result' => 'failed',
					'message' => array(
							$ex->getMessage() ) );
		}
		
		return $ret;
	}

	/**
	 *
	 * @todo comment
	 * @return multitype:
	 */
	protected function getTables() {
		$listTables = $this->getDb()->listTables();
		return empty($listTables) ? array() : $listTables;
	}

	public function toDbTimestamp($time) {
		// 2011-04-18T10:58:17.44611500+0200
		if (preg_match('/^\d{4}-\d{1,2}-\d{1,2}(T| )\d{1,2}:\d{1,2}:\d{1,2}(\.\d{1,8})?([+-]\d{0,4})?$/', $time) > 0) {
			return $time;
		}
		
		$split = explode(' ', $time);
		if (count($split) > 1) {
			// microtime(): 0.29098800 1302794182
			$s2 = explode('.', $split[0]);
			$milli = '.' . $s2[1];
			$ret = sprintf(date($this->getDbDateFormat(), $split[1]), '');
			
			// @todo next
		} else {
			// @todo implementieren
			$split = explode('.', $time);
			if (count($split) > 1) {
				// mit millisekunden gegeben
				$ret = sprintf(date($this->getDbDateFormat(), $split[0]), '.' . $split[1]);
			} else {
				$ret = sprintf(date($this->getDbDateFormat(), $time), '');
			}
		}
		$this->log()->debug(__METHOD__ . Hpls_Debug::print_r($time, ' from') . Hpls_Debug::print_r($ret, 'result'));
		return $ret;
	}

	protected function isJoinQuery(Hpls_Var_Clause $clause) {
		return $this->isConnectQuery($clause) && $this->isDeliverQuery($clause);
	}

	protected function isConnectQuery(Hpls_Var_Clause $clause) {
		$keys = $clause->getAllKeys();
		return count(array_intersect(array_keys(self::$CONNECT_FIELD_MAP), $keys)) > 0;
	}

	protected function isDeliverQuery(Hpls_Var_Clause $clause) {
		$keys = $clause->getAllKeys();
		return count(array_intersect(array_keys(self::$DELIVER_FIELD_MAP), $keys)) > 0;
	
	}

}