<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 03.08.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
require_once 'Hmt/Reason/Db.php';

/**
 *
 * @todo comment
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hmt_Reason_Cachedb extends Hmt_Reason_Db {
	const CONFIG_KEY_CACHEDB = Hpls_Mailtrace_Consts::CONFIG_KEY_CACHEDB;
	const COL_TABLEFROM = 'fromtable';
	
	const SQL_CREATE_ROLES = '
	DROP TABLE IF EXISTS roles;
	CREATE TABLE roles (
    id integer NOT NULL,
	  id_t character varying(1) NOT NULL,
    role_name character varying(50) NOT NULL,
    time_created datetime NOT NULL,
    time_updated datetime NOT NULL,
    inactive_state smallint NOT NULL DEFAULT 0,
    fromtable character varying(50)
);
	';
	const SQL_CREATE_RULES = '
	DROP TABLE IF EXISTS rule_ids;
	CREATE TABLE rule_ids (
    id integer NOT NULL,
	  id_t character varying(1) NOT NULL,
    rule_name character varying(50) NOT NULL,
    description text,
    time_created datetime NOT NULL,
    time_updated datetime NOT NULL,
    inactive_state smallint NOT NULL DEFAULT 0,
    fromtable character varying(50)
);';
	const SQL_CREATE_REGEX = 'DROP TABLE IF EXISTS regexrule;
	CREATE TABLE regexrule (
    id integer NOT NULL,
	  id_t character varying(1) NOT NULL,
    regex character varying(1024) NOT NULL,
    rule_id integer,
  	rule_id_t character varying(1) NOT NULL,
    deactivated boolean,
    description text,
    param_map character varying(512),
    priority integer DEFAULT 0 NOT NULL,
    time_created datetime NOT NULL,
    time_updated datetime NOT NULL,
    inactive_state smallint NOT NULL DEFAULT 0,
    fromtable character varying(50)
);';
	const SQL_CREATE_TEXTS = 'DROP TABLE IF EXISTS reasons_[[LANG]];
	CREATE TABLE reasons_[[LANG]] (
    id integer NOT NULL,
	  id_t character varying(1) NOT NULL,
    role_id integer NOT NULL,
  	role_id_t character varying(1) NOT NULL,
    rule_id integer NOT NULL,
  	rule_id_t character varying(1) NOT NULL,
    short_desc text NOT NULL,
    long_desc text,
    tag_desc character varying(50),
    time_created datetime NOT NULL,
    time_updated datetime NOT NULL,
    inactive_state smallint NOT NULL DEFAULT 0,
    fromtable character varying(50)
);';
	const SQL_CREATE_CONFIG = '
	DROP TABLE IF EXISTS config;
	CREATE TABLE "config" (
    "key" character varying(1024) NOT NULL,
    "value" text,
    "time_created" datetime NOT NULL,
    "time_updated" datetime NOT NULL,
    inactive_state smallint NOT NULL DEFAULT 0,
    fromtable character varying(50)
);';
	protected static $_instance = null;

	/**
	 *
	 * @return Hmt_Reason_Cachedb
	 */
	public static function i() {
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function __construct(array $config, array $fetchFactory, $name = null) {
		if (!isset($config['tocache'])) {
			$config['tocache'] = array(
					Hpls_Mailtrace_Consts::CONFIG_KEY_VENDORDB );
		} elseif (!is_array($config['tocache'])) {
			$config['tocache'] = explode(',', $config['tocache']);
		}
		parent::__construct($config, $fetchFactory, $name);
	}
	
	/**
	 *
	 * @todo comment
	 * @var Zend_Db_Adapter_Abstract
	 */
	protected $_db = null;
	protected $_created = false;

	public function deleteDb() {
		if (file_exists($this->_options['dbname'])) {
			@unlink($this->_options['dbname']);
		}
	}

	public function createDb() {
		$db = $this->_db;
		try {
			$db->beginTransaction();
			$db->exec(self::SQL_CREATE_CONFIG);
			$db->exec(self::SQL_CREATE_ROLES);
			$db->exec(self::SQL_CREATE_RULES);
			$db->exec(self::SQL_CREATE_REGEX);
			foreach ($this->getLangsToCache() as $lang) {
				$db->exec(str_ireplace('[[LANG]]', $lang, self::SQL_CREATE_TEXTS));
			}
			
			$db->commit();
		} catch (Exception $ex) {
			$this->log()->alert($ex);
			$db->rollBack();
			return false;
		}
		return true;
	}

	public function updateDb() {
		$db = $this->_db;
	}
	
	/* (non-PHPdoc)
	 * @see Hmt_Reason_Db::getDb()
	 */
	public function getDb() {
		$db = parent::getDb();
		
		//haben wir schon einmal überprüft ob die Tabellen existieren?
		if (FALSE === $this->_created) {
			//falls der cache noch nicht erstellt wurde, erstellen wir ihn
			if (FALSE === in_array(self::TABLE_RULES, $db->listTables())) {
				$this->createDb();
				$this->cacheDbs($this->getOriginalDbs());
			}
			$this->_created = true;
		}
		return $db;
	}

	public function createIndexes() {
	
	}

	public function connect() {
	
	}

	public function close() {
	
	}

	public function createStructure() {
		//bleibt leer, weil die DB immer von selbst erstellt wird mit createDB()
	}

	protected function getLangsToCache() {
		$ret = array();
		$dbs = $this->getOriginalDbs();
		foreach ($dbs as $db) {
			$ret = array_merge($ret, $db->getAvaiableLanguages());
		}
		return $ret;
	}

	protected function cacheDbs($dbs) {
		foreach ($dbs as $db) {
			
			//auslesen,vergleichen und in cache schreiben
			$this->copyDb($db->getDb(), $this->_db, self::TABLE_CONFIG, $db->getName());
			$this->copyDb($db->getDb(), $this->_db, self::TABLE_ROLES, $db->getName());
			$this->copyDb($db->getDb(), $this->_db, self::TABLE_RULES, $db->getName());
			$this->copyDb($db->getDb(), $this->_db, self::TABLE_REGEX, $db->getName());
			foreach ($db->getAvaiableLanguages() as $lang) {
				$this->copyDb($db->getDb(), $this->_db, self::TABLE_REASON_PREFIX . $lang, $db->getName());
			}
		}
	}

	protected function copyDb(Zend_Db_Adapter_Abstract $dbFrom, Zend_Db_Adapter_Abstract $dbTo, $table, $dbName) {
		$ctrlSel = $dbTo->select();
		$select = $dbFrom->select();
		$select->from($table);
		$stm = $dbFrom->query($select);
		$res = $stm->fetchAll();
		isDebug('hmt_reason_cachedb') && $this->log()->debug(Hpls_Debug::print_r($res, __METHOD__));
		foreach ($res as $insert) {
			isDebug('hmt_reason_cachedb') && $this->log()->debug(__METHOD__ . " $dbName.$table " . Hpls_Debug::print_r($insert));
			if (!isset($insert['id']) && !isset($insert['key'])) {
				$this->log()->err("'id' and/or 'id_t' and 'key' is empty in result for table '$dbName'.'$table'." . Hpls_Debug::print_r($insert, ' RESULT ROW'));
			}
			
			//überprüfen, ob's diese werte schon gibt
			//			$t = Hpls_Utils_Db::toWhere($dbTo, array_intersect_key($insert, array(
			//					'time_created' => 1,
			//					'time_updated' => 1 )), 'OR', '<');
			if (isset($insert['id'])) {
				$t2 = Hpls_Utils_Db::toWhere($dbTo, array(
						'id' => $insert['id'],
						'id_t' => $insert['id_t'] ), 'AND');
			
			} else { //für die config-tabelle, die 'key' statt 'id'/'id_t'
				$t2 = Hpls_Utils_Db::toWhere($dbTo, array(
						'key' => $insert['key'] ), 'AND');
			}
			$ctrlSel->reset();
			
			$ctrlSel->from(array(
					't' => $table ), array(
					'time_created',
					'time_updated',
					'gt_created' => '(t.time_created < :time_created1)', //
					'gt_updated' => '(t.time_updated < :time_updated1)' ));
			$t2['values'][':time_created1'] = $insert['time_created'];
			$t2['values'][':time_updated1'] = $insert['time_updated'];
			
			$ctrlSel->where($t2['sql']); //'('.$t['sql'].') AND ('.$t2['sql'].')');
			isDebug('hmt_reason_cachedb') && $this->log()->info(__METHOD__ . "  " . $ctrlSel->assemble() . Hpls_Debug::print_r($t2));
			$stm = $ctrlSel->query(PDO::FETCH_ASSOC, $t2['values']);
			$r = $stm->fetchAll();
			//gibt's schon werte?
			$insert[self::COL_TABLEFROM] = $dbName;
			if (count($r) > 0) {
				//JA -> überprüfen, ob neuere oder ältere werte vorliegen
				isDebug('hmt_reason_cachedb') && $this->log()->info(__METHOD__ . "  FOUND DOUBLE " . Hpls_Debug::print_r($r));
				if ($r[0]['gt_created'] != 0 || $r[0]['gt_updated'] != 0) {
					//NEUERE liegen vor -> UPDATE
					if (isset($insert['id'])) {
						isDebug('hmt_reason_cachedb') && $this->log()->info(__METHOD__ . "  UPDATE id: " . $t2['sql'] . " with DOUBLE " . Hpls_Debug::print_r($r));
						$dbTo->update($table, $insert, array(
								'id = ?' => $insert['id'] ));
					} else {
						//für config
						isDebug('hmt_reason_cachedb') && $this->log()->info(__METHOD__ . "  UPDATE key: " . $t2['sql'] . " with DOUBLE " . Hpls_Debug::print_r($r));
						$dbTo->update($table, $insert, array(
								'key = ?' => $insert['key'] ));
					}
				}
			} else {
				//NEIN -> neuen Datesatz einfügen
				$this->log()->info(__METHOD__ . "  INSERT INTO $table " . Hpls_Debug::print_r($insert));
				$dbTo->insert($table, $insert);
			}
		}
		//@todo in config-table max(updated) und max(created) schreiben
		$select = $dbFrom->select();
		$tables = array(
				't' => $table );
		$values = array(
				'max_updated' => 'max(t.time_updated)',
				'max_created' => 'max(t.time_created)' );
		$select->from($tables, $values);
		$stm = $dbFrom->query($select);
		$res = $stm->fetch(PDO::FETCH_ASSOC);
		if ($res === FALSE || count($res) === 0) {
			return null;
		}
		isDebug('hmt_reason_cachedb') && $this->log()->debug(Hpls_Debug::print_r($res, __METHOD__ . ' ' . $select->assemble()));
		try {
			$dbTo->insert(self::TABLE_CONFIG, array(
					'key' => "hmtr.cachedb.lastcopy.$dbName.$table.updated",
					'value' => $res['max_updated'],
					'time_created' => date(DATE_ISO8601, time()),
					'time_updated' => date(DATE_ISO8601, time()) ));
		} catch (Exception $ex) {
			isDebug('hmt_reason_cachedb') && $this->log()->err($ex);
			$dbTo->update(self::TABLE_CONFIG, array(
					'key' => "hmtr.cachedb.lastcopy.$dbName.$table.updated",
					'value' => $res['max_updated'],
					'time_updated' => date(DATE_ISO8601, time()) ), array(
					'"key" = ?' => "hmtr.cachedb.lastcopy.$dbName.$table.updated" ));
		}
		
		try {
			$dbTo->insert(self::TABLE_CONFIG, array(
					'key' => "hmtr.cachedb.lastcopy.$dbName.$table.created",
					'value' => $res['max_created'],
					'time_created' => date(DATE_ISO8601, time()),
					'time_updated' => date(DATE_ISO8601, time()) ));
		} catch (Exception $ex) {
			isDebug('hmt_reason_cachedb') && $this->log()->err($ex);
			$dbTo->update(self::TABLE_CONFIG, array(
					'key' => "hmtr.cachedb.lastcopy.$dbName.$table.created",
					'value' => $res['max_created'],
					'time_updated' => date(DATE_ISO8601, time()) ), array(
					'"key" = ?' => "hmtr.cachedb.lastcopy.$dbName.$table.created" ));
		}
	}

	protected function getOriginalDbs() {
		$config = Hpls_Settings::i()->getConfig('hmt.reason.storage');
		$dbConfig = $config->toArray();
		if (isset($dbConfig[self::CONFIG_KEY_CACHEDB]))
			unset($dbConfig[self::CONFIG_KEY_CACHEDB]);
		isDebug('hmt_reason_cachedb') && $this->log()->debug(Hpls_Debug::print_r($dbConfig, __METHOD__));
		
		$ret = array();
		foreach ($dbConfig as $key => $config) {
			$ret[$key] = new Hmt_Reason_Db($config, $this->_fetchFactory, $key);
		}
		isDebug('hmt_reason_cachedb') && $this->log()->debug(Hpls_Debug::print_r($ret, __METHOD__));
		return $ret;
	}
}