<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 03.08.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-10-23 16:54:02 +0200 (Di, 23. Okt 2012) $
 * last changed by: $LastChangedBy: lazee $
 * revision       : $LastChangedRevision: 9792 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.libs/Hmt/Reason/Db.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
class Hmt_Reason_Db extends Hpls_Utils_Logusing {
	
	const PREFIX_TRIGGER_REGEX = '/^create\.trigger\./i';
	const PREFIX_DATA_BEFORE_REGEX = '/^data\.initial\.before\.trigger\./i';
	const PREFIX_DATA_AFTER_REGEX = '/^data\.initial\.after\.trigger\./i';
	const PREFIX_TABLE_REGEX = '/^create\.table\./i';
	const PREFIX_INDEX_REGEX = '/^create\.index\./i';
	const PREFIX_GRANT_REGEX = '/^grant\./i';
	
	const SQL_FILE_TPL_MINIMAL = 'create.minimal.reasonsdb.sql';
	const SQL_FILE_TPL_MINIMAL_REASONLANG = 'create.minimal.reasonsdb.table.reasons.sql';
	const SQL_VAR_LOCALE = Hpls_Consts0::SQL_VAR_LOCALE;
	const SQL_VAR_DBKEY = Hpls_Consts0::SQL_VAR_DBKEY;
	const SQL_VAR_TABLEPREFIX = Hpls_Consts0::SQL_VAR_TABLEPREFIX;
	const SQL_VAR_OWNER = Hpls_Consts0::SQL_VAR_OWNER;
	
	const TABLE_REGEX = 'regexrule';
	const TABLE_RULES = 'rule_ids';
	const TABLE_REASON_PREFIX = 'reasons_';
	const TABLE_ROLES = 'roles';
	const TABLE_CONFIG = 'config';
	
	const FIELD_STATE = 'inactive_state';
	
	const INACTIVESTATE_INACTIVE = 1;
	const INACTIVESTATE_DELETED = -1;
	const INACTIVESTATE_ACTIVE = 0;
	
	const CHECK_PASSED = 'check_passed';
	
	/**
	 *
	 * @todo comment
	 * @var Zend_Db_Adapter_Abstract
	 */
	protected $_db = null;
	/**
	 *
	 * @todo comment
	 * @var array:string:int
	 */
	protected $_roleMap = null;
	protected $_ruleIDmap = null;
	protected $_options = null;
	protected $_name = null;
	protected $_fetchFactory = array();
	protected $_result_readyness = null;

	public function __construct(array $config, array $fetchFactory, $name = null) {
		$this->_db = Hpls_Utils_Db::mapDriverType2ZendDb($config);
		$this->_options = $config;
		$this->_name = $name;
		$this->_fetchFactory = $fetchFactory;
	}

	/**
	 *
	 * @return Zend_Db_Adapter_Abstract
	 */
	public function getDb() {
		return $this->_db;
	}

	/**
	 * Enter description here .
	 * ..
	 * 
	 * @return array:string Die Sprachen als Locale-Auflistung im array.
	 */
	public function getAvaiableLanguages() {
		$tables = $this->getDb()->listTables();
		if (empty($tables))
			return array();
		isDebug('hmt_reason') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($tables, 'TABLES'));
		$tables = Hpls_Utils::array_values_startWith($tables, self::TABLE_REASON_PREFIX);
		isDebug('hmt_reason') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($tables, 'TABLES filtered'));
		$ret = array();
		foreach ($tables as $tablename) {
			$ret[] = strtolower(substr($tablename, strlen(self::TABLE_REASON_PREFIX)));
		}
		return $ret;
	}

	/**
	 * Gibt den Version der Datensätze wieder, d.h. das jüngste Änderungsdatum eines Datensatzes.
	 * ACHTUNG: es ist nicht die Version des Datenbankschemas.
	 * @todo comment
	 * @return Ambigous <string, unknown>
	 */
	public function getVersion() {
		$dates = $this->getDates();
		$ret = '0';
		foreach ($dates as $table => $date) {
			if ($table == self::TABLE_CONFIG)
				// CONFIG-Table zählt nicht in die version hinein
				continue;
			if ($date > $ret) {
				$ret = $date;
			}
		}
		return $ret;
	}

	/**
	 *
	 * @todo comment
	 * @return Ambigous <string, unknown>
	 */
	public function getConfigValue($key) {
		$sql = $this->getDb()->select();
		$sql->from(self::TABLE_CONFIG);
		$sql->where($this->getDb()->quoteIdentifier('key') . "=:key");
		$stm = $this->getDb()->query($sql, array(
				'key' => $key ));
		$stm->execute();
		$ret = array();
		$ret = $stm->fetch(PDO::FETCH_ASSOC);
		return $ret['value'];
	}

	/**
	 *
	 * @todo comment
	 * @return array:string
	 */
	public function getDates() {
		
		$tables = array(
				self::TABLE_ROLES,
				self::TABLE_RULES,
				self::TABLE_CONFIG,
				self::TABLE_REGEX );
		
		foreach ($this->getAvaiableLanguages() as $lang) {
			$tables[] = self::TABLE_REASON_PREFIX . $lang;
		}
		$ret = array();
		foreach ($tables as $table) {
			$ret[$table] = $this->getDatesFrom($table);
		}
		
		return $ret;
	}

	protected function getDatesFrom($tablename) {
		$select = $this->getDb()->select();
		$select->from(array(
				'ro' => $tablename ), array(
				"max_update_$tablename" => 'max(ro.time_updated)' ));
		$this->log()->info(__METHOD__ . ' SQL: ' . $select->assemble());
		$stm = $this->getDb()->query($select);
		$res = $stm->fetch(PDO::FETCH_COLUMN);
		if ($res === FALSE || count($res) === 0) {
			return null;
		}
		
		return $res;
	
	}

	public function getRegEx($regexID) {
		$id = explode(Hmt_ReasonAbstract::ID_SEP, $regexID);
		if (count($id) != 2) {
			throw new InvalidArgumentException("'$regexID' not in a format of '<id-seq-nr>-<id_t>'", -1);
		}
		$sql = $this->getDb()->select();
		$sql->from(self::TABLE_REGEX)->where($this->getDb()->quoteIdentifier('id') . ' = :rid')->where($this->getDb()->quoteIdentifier('id_t') . ' = :ridt');
		$this->log()->debug(Hpls_Debug::print_r($sql->assemble(), __METHOD__));
		$idValues = array(
				':rid' => $id[0],
				':ridt' => $id[1] );
		$this->log()->debug(Hpls_Debug::print_r($idValues, __METHOD__ . ' SQL: ' . $sql->assemble()));
		$stm = $this->getDb()->query($sql, $idValues);
		$stm->execute();
		return $this->fetch($stm, __FUNCTION__);
	}

	public function getRule($id) {
		$id = explode(Hmt_ReasonAbstract::ID_SEP, $id);
		if (count($id) != 2) {
			throw new InvalidArgumentException("'$id' not in a format of '<id-seq-nr>-<id_t>'", -1);
		}
		
		$sql = $this->getDb()->select();
		$sql->from(self::TABLE_RULES)->where($this->getDb()->quoteIdentifier('id') . ' = :rid')->where($this->getDb()->quoteIdentifier('id_t') . ' = :ridt');
		$idValues = array(
				':rid' => $id[0],
				':ridt' => $id[1] );
		$this->log()->debug(Hpls_Debug::print_r($idValues, __METHOD__ . ' SQL: ' . $sql->assemble()));
		$stm = $this->getDb()->query($sql, $idValues);
		$stm->execute();
		return $this->fetch($stm, __FUNCTION__);
	}

	/**
	 *
	 * @todo comment
	 * @return array:Hmt_Reason_Rule Hmt_Reason_Edit_Rule
	 */
	public function getRules() {
		$sql = $this->getDb()->select();
		$sql->from(self::TABLE_RULES)->order('id');
		$stm = $this->getDb()->query($sql);
		$stm->execute();
		$ret = array();
		foreach ($this->fetch($stm, __FUNCTION__) as $rule) {
			$ret[$rule->getID()] = $rule;
		}
		
		return $ret;
	}

	public function getRoles() {
		$sql = $this->getDb()->select();
		$sql->from(self::TABLE_ROLES);
		$stm = $this->getDb()->query($sql);
		$stm->execute();
		$ret = array();
		foreach ($this->fetch($stm, __FUNCTION__) as $role) {
			$ret[$role->getID()] = $role;
		}
		return $ret;
	}

	public function getReasonText($ruleID, $lang = null) {
		$id = explode(Hmt_ReasonAbstract::ID_SEP, $ruleID);
		if (count($id) != 2) {
			throw new InvalidArgumentException("'$ruleID' not in a format of '<id-seq-nr>-<id_t>'", -1);
		}
		if (!is_null($lang)) {
			if (!is_array($lang)) {
				$lang = array(
						trim($lang) );
			}
			foreach ($lang as $l) {
				if (!in_array($l, $this->getAvaiableLanguages())) {
					require_once 'Hpls/Mailtrace/Exception/Reason.php';
					throw new Hpls_Mailtrace_Exception_Reason("Language '$l' is not avaiable.");
				}
			}
		
		} else {
			$lang = $this->getAvaiableLanguages();
		}
		$ret = array();
		foreach ($lang as $l) {
			$sql = $this->getDb()->select();
			$sql->from(self::TABLE_REASON_PREFIX . $l)->where($this->getDb()->quoteIdentifier('rule_id') . ' = :rid')->where($this->getDb()->quoteIdentifier('rule_id_t') . ' = :ridt');
			$idValues = array(
					':rid' => $id[0],
					':ridt' => $id[1] );
			isDebug('hmt_reason_sql') && $this->log()->debug(Hpls_Debug::print_r($idValues, __METHOD__ . ' SQL: ' . $sql->assemble()));
			$stm = $this->getDb()->query($sql, $idValues);
			$stm->execute();
			$rols = array();
			foreach ($this->fetch($stm, __FUNCTION__) as $r) {
				$rols[$r->getRoleId()] = $r;
			}
			$ret[$l] = $rols;
		}
		return $ret;
	}

	public function getReasonTextById($lang, $textid) {
		$id = explode(Hmt_ReasonAbstract::ID_SEP, $textid);
		if (count($id) != 2) {
			throw new InvalidArgumentException("'$textid' not in a format of '<id-seq-nr>-<id_t>'", -1);
		}
		if (!in_array($lang, $this->getAvaiableLanguages())) {
			require_once 'Hpls/Mailtrace/Exception/Reason.php';
			throw new Hpls_Mailtrace_Exception_Reason("Language '$lang' is not avaiable.");
		}
		
		$sql = $this->getDb()->select();
		$sql->from(self::TABLE_REASON_PREFIX . $lang)->where($this->getDb()->quoteIdentifier('id') . ' = :rid')->where($this->getDb()->quoteIdentifier('id_t') . ' = :ridt');
		$idValues = array(
				':rid' => $id[0],
				':ridt' => $id[1] );
		isDebug('hmt_reason_sql') && $this->log()->debug(Hpls_Debug::print_r($idValues, __METHOD__ . ' SQL: ' . $sql->assemble()));
		$stm = $this->getDb()->query($sql, $idValues);
		$stm->execute();
		$ret = $this->fetch($stm, __FUNCTION__);
		foreach ($ret as $r) {
			$r->setLanguage($lang);
		}
		return $ret;
	}

	/**
	 *
	 * @todo comment
	 *       <p>Aufgrund der bisweilen enormen Größe der langen Erklärung, wird
	 *       diese jedes Mal aus
	 *       der DB oder vom File-Cache gelesen, statt sie im Speicher
	 *       vorzuhalten. Die lange Erklärung wird ohnehin
	 *       nur auf den extra Erklärungsseiten und damit nur eine (mit Hobs ein
	 *       paar wenige) pro Controller-Durchlauf aufgerufen.
	 * @param $ruleID string
	 * @param $role string
	 * @param $lang unknown_type
	 * @return NULL
	 */
	public function getLongExplanation($ruleID, $role = 'admin', $lang = 'de_de') {
		if ($ruleID instanceof Hmt_Reason_Item) {
			$ruleID = $ruleID->getRuleID();
		}
		$id = explode(Hmt_ReasonAbstract::ID_SEP, $ruleID);
		if (count($id) != 2) {
			throw new InvalidArgumentException("'$ruleID' not in a format of '<id-seq-nr>-<id_t>'", -1);
		}
		// $ruleID = $this->mapRuleID($ruleID);
		
		// EDIT: Das Roleplaying wird erstmal entfernt, da dieses Feature
		// aktuell Probleme macht und im Grunde erst in ferner Zukunft benutzt
		// werden würde.
		
		#$roleID = $this->getRoleID($role);
		
		/*
		$values = array(
				':rule_id' => $id[0],
				':rule_id_t' => $id[1],
				':role_id' => $roleID[0],
				':role_id_t' => $roleID[1] );
		*/
		
		$values = array(
				':rule_id' => $id[0],
				':rule_id_t' => $id[1] );
		
		
		// @todo neuer primary key
		$sql = $this->getDb()->select();
		$sql->from(strtolower(self::TABLE_REASON_PREFIX . $lang), array(
				'long_desc' ))->where('rule_id = :rule_id')->where('rule_id_t = :rule_id_t');
		isDebug('hmt_reason_sql') && $this->log()->debug(__METHOD__ . '[' . get_class($this) . ']' . Hpls_Debug::print_r($values, ' SQL: ' . $sql->assemble() . ' VALUES '));
		$stm = $sql->query(PDO::FETCH_ASSOC, $values);
		$res = $stm->fetchAll();
		isDebug('hmt_reason') && $this->log()->debug(__METHOD__ . '[' . get_class($this) . ']' . Hpls_Debug::print_r($res, ' RESULT'));
		if (count($res) == 0) {
			return null;
		}
		$res = array_change_key_case($res, CASE_LOWER);
		return $res[0]['long_desc'];
	}

	/**
	 *
	 * @todo comment
	 * @param $ruleID string
	 * @return array:Hmt_Reason_RegEx array:Hmt_Reason_RegEx_Edit
	 */
	public function getRegExs($ruleID) {
		$id = explode(Hmt_ReasonAbstract::ID_SEP, $ruleID);
		if (count($id) != 2) {
			throw new InvalidArgumentException("'$ruleID' not in a format of '<id-seq-nr>-<id_t>'", -1);
		}
		$sql = $this->getDb()->select();
		$sql->from(self::TABLE_REGEX)->where($this->getDb()->quoteIdentifier('rule_id') . ' = :rid')->where($this->getDb()->quoteIdentifier('rule_id_t') . ' = :ridt')->order('priority DESC');
		$values = array(
				':rid' => $id[0],
				':ridt' => $id[1] );
		isDebug('hmt_reason', 'sql') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($values, ' SQL: ' . $sql->assemble() . ' VALUES'));
		$stm = $this->getDb()->query($sql, $values);
		$stm->execute();
		return $this->fetch($stm, __FUNCTION__);
	}

	/**
	 *
	 * @todo comment
	 * @return NULL
	 */
	public function getRegEx4use() {
		$regexs = array();
		$sql = $this->getDb()->select();
		$sql->from(self::TABLE_REGEX, array(
				'regex',
				'rule_id',
				'rule_id_t',
				'param_map',
				'priority' ))->where($this->getDb()->quoteIdentifier('inactive_state') . ' = 0')->order('priority DESC');
		isDebug('hmt_reason_sql') && $this->log()->debug(__METHOD__ . ' SQL: ' . $sql->assemble());
		$stm = $this->getDb()->query($sql);
		$stm->execute();
		$rulesActive = $this->getActiveRulesIDs();
		isDebug('hmt_reason_matching') && $this->log()->debug(Hpls_Debug::print_r($rulesActive, __METHOD__));
		
		while ($row = $stm->fetch(PDO::FETCH_ASSOC)) {
			$row = array_change_key_case($row, CASE_LOWER);
			// nur regex aus aktiven regeln
			if (array_key_exists($row['rule_id'] . Hmt_Reason::ID_SEP . $row['rule_id_t'], $rulesActive)) {
				$regexs[$row['regex']] = $row;
			}
		}
		isDebug('hmt_reason_regex_load') && $this->log()->debug(__METHOD__ . ' loaded ' . Hpls_Debug::print_r($regexs, 'RegEx'));
		return $regexs;
	
	}

	public function getExplanations($role, $lang) {
		$role = strtolower($role);
		$lang = strtolower($lang);
		if (FALSE === in_array($lang, $this->getAvaiableLanguages())) {
			require_once 'Hpls/Mailtrace/Exception/Reason.php';
			throw new Hpls_Mailtrace_Exception_Reason("Language '$lang' is currently not avaiable in the reason system.", 2);
		}
		$roleID = $this->getRoleID($role);
		
		// ----------------------------------------------------------
		// OVERRIDE:
		// da es Probleme mit dem Fallback auf Vendor Rules gibt
		// (soll: wenn kein custom, dann nimm vendor)
		// nehmen wir hier mal einfach den "vendor" an.
		
		$roleID = array('1','v');
		// ----------------------------------------------------------
		
		$values = array(
				':role_id' => $roleID[0],
				':role_id_t' => $roleID[1] );
		$sql = $this->getDb()->select();
		$sql->from(self::TABLE_REASON_PREFIX . $lang, array(
				'rule_id',
				'rule_id_t',
				'tag_desc',
				'short_desc' ))->where('role_id = :role_id')->where('role_id_t = :role_id_t');
		isDebug('hmt_reason_sql') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($values, ' SQL: ' . $sql->assemble() . ' VALUES'));
		$stm = $sql->query(PDO::FETCH_ASSOC, $values);
		$res = array();
		while ($row = $stm->fetch()) {
			$row = array_change_key_case($row, CASE_LOWER);
			$res[$row['rule_id'] . Hmt_Reason::ID_SEP . $row['rule_id_t']] = $row;
		}
		isDebug('hmt_reason') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($res, 'RESULT'));
		return $res;
	}

	/**
	 *
	 * @todo comment
	 * @param $role string
	 * @return int NULL
	 */
	public function getRoleID($role) {
		if (is_null($this->_roleMap)) {
			$sql = $this->getDb()->select();
			$sql->from(self::TABLE_ROLES, array(
					'id',
					'id_t',
					'role_name' ));
			isDebug('hmt_reason_sql') && $this->log()->debug(__METHOD__ . ' SQL ' . $sql->assemble());
			
			$stm = $sql->query();
			$this->_roleMap = array();
			while ($row = $stm->fetch(PDO::FETCH_ASSOC)) {
				$row = array_change_key_case($row, CASE_LOWER);
				$this->_roleMap[$row['role_name']] = array(
						$row['id'],
						$row['id_t'] );
			}
		
		}
		return isset($this->_roleMap[$role]) ? $this->_roleMap[$role] : null;
	}

	/**
	 *
	 * @todo comment
	 * @param $ruleID unknown_type
	 * @return Ambiguous NULL
	 */
	public function mapRuleID($ruleID) {
		if (is_null($this->_ruleIDmap)) {
			$sql = $this->getDb()->select();
			$sql->from(self::TABLE_RULES, array(
					'id',
					'rule_name' ));
			isDebug('hmt_reason_sql') && $this->log()->debug(__METHOD__ . ' SQL ' . $sql->assemble());
			$stm = $sql->query();
			$this->_ruleIDmap = array();
			while ($row = $stm->fetch(PDO::FETCH_ASSOC)) {
				$row = array_change_key_case($row, CASE_LOWER);
				$this->_ruleIDmap[$row['rule_name']] = $row['id'];
			}
		}
		if (is_int($ruleID))
			return $ruleID;
		
		return isset($this->_ruleIDmap[$ruleID]) ? $this->_ruleIDmap[$ruleID] : null;
	
	}

	/**
	 *
	 * @todo comment
	 *       ACHTUNG: Cached die Ergebnisse, deshalb muss zur erneuerten realen
	 *       Überprüfung ein neues Object erzeugt werden!
	 * @return array <ul><li><b>[connection]</b></li>
	 *         <li><b>[structure]</b></li>
	 *         <li><b>[data]</b></li>
	 *         </ul>
	 */
	public function checkReadyness() {
		if (is_null($this->_result_readyness)) {
			$this->_result_readyness = array(
					'connection' => $this->checkDbConnection(),
					'structure' => $this->checkDbStructure(),
					'data' => $this->checkDbData() );
		}
		return $this->_result_readyness;
	}

	/**
	 *
	 * @todo comment
	 * @return boolean
	 */
	public function isReadyToUse() {
		$ret = $this->checkReadyness();
		#$this->log()->debug('####### !!! ####### READYNESS result:');
		#$this->log()->debug(print_r($ret, TRUE));		
		return $ret['connection']['result'] == self::CHECK_PASSED && $ret['structure']['result'] == self::CHECK_PASSED && $ret['data']['result'] == self::CHECK_PASSED;
	}

	protected function getActiveRulesIDs() {
		$sql = $this->getDb()->select();
		$sql->from(self::TABLE_RULES, array(
				'id',
				'id_t' ))->where('inactive_state = ' . Hmt_Reason::INACTIVESTATE_ACTIVE)->order('id');
		isDebug('hmt_reason_sql') && $this->log()->debug(__METHOD__ . ' SQL: ' . $sql->assemble());
		$stm = $this->getDb()->query($sql);
		$stm->execute();
		$ret = array();
		while ($rule = $stm->fetch(PDO::FETCH_ASSOC)) {
			$ret[$rule['id'] . Hmt_Reason::ID_SEP . $rule['id_t']] = array(
					'id' => $rule['id'],
					'id_t' => $rule['id_t'] );
		}
		return $ret;
	}

	protected function fetch($stm, $inFunction) {
		return call_user_func($this->_fetchFactory, $stm, $inFunction);
	}

	/**
	 *
	 * @return the $_name
	 */
	protected function getName() {
		return $this->_name;
	}

	/**
	 *
	 * @todo comment
	 * @return multitype:string NULL multitype:NULL |multitype:string
	 */
	protected function checkDbConnection() {
		try {
			$this->getDb()->beginTransaction();
			$this->getDb()->commit();
		} catch (Exception $ex) {
			return array(
					'result' => 'failed',
					'message' => array(
							$ex->getMessage() ),
					'code' => $ex->getCode() );
		}
		return array(
				'result' => self::CHECK_PASSED );
	}

	/**
	 *
	 * @todo comment
	 * @return multitype:string NULL multitype:NULL |multitype:string
	 */
	protected function checkDbStructure() {
		try {
			$ret = array(
					'result' => self::CHECK_PASSED,
					'messages' => array() );
			$tables = $this->getDb()->listTables();
			if (!in_array(self::TABLE_CONFIG, $tables)) {
				$ret['message'][] = self::TABLE_CONFIG . ' missing';
			}
			if (!in_array(self::TABLE_REGEX, $tables)) {
				$ret['message'][] = self::TABLE_REGEX . ' missing';
			}
			if (!in_array(self::TABLE_ROLES, $tables)) {
				$ret['message'][] = self::TABLE_ROLES . ' missing';
			}
			if (!in_array(self::TABLE_RULES, $tables)) {
				$ret['message'][] = self::TABLE_RULES . ' missing';
			}
			
			// reason languages
		} catch (Exception $ex) {
			return array(
					'result' => 'failed',
					'message' => array(
							$ex->getMessage() ),
					'code' => $ex->getCode() );
		}
		return array(
				'result' => self::CHECK_PASSED );
	}

	/**
	 *
	 * @todo comment
	 * @return string
	 */
	protected function checkDbData() {
		return array(
				'result' => self::CHECK_PASSED );
	}

	protected function getDbType() {
		$db = $this->getDb();
		
		if ($db instanceof Zend_Db_Adapter_Pdo_Mysql) {
			return Hpls_Consts0::DB_TYPE_MYSQL;
		} elseif ($db instanceof Zend_Db_Adapter_Pdo_Pgsql) {
			return Hpls_Consts0::DB_TYPE_PGSQL;
		} elseif ($db instanceof Zend_Db_Adapter_Pdo_Sqlite) {
			return Hpls_Consts0::DB_TYPE_SQLITE;
		}
		return null;
	}

	protected function getDbKey() {
		$this->log()->debug(Hpls_Debug::print_r($this->_name,__METHOD__));
		switch (strtolower($this->_name)) {
			case Hpls_Mailtrace_Consts::CONFIG_KEY_CUSTORMERDB :
				return 'c';
			case Hpls_Mailtrace_Consts::CONFIG_KEY_VENDORDB :
				return 'v';
			case Hpls_Mailtrace_Consts::CONFIG_KEY_CACHEDB :
			default :
				return 'h';
		}
	}

	public function createStructure() {
		
		isDebug('hmt_reason_create') && $this->log()->debug(__METHOD__ . ' START');
		$this->createDatabase();
		$this->createTables();
		$this->grantRights();
		$this->insertFirstDataBeforeTriggers();
		$this->createTriggers();
		$this->insertFirstDataAfterTriggers();
		$this->createIndexes();

		$this->createLanguage(Hpls_Application0::i()->getDefaultLocale());
		isDebug('hmt_reason_create') && $this->log()->debug(__METHOD__ . ' END');
	}

	/**
	 * @todo comment
	 * @param string $lang
	 * @throws InvalidArgumentException
	 * @throws Hpls_Mailtrace_Exception Wenn irgend
	 */
	public function createLanguage($lang) {
		if (empty($lang)) {
			throw new InvalidArgumentException('$lang must not be empty.', Hpls_Err::ERR_ARGUMENT_EMPTY);
		}
		$lang = strtolower($lang);
		
		$replaces = array(
				self::SQL_VAR_TABLEPREFIX => $this->_options['tableprefix'],
				self::SQL_VAR_DBKEY => $this->getDbKey(),
				self::SQL_VAR_LOCALE => $lang,
				'[[OWNER]]' => (string) $this->_options['username'] );
		$this->processSqlFiles(str_replace('/^', '/^lang\.', self::PREFIX_TABLE_REGEX), __FUNCTION__, Hpls_Mailtrace_Err::ERR_REASON_CREATE, $replaces);
		$this->processSqlFiles(str_replace('/^', '/^lang\.', self::PREFIX_GRANT_REGEX), __FUNCTION__, Hpls_Mailtrace_Err::ERR_REASON_CREATE_GRANT, $replaces);
		$this->processSqlFiles(str_replace('/^', '/^lang\.', self::PREFIX_DATA_BEFORE_REGEX), __FUNCTION__, Hpls_Mailtrace_Err::ERR_REASON_CREATE_INITIAL_DATA, $replaces);
		$this->processSqlFiles(str_replace('/^', '/^lang\.', self::PREFIX_TRIGGER_REGEX), __FUNCTION__, Hpls_Mailtrace_Err::ERR_REASON_CREATE_TRIGGER, $replaces);
		$this->processSqlFiles(str_replace('/^', '/^lang\.', self::PREFIX_DATA_AFTER_REGEX), __FUNCTION__, Hpls_Mailtrace_Err::ERR_REASON_CREATE_INITIAL_DATA, $replaces);
		$this->processSqlFiles(str_replace('/^', '/^lang\.', self::PREFIX_INDEX_REGEX), __FUNCTION__, Hpls_Mailtrace_Err::ERR_REASON_CREATE_INDEX, $replaces);
	
	}
	
	protected function createDatabase() {
		try {
		if (FALSE === Hpls_Utils_Db::dbExists($this->getDbAdapterConfig())) {
			Hpls_Utils_Db::createDB($this->getDbAdapterConfig());
		}
		} catch (Exception $ex) {
			$this->log()->err($ex);
			require_once 'Hpls/Mailtrace/Exception.php';
			throw new Hpls_Mailtrace_Exception("Error in creating reason-db while executing step createDatabase.", $errCode, $ex);
		}
	}

	protected function createTables() {
		// TODO Auto-generated method stub
		isDebug('hmt_reason_create') && $this->log()->debug(__METHOD__ . ' START');
		$this->processSqlFiles(self::PREFIX_TABLE_REGEX, __FUNCTION__, Hpls_Mailtrace_Err::ERR_REASON_CREATE, array(
				self::SQL_VAR_TABLEPREFIX => $this->_options['tableprefix'],
				self::SQL_VAR_DBKEY => $this->getDbKey() ));
	}
	
	/*
	 * (non-PHPdoc) @see Hmt_Log_ManageAbstract::createTriggers()
	*/
	protected function createTriggers() {
		// TODO Auto-generated method stub
		isDebug('hmt_reason_create') && $this->log()->debug(__METHOD__ . ' START');
		$this->processSqlFiles(self::PREFIX_TRIGGER_REGEX, __FUNCTION__, Hpls_Mailtrace_Err::ERR_REASON_CREATE_TRIGGER, array(
				self::SQL_VAR_TABLEPREFIX => $this->_options['tableprefix'],
				self::SQL_VAR_DBKEY => $this->getDbKey() ));
	}

	protected function grantRights() {
		isDebug('hmt_reason_create') && $this->log()->debug(__METHOD__ . ' START');
		$this->processSqlFiles(self::PREFIX_GRANT_REGEX, __FUNCTION__, Hpls_Mailtrace_Err::ERR_REASON_CREATE_GRANT, array(
				self::SQL_VAR_TABLEPREFIX => $this->_options['tableprefix'],
				self::SQL_VAR_DBKEY => $this->getDbKey(),
				'[[OWNER]]' => (string) $this->_options['username'] ));
	}

	protected function insertFirstDataBeforeTriggers() {
		isDebug('hmt_reason_create') && $this->log()->debug(__METHOD__ . ' START');
		$this->processSqlFiles(self::PREFIX_DATA_BEFORE_REGEX, __FUNCTION__, Hpls_Mailtrace_Err::ERR_REASON_CREATE_INITIAL_DATA, array(
				self::SQL_VAR_TABLEPREFIX => $this->_options['tableprefix'],
				self::SQL_VAR_DBKEY => $this->getDbKey() ));
	}

	protected function insertFirstDataAfterTriggers() {
		isDebug('hmt_reason_create') && $this->log()->debug(__METHOD__ . ' START');
		$this->processSqlFiles(self::PREFIX_DATA_AFTER_REGEX, __FUNCTION__, Hpls_Mailtrace_Err::ERR_REASON_CREATE_INITIAL_DATA, array(
				self::SQL_VAR_TABLEPREFIX => $this->_options['tableprefix'],
				self::SQL_VAR_DBKEY => $this->getDbKey() ));
	}
	protected function createIndexes() {
		// TODO Auto-generated method stub
		isDebug('hmt_reason_create') && $this->log()->debug(__METHOD__ . ' START');
		$this->processSqlFiles(self::PREFIX_INDEX_REGEX, __FUNCTION__, Hpls_Mailtrace_Err::ERR_REASON_CREATE_INDEX, array(
				self::SQL_VAR_TABLEPREFIX => $this->_options['tableprefix'],
				self::SQL_VAR_DBKEY => $this->getDbKey() ));
	}
	
	/**
	 * @todo comment
	 * @param unknown_type $regex
	 * @param unknown_type $stepName
	 * @param unknown_type $errCode
	 * @param unknown_type $replaces
	 * @throws Hpls_Mailtrace_Exception
	 */
	protected function processSqlFiles($regex, $stepName, $errCode, $replaces = null) {
		isDebug('hmt_reason_create') && $this->log()->debug(__METHOD__ . ' START');
		try {
			$files = $this->getSqlFiles($regex);
			foreach ($files as $file) {
				$text = file_get_contents($file);
				foreach (Hpls_Utils::extractSections($text, Hpls_Consts0::MARKER_PHPSQL_BEGIN, Hpls_Consts0::MARKER_PHPSQL_END) as $sql) {
					if (!empty($replaces)) {
						$sql = str_replace(array_keys($replaces), array_values($replaces), $sql);
					}
// 					isDebug('hmt_reason_create') &&
					 $this->log()->debug(__METHOD__ . ' sql: ' . $sql);
					// $stm = $this->getDb()->closeConnection();
					$stm = $this->getDb()->prepare($sql);
					if (!$stm->execute()) {
						$this->log()->err(__METHOD__ . ' ERROR in SQL');
					}
				}
			}
		} catch (Exception $ex) {
			$this->log()->err($ex);
			require_once 'Hpls/Mailtrace/Exception.php';
			throw new Hpls_Mailtrace_Exception("Error in creating reason-db while executing step $stepName.", $errCode, $ex);
		}
	}

	protected function getSqlFiles($regex) {
		$cfgDir = Hpls_Application0::i()->getConfigDir(Hpls_Mailtrace_Consts::COMP_NAME);
		isDebug('hmt_reason_create') && $this->log()->debug(__METHOD__ . " START - cfgdir: $cfgDir - regex: $regex");
		
		$sqlFile1 = $cfgDir . DIRECTORY_SEPARATOR . 'SQL' . DIRECTORY_SEPARATOR . $this->getDbType() . DIRECTORY_SEPARATOR . 'reason';
		isDebug('hmt_reason_create') && $this->log()->debug(Hpls_Debug::print_r($this->_options, __METHOD__ . ' CONFIG'));
		$files = scandir($sqlFile1);
		isDebug('hmt_reason_create') && $this->log()->debug(Hpls_Debug::print_r($files, __METHOD__ . ' FOUND FILES'));
		$ret = array();
		foreach ($files as $file) {
			if (preg_match($regex, $file)) {
				$ret[] = $sqlFile1 . DIRECTORY_SEPARATOR . $file;
			}
		}
		sort($ret);
		isDebug('hmt_reason_create') && $this->log()->debug(Hpls_Debug::print_r($ret, __METHOD__ . ' RESULT'));
		return $ret;
	}
	
	protected function getDbName() {
		return $this->_options['dbname'];
	}
	
	protected function getDbAdapterConfig() {
		return $this->_options;
	}

}