<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 10.03.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Var/Parser.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hmt_Reason_Item {
	public static $DELIVER_MAP = array(
			'd_id' => Hmt_Db_Consts::F_D_ID,
			'd_ref_c_id' => Hmt_Db_Consts::F_D_REF_C_ID,
			Hpls_Mailtrace_Consts::SRCH_MAILTO => Hpls_Mailtrace_Consts::SRCH_MAILTO,
			//			Hpls_Mailtrace_Consts::SRCH_D_MAILTO_ADR => Hmt_Db_Consts::F_D_MAILTO_ADR,
			//			Hpls_Mailtrace_Consts::SRCH_D_MAILTO_DMN => Hmt_Db_Consts::F_D_MAILTO_DOMAIN,
			Hpls_Mailtrace_Consts::SRCH_D_ORIGINAL_MAILTO => Hpls_Mailtrace_Consts::SRCH_D_ORIGINAL_MAILTO,
			//			Hpls_Mailtrace_Consts::SRCH_D_ORIGINAL_MAILTO_ADR => Hmt_Db_Consts::F_D_ORIGTO_ADR,
			//			Hpls_Mailtrace_Consts::SRCH_D_ORIGINAL_MAILTO_DMN => Hmt_Db_Consts::F_D_ORIGTO_DOMAIN,
			Hpls_Mailtrace_Consts::SRCH_STATUS => Hmt_Db_Consts::F_D_STATUS,
			Hpls_Mailtrace_Consts::SRCH_REASON => Hmt_Db_Consts::F_D_REASON,
			Hpls_Mailtrace_Consts::SRCH_RELAY_HOST => Hmt_Db_Consts::F_D_RELAYHOST,
			Hpls_Mailtrace_Consts::SRCH_D_QUEUE_ID => Hmt_Db_Consts::F_D_QUEUE_ID,
			Hpls_Mailtrace_Consts::SRCH_D_TIME => Hmt_Db_Consts::F_D_TIME );
	public static $CONNECT_MAP = array(
			Hpls_Mailtrace_Consts::SRCH_C_ID => 'c_id',
			Hpls_Mailtrace_Consts::SRCH_CLIENT => Hmt_Db_Consts::F_C_CLIENT,
			'c_client_hostname' => '',
			'c_client_ip' => '',
			//			Hpls_Mailtrace_Consts::SRCH_MAILFROM => Hmt_Db_Consts::F_C_MAILFROM,
			Hpls_Mailtrace_Consts::SRCH_MAILFROM => Hpls_Mailtrace_Consts::SRCH_MAILFROM,
			//			Hpls_Mailtrace_Consts::SRCH_C_MAIL_FROM_ADR => Hmt_Db_Consts::F_C_MAILFROM_ADR,
			//			Hpls_Mailtrace_Consts::SRCH_C_MAIL_FROM_DMN => Hmt_Db_Consts::F_C_MAILFROM_DOMAIN,
			Hpls_Mailtrace_Consts::SRCH_MSG_ID => Hmt_Db_Consts::F_C_MSG_ID,
			Hpls_Mailtrace_Consts::SRCH_SIZE => Hmt_Db_Consts::F_C_SIZE,
			Hpls_Mailtrace_Consts::SRCH_C_QUEUE_ID => Hmt_Db_Consts::F_C_QUEUE_ID,
			Hpls_Mailtrace_Consts::SRCH_MX => Hmt_Db_Consts::F_C_MXNAME,
			Hpls_Mailtrace_Consts::SRCH_C_TIME => Hmt_Db_Consts::F_C_TIME );
	
	private $_varParser = null;
	public $deliverEntry = null;
	public $connectEntry = null;
	public $role = 'admin';
	public $lang = 'de_dE';
	public $matches = null;
	public $shortExplain = null;
	public $reasonTag = null;

	public function __construct($regexData, $matches = null, $deliverEntry = null) {
		if (FALSE == is_null($regexData)) {
			$this->setRegExData($regexData);
		}
		
		$this->deliverEntry = $deliverEntry;
		$this->setMatches($matches);
	}

	public function getReasonTag() {
		if (is_null($this->reasonTag)) {
			$this->reasonTag = Hmt_Reason::i()->getReasonTag($this, $this->getRole(), $this->getLang());
		}
		$parser = $this->getVarParser();
		$parser->clear();
		$parser->parse($this->reasonTag);
		foreach ($parser->getContainedVars() as $var) {
			$parser->setValue($var, $this->getVarValue($var));
		}
		return $parser->getReplaced();
	}

	public function getShortExplanation() {
		if (is_null($this->shortExplain)) {
			$this->shortExplain = Hmt_Reason::i()->getShortExplanation($this, $this->getRole(), $this->getLang());
		}
		$parser = $this->getVarParser();
		$parser->clear();
		$parser->parse($this->shortExplain);
		foreach ($parser->getContainedVars() as $var) {
			$parser->setValue($var, $this->getVarValue($var));
		}
		
		Hpls_Debug::log()->debug(__METHOD__ . Hpls_Debug::print_r($parser->getReplaced(), 'REPLACED'));
		//		$parser->getContainedVars();
		

		return $parser->getReplaced(); //$this->shortExplain;
	}

	public function getLongExplanation() {
		$explain = Hmt_Reason::i()->getLongExplanation($this->getRuleID(), $this->getRole(), $this->getLang());
		$parser = $this->getVarParser();
		$parser->clear();
		$parser->parse($explain);
		foreach ($parser->getContainedVars() as $var) {
			$parser->setValue($var, $this->getVarValue($var));
		}
		return $parser->getReplaced();
	}

	public function setConnect(Hmt_Log_ConnectEntry $entry) {
		$this->connectEntry = $entry;
	}

	public function setRegExData(array $values) {
		array_change_key_case($values, CASE_LOWER);
		if (isset($values['param_map'])) {
			$param_map = Hpls_Utils::split_quoted_correct(';', $values['param_map']);
			$map = array();
			foreach ($param_map as $param) {
				
				$split = Hpls_Utils::split_quoted_correct('=', $param);
				if (count($split) == 2) {
					$v = strtolower(trim($split[1]));
					$v = preg_replace('/^"([^"]*)"$/', '$1', $v);
					$map[intval(trim($split[0]))] = Hpls_Utils::str_deqoute(strtolower(trim($split[1])));
				}
			}
			$values['param_map'] = $map;
		} else {
			$values['param_map'] = array();
		}
		foreach ($values as $key => $value) {
			$this->{$key} = $value;
		}
	}

	public function setMatches($matches) {
		$this->matches = new stdClass();
		foreach ($matches as $idx => $v) {
			$idx = $this->mapMatchParameter($idx);
			if (!is_null($idx)) {
				$this->matches->{$idx} = $v;
			}
		}
	}

	public function getRuleID() {
		return $this->rule_id . Hmt_Reason::ID_SEP . $this->rule_id_t;
	}

	public function getRole() {
		return $this->role;
	}

	public function setRole($role) {
		$this->role = $role;
	}

	public function getLang() {
		return $this->lang;
	}

	public function setLanguage($lang) {
		$this->lang = $lang;
	}

	protected function mapMatchParameter($matchIdx) {
		if ($matchIdx == 0)
			return '0';
		if (isset($this->param_map) && isset($this->param_map[$matchIdx])) {
			return $this->param_map[$matchIdx];
		}
		return null;
	}

	/**
	 * @todo comment
	 * @return Hpls_Var_Parser
	 */
	protected function getVarParser() {
		if (is_null($this->_varParser)) {
			$this->_varParser = new Hpls_Var_Parser();
		}
		return $this->_varParser;
	}

	protected function getVarValue($varName) {
		$varName = strtolower($varName);
		if (strncmp('t:d_', $varName, 4) === 0) {
			//deliver
			$varName = substr($varName, 2);
			if (isset(self::$DELIVER_MAP[$varName])) {
				Hpls_Debug::log()->debug(__METHOD__ . Hpls_Debug::print_r($this->deliverEntry));
				return $this->deliverEntry->{self::$DELIVER_MAP[$varName]};
			}
			return "{{{deliver:$varName}}}";
		} elseif (strncmp('t:c_', $varName, 4) === 0) {
			//connect
			if (is_null($this->connectEntry)) {
				return strtoupper("[[$varName]]");
				
//				require_once 'Hpls/Mailtrace/Exception/Reason.php';
//				throw new Hpls_Mailtrace_Exception_Reason('ConnectEntry is not set.');
			}
			if (isset(self::$CONNECT_MAP[$varName])) {
				return $this->connectEntry->{self::$CONNECT_MAP[$varName]};
			}
			return "{{{connect:$varName}}}";
		} else {
			//parameter
			if (isset($this->matches->$varName))
				return $this->matches->$varName;
			else
				return "{{{parameter:$varName}}}";
		}
	}

	/**
	 * @return the $deliverEntry
	 */
	public function getDeliverEntry() {
		return $this->deliverEntry;
	}

	/**
	 * @return the $connectEntry
	 */
	public function getConnectEntry() {
		return $this->connectEntry;
	}

	/**
	 * @param field_type $deliverEntry
	 */
	public function setDeliverEntry($deliverEntry) {
		$this->deliverEntry = $deliverEntry;
	}

	/**
	 * @param field_type $connectEntry
	 */
	public function setConnectEntry($connectEntry) {
		$this->connectEntry = $connectEntry;
	}

}