<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created   : May 25, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Acl/Reader/Xml.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Zend/Config/Xml.php';
require_once 'Hpls/Acl/Reader/Abstract.php';
require_once 'Hpls/Var/Clause.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package AppCore
 * @subpackage Rechtesystem
 * @version $LastChangedRevision: 9196 $
 * @since 1.0ufz
 */
class Hpls_Acl_Reader_Xml extends Hpls_Acl_Reader_Abstract {

	public function __construct($configPath = null) {
		if (is_null($configPath)) {
			$configPath = Hpls_Application::i()->getConfigDir();
		}
		parent::__construct($configPath);
	}

	/* (non-PHPdoc)
	 * @see Hpls_Acl_Reader_Abstract::readRawResource()
	 */
	protected function readRawResource() {
		isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " START");
		$retConfig = new Zend_Config(array(), true);
		//dirs finden
		$dirs = $this->getAclDirs();
		isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " checking dirs: " . Hpls_Debug::print_r($dirs));
		//dir depend.xml lesen
		if (count($dirs) > 0) {
			foreach ($dirs as $aclDir) {
				$xmlFile = "$aclDir/" . Hpls_Acl_Consts::ACL_DEPEND_FILENAME_XML;
				isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " file: $xmlFile");
				if (file_exists($xmlFile)) {
					try {
						isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " found: $xmlFile");
						$config = new Zend_Config_Xml($xmlFile, Hpls_Acl_Consts::ACL_DEPEND_CONFIG_SECTION);
						$retConfig->merge($config);
					} catch (Zend_Config_Exception $ex) {
						$this->log()->err("$xmlFile is not readable as XML-config: $ex");
					}
				}
			}
		}
		//		if (DEBUG) {
		//		Hpls_Utils::getLogger()->logWVar("Dirs: ",$dirs);
		//		Hpls_Utils::getLogger()->logWVar("ACLConfig: ",$retConfig->toArray());
		//		}
		return $retConfig->toArray();
	}

	/* (non-PHPdoc)
	 * @see Hpls_Acl_Reader_Abstract::readModuleAcls()
	 */
	protected function readModuleResMap() {
		// TODO Auto-generated method stub
		isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " START");
		//dirs finden
		$dirs = $this->getAclDirs();
		$ret = new Zend_Config(array(), TRUE);
		//dir depend.xml lesen
		if (count($dirs) > 0) {
			//			isDebug('acl', 'acl_read') &&
			$this->log()->debug(__METHOD__ . " checking dirs: " . Hpls_Debug::print_r($dirs));
			foreach ($dirs as $aclDir) {
				$dircontent = scandir($aclDir);
				foreach ($dircontent as $file) {
					isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " *** trying '$file'");
					
					if (preg_match('/' . Hpls_Acl_Consts::ACL_ACL_FILENAME_POSTFIX_XML . '$/', $file) > 0) {
						//						isDebug('acl', 'acl_read') &&
						isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " *** found '$file'");
						
						try {
							$content = file_get_contents("$aclDir/$file");
							$tmpContent = simplexml_load_string($content);
							isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($tmpContent, " *** XML parsed ***"));
							$result = $this->parseContentAcls($tmpContent);
							$ret->merge($result);
						} catch (Zend_Config_Exception $ex) {
							$this->log()->err("$aclDir/$file is not readable as XML-config: $ex");
						}
					}
				}
			}
		}
		isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " END" . Hpls_Debug::print_r($ret, 'XML'));
		return $ret;
	}

	/**
	 * @todo comment
	 * @param SimpleXMLElement $itemXml
	 * @return Zend_Config
	 */
	protected function parseContentAcls(SimpleXMLElement $itemXml) {
		//		isDebug('acl', 'acl_read') &&
		isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " START");
		$ret = new Zend_Config(array(), true);
		if (isset($itemXml->module)) {
			
			foreach ($itemXml->module as $moduleXml) {
				isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " module");
				$mCfg = new Zend_Config(array(), true);
				$this->parseAcls($moduleXml, $mCfg);
				if (isset($moduleXml)) {
					foreach ($moduleXml->controller as $controllerXml) {
						isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " controller");
						$cCfg = new Zend_Config(array(), true);
						$this->parseAcls($controllerXml, $cCfg);
						if (isset($controllerXml->action)) {
							foreach ($controllerXml->action as $actionXml) {
								isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " action");
								$aCfg = new Zend_Config(array(), true);
								$this->parseAcls($actionXml, $aCfg);
								if (isset($actionXml->param)) {
									foreach ($actionXml->param as $paramXml) {
										isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " param");
										$pCfg = new Zend_Config(array(), true);
										$this->parseAcls($paramXml, $pCfg);
										$aCfg->{$paramXml['name']} = $pCfg;
									}
								}
								$cCfg->{$actionXml['name']} = $aCfg;
							
							}
						}
						$mCfg->{$controllerXml['name']} = $cCfg;
					}
				}
				$ret->{$moduleXml['name']} = $mCfg;
			}
		}
		return $ret;
	}

	/**
	 * Liest den Block zu ACLs ein
	 * @param SimpleXMLElement $itemXml
	 * @param Zend_Config $cfg
	 */
	protected function parseAcls(SimpleXMLElement $itemXml, Zend_Config $cfg) {
		isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " START");
		$ret = array();
		if (isset($itemXml->acl)) {
			foreach ($itemXml->acl as $aclXml) {
				if (isset($aclXml->and)) {
					foreach ($aclXml->and as $aclAndXml) {
						$this->parseAclInner($aclAndXml, $cfg, Hpls_Var_Clause::CL_AND);
					}
				} elseif (isset($aclXml->or)) {
					foreach ($aclXml->or as $aclOrXml) {
						$this->parseAclInner($aclOrXml, $cfg, Hpls_Var_Clause::CL_OR);
					}
				} elseif (isset($aclXml->not)) {
					foreach ($aclXml->not as $aclNotXml) {
						$this->parseAclInner($aclNotXml, $cfg, Hpls_Var_Clause::CL_NOT);
					}
				}
			}
		}
	
	}

	/**
	 * @todo comment
	 * @param SimpleXMLElement $aclXml
	 * @param Hpls_Navigation_PageInterface $itemPage
	 * @param string $clauseLogic
	 */
	protected function parseAclInner(SimpleXMLElement $aclXml, Zend_Config $itemPage, $clauseLogic) {
		isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . " START");
		if (isset($aclXml->resource)) {
			$clause = new Hpls_Var_Clause($clauseLogic);
			foreach ($aclXml->resource as $resoureXml) {
				$res = Hpls_Acl::splitResourceNeeds((string) $resoureXml['refid']);
				isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($res, " *** Resource parsed ***"));
				if (isset($resoureXml['privileges'])) {
					$privs = explode(Hpls_Acl_Consts::ID_ID_SEPARATOR, strtolower(trim((string) $resoureXml['privileges'])));
				} else {
					$privs = array();
				}
				foreach ($res as $rName => $rV) {
					$p = isset($rV['privileges']) > 1 ? array_merge($privs, $rV['privileges']) : $privs;
					$clause->addValue($rName, $p);
				}
			}
			$itemPage->__resources__ = $clause;
		}
		if (isset($aclXml->role)) {
			$clause = new Hpls_Var_Clause($clauseLogic);
			foreach ($aclXml->role as $roleXml) {
				$clause->addValue((string) $roleXml['refid'], (string) $roleXml['refid']);
			}
			$itemPage->__roles__ = $clause;
		}
	}

}