<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Feb 9, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-02-21 15:00:14 +0100 (Di, 21. Feb 2012) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9222 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.libs/Hpls/Application.php $
 *
 */
/**
 */
require_once 'Zend/Registry.php';
require_once 'Zend/Application.php';
require_once 'Zend/Loader/Autoloader.php';
require_once 'Zend/Session/Namespace.php';
require_once 'Hpls/Application0.php';
require_once 'Hpls/Acl.php';
require_once 'Hpls/Debug.php';
require_once 'Hpls/Utils.php';
require_once 'Hpls/Log.php';
require_once 'Hpls/Msgs.php';
require_once 'Hpls/Settings.php';
require_once 'Hpls/Settings/Handler/Config.php';

/**
 *
 * @todo comment
 *      
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *           
 * @package HPLS
 * @subpackage subpackage
 * @version $LastChangedRevision: 9222 $
 * @since alpha
 *       
 */
abstract class Hpls_Application extends Hpls_Application0 {
	protected static $POSSIBLE_PRODUCTS = array(
			'ac',
			'him',
			'has',
			'hma',
			'hmt' );
	
	/**
	 *
	 * @var Zend_Log
	 */
	protected static $_nullLogger = null;
	/**
	 *
	 * @var Zend_Controller_Front
	 */
	protected $front = null;
	protected $_initDone = false;
	protected $_initModules = array();
	/**
	 * Pfad zu den statischen Config-Dateien
	 * @var string
	 */
	protected $_configDir = null;
	/**
	 * Pfad zu den variablen bzw. beschreibbaren Config-Dateien.
	 * @var string
	 */
	protected $_variableConfigDir = null;
	public $url = null;
	protected $_installedHPLSProducts = null;
	protected $_controllerPluginstoSetup = null;

	/**
	 *
	 * @return void
	 */
	public function run() {
		if ($this->_initDone === false) {
			$this->init();
		}
		
		$this->checkLicense();
		$this->startDispatch();
	}

	protected function initApplicationConfigs() {
		// application.ini aus configs und etc lesen
		$ini = new Zend_Config_Ini($this->_configDir . DIRECTORY_SEPARATOR . 'AC' . DIRECTORY_SEPARATOR . Hpls_Consts0::FILE_CONFIG_APPLICATION, null, true);
		if (isset($ini->application) && isset($ini->application->user_config_dir)) {
			$this->_variableConfigDir = (string) $ini->application->user_config_dir;
		}
		$filename = $this->_configDir . DIRECTORY_SEPARATOR . Hpls_Consts0::FILE_CONFIG_APPLICATION;
		if (file_exists($filename)) {
			$userIni = new Zend_Config_Ini($filename, null, true);
			$userIni = $userIni->toArray();
			if (isset($userIni['webui'])) {
				if (isset($userIni['webui']['url'])) {
					$ini->frontcontroller->baseUrl = $userIni['webui']['url'];
				}
				if (isset($userIni['webui']['config_dir'])) {
					$this->_variableConfigDir = (string) $userIni['webui']['config_dir'];
				}
			}
			if (isset($userIni['cache']['path'])) {
				$ini->cache->default->backend->cache_dir = $userIni['cache']['path'];
			}
			if (isset($userIni['logging'])) {
				// @todo type SYSLOG|FILE einstellbar machen
				if (isset($userIni['logging']['logger']) && is_array($userIni['logging']['logger'])) {
					$ini->log = new Zend_Config(isset($userIni['logging']['logger']), true);
				}
				if (isset($ini->log->stream) && isset($ini->log->stream->writerParams) && isset($ini->log->stream->writerParams->stream)) {
					$ini->log->stream->writerParams->stream = $userIni['logging']['path'];
				}
				if (isset($userIni['logging']['level']) && isset($ini->log->stream) && isset($ini->log->stream->filterParams)) {
					$ini->log->stream->filterParams->priority = $userIni['logging']['level'];
				}
			}
			if (isset($userIni['session'])) {
				foreach (array(
						'name',
						'use_only_cookies',
						'remember_me_seconds',
						'save_path' ) as $key) {
					if (isset($userIni['session'][$key])) {
						$ini->session->{$key} = $userIni['session'][$key];
					}
				}
				if (isset($userIni['session']['lifetime'])) {
					$ini->session->remember_me_seconds = (string) $userIni['session']['lifetime'];
				}
				if (isset($userIni['session']['path'])) { // überschreibt
				                                          // 'save_path' von oben, damit in
				                                          // der user-application.ini pfade
				                                          // immer mit 'path' angegeben
				                                          // werden können
					$ini->session->save_path = (string) $userIni['session']['path'];
				}
			}
			if (isset($userIni['i18n']['lang'])) {
				$ini->i18n->lang = (string) $userIni['i18n']['lang'];
			}
		}
		$ini->webui->view->smarty->compile_dir = $ini->cache->default->backend->cache_dir . DIRECTORY_SEPARATOR . (isset($ini->webui->view->smarty->compile_dir) ? $ini->webui->view->smarty->compile_dir : "templates");
		
		Hpls_Settings::i()->registerHandler('application', new Hpls_Settings_Handler_Config($ini));
		$config = $ini->toArray();
		$config = array_change_key_case($config, CASE_LOWER);
		Zend_Registry::set(Hpls_Consts0::REGK_CONFIG, $config);
	}

	protected function initSubModules() {
		$cfg = $this->getConfig('application');
		if (isset($cfg['configPlugin'])) {
			// erst alle Klassen erstellen
			foreach ($cfg['configPlugin'] as $className => $include) {
				require_once $include;
				$classObj = new $className($this);
				$this->_initModules[$className] = $classObj;
			}
			// ... dann die Klasse ausführen, damit sie auf sich gegenseitig
			// zugreifen können
			foreach ($this->_initModules as $classObj) {
				$classObj->init();
			}
		}
	
	}

	/**
	 * Generiert eine uniqid für diese Request.
	 * Die ID wird im Front-Controller gespeichert.
	 * Diese ID wird in alle Fehlerlogs mitgeloggt.
	 * (zur Zeit nur Frontend)
	 * Die ID wird auch bei Fehler, im WebGUI angezeigt.
	 */
	protected function initRequestId() {
		$requestId = uniqid();
		$this->getFrontController()->setParam('requestId', $requestId);
	}

	protected function initLogger() {
		$this->log()->debug(__METHOD__ . " START");
		$config = Zend_Registry::get(Hpls_Consts::REGK_CONFIG);
		$logConfigs = $config['log'];
		
		$this->log()->debug(print_r($logConfigs, true));
		$logger = Hpls_Log::factory($logConfigs);
		$logger->setRequestID(md5($this->getFrontController()->getParam('requestId', uniqid())));
		Zend_Registry::set(Hpls_Consts::REGK_LOGGER, $logger);
	}

	protected function initDebug() {
		$this->log()->debug(__METHOD__ . " START");
		
		$this->log()->debug(__METHOD__ . " setting DEBUG-CONFIG");
		$dbgConfig = Hpls_Utils::readConfigIniCascade('debug.ini', 'debug'); // new
		                                                                     // Zend_Config_Ini("$configDir/debug.ini",
		                                                                     // 'debug');
		Zend_Registry::set(REGBCK_DEBUG, $dbgConfig->toArray());
		if (!isset($dbgConfig->environment))
			define('APPLICATION_ENV', $dbgConfig->environment);
		else
			define('APPLICATION_ENV', isDebug() ? 'development' : 'production');
		define('DEBUG', isDebug());
	}

	protected function initHplsSettings() {
		Hpls_Settings::i()->registerHandler('ac', new Hpls_Settings_Handler_Component('AC'));
		// Hpls_Settings::i()->registerHandler('ac.auth', new
	// Hpls_Settings_Handler_Xml('auth.xml','ac.auth'));
	}

	public function checkLicense() {
		$this->log()->debug(__METHOD__ . " START");
		
		$this->log()->info(__FUNCTION__ . ": passed.");
	}

	protected function startDispatch() {
		$this->log()->debug("#######################################################################################");
		$this->log()->info("#### START dispatch loop ####");
		$this->front->dispatch();
		$this->log()->info("#### FINISHED dispatch loop ####");
		$this->log()->debug("#######################################################################################");
	}

	/**
	 *
	 * @return Zend_Controller_Front
	 */
	public function getFrontController() {
		if (is_null($this->front)) {
			$this->front = Zend_Controller_Front::getInstance();
		}
		return $this->front;
	}

	public function getConfigDir($product = null) {
		if (is_null($product)) {
			return $this->_configDir;
		} else {
			return $this->_configDir . '/' . strtoupper($product);
		}
	}

	public function getVariableConfigDir($product = null) {
		if (is_null($this->_variableConfigDir)) {
			$this->_variableConfigDir = APPLIANCE_PART_PATH . DIRECTORY_SEPARATOR . 'etc';
		}
		if (is_null($product)) {
			return $this->_variableConfigDir;
		} else {
			return $this->_variableConfigDir . DIRECTORY_SEPARATOR . strtoupper($product);
		}
	}

	/**
	 *
	 * @todo comment
	 * @param $compname string       	
	 * @return array NULL
	 * @throws Hpls_Exception_Configuration If the configuration is missing for
	 *         $compname.
	 * @deprecated use Hpls_Settings instead
	 */
	public function getComponentConfig($compname) {
		$comps = Zend_Registry::get(Hpls_Consts::REGK_COMPONENTS);
		if (is_array($comps)) {
			if (!is_array($comps[$compname])) {
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration("No configuration for module '$compname'. Probaly component is missing.", $compname, 10);
			}
			return $comps[$compname][Hpls_Consts::CFG_COMP_CONFIG];
		}
		return null;
	}

	/**
	 *
	 * @param
	 *       	 $sectionName
	 * @param $compName (Projekt)Name
	 *       	 des Produkts/Komponente
	 * @return Zend_Config mixed
	 */
	public function getConfig($sectionName, $compName = 'default') {
		$cfg = Zend_Registry::get(Hpls_Consts::REGK_CONFIG);
		
		return isset($cfg[$sectionName]) ? $cfg[$sectionName] : null;
	}

	/**
	 *
	 * @todo comment
	 *      
	 * @param $className unknown_type       	
	 * @return Hpls_Application_ConfigAbstract NULL
	 */
	public function getConfigurer($className) {
		return isset($this->_initModules[$className]) ? $this->_initModules[$className] : null;
	}

	public function getInstalledHPLSProducts() {
		if (is_null($this->_installedHPLSProducts)) {
			$ret = array();
			$dirs = scandir($this->getConfigDir());
			foreach ($dirs as $dir) {
				if (in_array(strtolower($dir), self::$POSSIBLE_PRODUCTS)) {
					$ret[strtolower($dir)] = $dir;
				}
			}
			$this->_installedHPLSProducts = $ret;
		}
		return $this->_installedHPLSProducts;
	}

	public function getThemeName() {
		return 'HPLS';
	}

	public function getThemeUrl() {
		return $this->getFrontController()->getBaseUrl() . '/' . $this->getThemeName();
	}

	public function getThemeHtmlPath() {
		return APPLIANCE_PATH . DIRECTORY_SEPARATOR . 'frontend' . DIRECTORY_SEPARATOR . 'public' . DIRECTORY_SEPARATOR . $this->getThemeName();
	}

	public function getLicense() {
		if (!Zend_Registry::isRegistered(Hpls_Consts::REGK_LICENSE)) {
			$licManager = Hpls_License_Manager::getInstance();
			Zend_Registry::set(Hpls_Consts::REGK_LICENSE, $licManager);
		}
		return Zend_Registry::get(Hpls_Consts::REGK_LICENSE);
	}

	/**
	 *
	 * @todo comment
	 *      
	 * @return Zend_Log
	 */
	protected function log() {
		if (Zend_Registry::isRegistered(Hpls_Consts::REGK_LOGGER)) {
			return Zend_Registry::get(Hpls_Consts::REGK_LOGGER);
		} else {
			if (is_null(self::$_nullLogger)) {
				
				// DEBUG set to null-logger!
				self::$_nullLogger = new Zend_Log();
				require_once 'Zend/Log/Writer/Null.php';
				self::$_nullLogger->addWriter(new Zend_Log_Writer_Null());
				// require_once 'Zend/Log/Writer/Stream.php';
				// self::$_nullLogger->addWriter(new
				// Zend_Log_Writer_Stream(APPLIANCE_PART_PATH .
				// '/logs/AC_init.log'));
				// self::$_nullLogger->addFilter(Zend_Log::ERR);
				Zend_Registry::set(Hpls_Consts::REGK_LOGGER, self::$_nullLogger);
			}
			return self::$_nullLogger;
		}
	
	}

	protected function initAutoloader() {
		$this->log()->debug(__METHOD__ . " START");
		Zend_Loader_Autoloader::getInstance();
	}

	/**
	 *
	 * @param $filename string       	
	 * @param $section string       	
	 * @return Zend_Config
	 */
	protected function readIniCascaded($filename, $section = null, array $forProducts = array()) {
		return Hpls_Utils::readConfigIniCascade($filename, $section);
	}

	protected function readXmlCascaded($filename, $section = null, array $forProducts = array()) {
		return Hpls_Utils::readConfigXmlCascade($filename, $section);
	}

}

