<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created        : 21.10.2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.libs/Hpls/Application/Config/Constraints.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Application/ConfigAbstract.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9196 $
 * @since Magdeburg:Produktiv2
 */
class Hpls_Application_Config_Constraints extends Hpls_Application_ConfigAbstract {
	
	protected $_configIni = null;

	/**
	 * 
	 */
	public function init() {
		isDebug('method_trace') && $this->log()->debug(__METHOD__ . " START");
		$globalCfg = $this->getConfig('application');
		$this->_configIni = $this->readIniCascaded('constraints.ini')->toArray();
		
		if (isset($globalCfg[Hpls_Consts::CFG_GLOBAL_UNIFICATION_ON]) && TRUE === Hpls_Utils::toBool($globalCfg[Hpls_Consts::CFG_GLOBAL_UNIFICATION_ON])) {
			$this->initUnificationConfig();
		} else {
			$frontInitObj = $this->getApplicationConfigurer('Hpls_Application_Config_Frontcontrollerplugins');
			if (is_object($frontInitObj)) {
				$frontInitObj->setDontInclude('Hpls_Controller_Plugin_Unification');
			}
		}
		if (isset($globalCfg[Hpls_Consts::CFG_GLOBAL_VALIDATION_ON]) && TRUE === Hpls_Utils::toBool($globalCfg[Hpls_Consts::CFG_GLOBAL_VALIDATION_ON])) {
			$this->initValidationConfig();
		} else {
			$frontInitObj = $this->getApplicationConfigurer('Hpls_Application_Config_Frontcontrollerplugins');
			if (is_object($frontInitObj)) {
				$frontInitObj->setDontInclude('Hpls_Controller_Plugin_Validation');
			}
		}
		if (isset($globalCfg[Hpls_Consts::CFG_GLOBAL_POLICYCHECK_ON]) && TRUE === Hpls_Utils::toBool($globalCfg[Hpls_Consts::CFG_GLOBAL_POLICYCHECK_ON])) {
			$this->initPolicycheckConfig();
		} else {
			$frontInitObj = $this->getApplicationConfigurer('Hpls_Application_Config_Frontcontrollerplugins');
			if (is_object($frontInitObj)) {
				$frontInitObj->setDontInclude('Hpls_Controller_Plugin_Validation');
			}
		}
	
	}

	protected function initUnificationConfig() {
		isDebug('method_trace') && $this->log()->debug(__METHOD__ . " START");
		$array = $this->transformConfig('unifiers', 'unifier');
		//@todo Zend_Cache
		isDebug('constraints') && $this->log()->debug(Hpls_Debug::print_r($array, __METHOD__));
		Zend_Registry::set(Hpls_Consts0::REGK_UNIFICATION_CONFIG, $array);
	}

	protected function initValidationConfig() {
		isDebug('method_trace') && $this->log()->debug(__METHOD__ . " START");
		$array = $this->transformConfig('validators', 'validator');
		//@todo Zend_Cache
		isDebug('constraints') && $this->log()->debug(Hpls_Debug::print_r($array, __METHOD__));
		Zend_Registry::set(Hpls_Consts0::REGK_VALIDATION_CONFIG, $array);
	}

	protected function initPolicycheckConfig() {
		isDebug('method_trace') && $this->log()->debug(__METHOD__ . " START");
		$array = $this->transformConfig('policies', 'policy');
		isDebug('constraints') && $this->log()->debug(Hpls_Debug::print_r($array, __METHOD__));
		//@todo Zend_Cache
		Zend_Registry::set(Hpls_Consts0::REGK_POLICY_CONFIG, $array);
	
	}

	/**
	 * Überführt die Einstellungsangaben aus dem INI-Format in eine interne Repräsentation
	 * @param unknown_type $section
	 * @param unknown_type $singleSection
	 * @throws Hpls_Exception_Configuration
	 * @return Ambigous <multitype:, multitype:multitype: , unknown>
	 */
	protected function transformConfig($section, $singleSection) {
		//@todo Zend_Cache
		$ret = array();
		$errors = array();
		isDebug('constraints') && $this->log()->debug(Hpls_Debug::print_r($this->_configIni, __METHOD__ . "($section)"));
		//[mailtrace].search
		$copies = array();
		foreach ($this->_configIni as $module => $controllers) {
			$module = strtolower($module);
			$ret[$module] = array();
			//[mailtrace].search.query: search => query,...
			foreach ($controllers as $controller => $actions) {
				$controller = strtolower($controller);
				$ret[$module][$controller] = array();
				//[mailtrace].search.query.X: query => X,...
				foreach ($actions as $action => $jobs) {
					//					$ret[$module][$controller][$action] = array();
					$action = strtolower($action);
					$checker = array();
					foreach ($jobs as $job => $description) {
						$job = strtolower($job);
						if ($job == '__copy__') {
							//von wo anders kopieren
							$copies["$module/$controller/$action"] = $description;
						} else {
							if (!isset($description['fields'])) {
								$errors[] = "key 'fields' is missing in section $module.$controller.$action.$job";
								continue;
							}
							foreach (array(
									'all',
									'each' ) as $quant) {
								if (isset($description[$quant])) {
									if (isset($description[$quant][$section])) {
										$classes = explode(',', $description[$quant][$section]);
										foreach ($classes as $class) {
											if ((strncmp($class, 'Hpls_', 5) === 0) || (strncmp($class, 'Zend_', 5) === 0)) {
												
											} else {
												$class = $this->getDefaultPrefixedClass($class, $section);
											}
											$path = explode('_', $class);
											$options = null;
											if (!isset($checker[$class])) {
												$checker[$class] = array();
											}
											//übernehmen
											$checker[$class]['file'] = implode(DIRECTORY_SEPARATOR, $path) . '.php';
											//zusätzliche optionen für die CheckClass
											$checker[$class]['options'] = $options;
											//wenn zum ersten $class in verbindung mit $quant auftaucht, array initialsieren
											if (!isset($checker[$class][$quant]) || !is_array($checker[$class][$quant])) {
												$checker[$class][$quant] = array();
											}
											$checker[$class][$quant][] = is_array($description['fields']) ? $description['fields'] : explode(',', $description['fields']);
											
										}
									}
									if (isset($description[$quant][$singleSection])) {
										//jetzt folgt eine arraylist
										foreach ($description[$quant][$singleSection] as $constraint) {
											if (isset($constraint['name'])) {
												$class = $constraint['name'];
												$path = explode('_', $class);
												if (!isset($checker[$class])) {
													$checker[$class] = array();
												}
												if (isset($constraint['file'])) {
													$checker[$class]['file'] = $constraint['file'];
												} else {
													$checker[$class]['file'] = implode(DIRECTORY_SEPARATOR, $path) . '.php';
												}
												
												if (isset($constraint['options'])) {
													$checker[$class]['options'] = $constraint['options'];
												}
												
												if (!isset($checker[$class][$quant]) || !is_array($checker[$class][$quant])) {
													$checker[$class][$quant] = array();
												}
												$checker[$class][$quant][] = is_array($description['fields']) ? $description['fields'] : explode(',', $description['fields']);
											
											}
										
										}
									
									}
								}
							} //foreach $quant
						} //if copy
					} //foreach $jobs
					$ret[$module][$controller][$action] = $checker;
				} //foreach $actions
			} //foreach $constrollers
		}
		foreach ($copies as $to => $from) {
			$fromKeys = explode('.', $from);
			$toKeys = explode('/', $to);
			if (count($fromKeys) != count($toKeys)) {
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration('In "constraints.ini" "__copy__" requires that destination "' . $to . '" is on same level as source (current: "' . $from . '").');
			}
			switch (count($fromKeys)) {
				case 1 :
				case 2 :
				case 3 :
				default :
					if (!isset($ret[$toKeys[0]])) {
						$ret[$toKeys[0]] = array(
								$toKeys[1] => array(
										$toKeys[2] => $ret[$fromKeys[0]][$fromKeys[1]][$fromKeys[2]] ) );
					} elseif (!isset($ret[$toKeys[0]][$toKeys[1]])) {
						$ret[$toKeys[0]][$toKeys[1]] = array(
								$toKeys[2] => $ret[$fromKeys[0]][$fromKeys[1]][$fromKeys[2]] );
					} else {
						$ret[$toKeys[0]][$toKeys[1]][$toKeys[2]] = $ret[$fromKeys[0]][$fromKeys[1]][$fromKeys[2]];
					}
			
			}
		}
		
		if (count($errors) > 0) {
			require_once 'Hpls/Exception/Configuration.php';
			throw new Hpls_Exception_Configuration('Something is wrong with the configuration file "checks.ini": ' . Hpls_Debug::print_r($err));
		}
		isDebug('constraints') && $this->log()->debug(Hpls_Debug::print_r($ret, __METHOD__ . "($section)"));
		return $ret;
	}

	protected function getDefaultPrefixedClass($class, $section) {
		switch ($section) {
			case 'unifiers' :
				return 'Hpls_Constraint_Unifier_' . $class;
			case 'validators' :
				return 'Hpls_Constraint_Validate_' . $class;
			case 'policies' :
				return 'Hpls_Constraint_Policy_' . $class;
		}
		return $class;
	}

}
