<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Jan 25, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-12-06 10:48:28 +0100 (Do, 06. Dez 2012) $
 * last changed by: $LastChangedBy: d.mohn $
 * revision       : $LastChangedRevision: 9847 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Auth/Adapter/Abstract.php $
 *
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 *
 */
require_once 'Zend/Auth/Adapter/Interface.php';
require_once 'Hpls/Var/Matcher.php';

/**
 * @todo comment
 * 
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9847 $
 * @since Magdeburg:Produktiv2
 */
abstract class Hpls_Auth_Adapter_Abstract implements 
	Zend_Auth_Adapter_Interface {
	const METHOD_MD5 = Hpls_Consts0::AUTH_CRYPTMETHOD_MD5;
	const METHOD_PLAIN = Hpls_Consts0::AUTH_CRYPTMETHOD_PLAIN;
	
	const OPT_USERNAME = Hpls_Consts0::OPT_USERNAME;
	const OPT_PASSWORD = Hpls_Consts0::OPT_PASSWORD;
	protected $_options = array();
	/**
	 * @todo comment
	 * @var Hpls_Var_Matcher
	 */
	protected $_loginMapper = null;
	protected $_sourceName = null;

	public function __construct($sourcename, $options = array(), array $roleOptions = array()) {
		$this->_sourceName = $sourcename;
		if ($options instanceof Zend_Config) {
			$options = $options->toArray();
		}
		if (!empty($roleOptions) || !isset($options['roles'])) {
			$options['roles'] = $roleOptions;
		}
		$this->setOptions($options);
		Hpls_Utils::log()->debug(get_class($this) . Hpls_Debug::print_r($options, ' OPTIONS '));
	}

	public function __get($name) {
		return $this->_options[strtolower($name)];
	}

	public function __set($varName, $value) {
		$this->_options[strtolower($varName)] = $value;
	}

	public function __isset($varname) {
		return isset($this->_options[$varname]);
	}

	public function __unset($varname) {
		unset($this->_options[$varname]);
	}

	function setPassword($password) {
		$this->_options[self::OPT_PASSWORD] = trim($password);
	}

	function setUsername($username) {
		$this->_options[self::OPT_USERNAME] = trim($username);
		$this->getLoginMapper()->setVariable(Hpls_Consts0::AUTH_VAR_LOGIN, $this->_options[self::OPT_USERNAME]);
	
	}

	function getUsername() {
		return $this->getOption(self::OPT_USERNAME, null);
	}

	function getPassword() {
		return $this->getOption(self::OPT_PASSWORD, null);
	}

	/**
	 * Gibt den Nutzernamen zurück, der zum ersten Anmelden genommen werden soll.
	 * 
	 * Wenn ein Proxy-User gebraucht wird, wird hier der Proxy-Username zurück gegeben, und getUsername() gibt den eigentlich zu authentifizierenden Username zurück.
	 * Diese Funktionalität muss allerdings in Ableitungen implementiert werden, in Abstract ist die Rückgabe == getUsername()
	 * 
	 * @return string
	 */
	function getAuthUsername() {
		return $this->getUsername();
	}

	function getAuthPassword() {
		return $this->getPassword();
	}

	/**
	 * @return the $_sourceName
	 */
	public function getSourceName() {
		return $this->_sourceName;
	}

	/**
	 * @param field_type $_sourceName
	 */
	public function setSourceName($_sourceName) {
		$this->_sourceName = $_sourceName;
	}

	function &getOptions() {
		return $this->_options;
	}

	function getOption($key, $default = null) {
		return $this->__isset($key) ? $this->__get($key) : $default;
	}

	function setOptions(array $options) {
		$options = array_change_key_case($options, CASE_LOWER);
		if (!empty($options[Hpls_Consts0::OPT_LOGINMAPPING])) {
			$t = $options[Hpls_Consts0::OPT_LOGINMAPPING];
			if ($t instanceof Hpls_Var_Matcher) {
				$this->_loginMapper = $t;
			} elseif (is_array($t)) {
				$this->_loginMapper = new Hpls_Var_Matcher(array(
						'match' => $t ));
			}
		} else {
			//default mapper erzeugen
			$this->_loginMapper = Hpls_Var_Matcher::createLoginMapper();
		}
		$this->_options = $options;
	
	}

	function setOption($key, $value) {
		$this->__set($key, $value);
	}

	/**
	 * @todo comment
	 */
	function generateIdentity(array $optional = array()) {
		$ret = array(
				'username' => $this->getUsername(),
				'source' => $this->getSourceName()
		);
		return array_merge($ret, $optional);
	}

	protected function setDefaults(array $keyValue) {
		foreach ($keyValue as $k => $v) {
			if (!isset($this->_options[$k])) {
				$this->_options[$k] = $v;
			}
		}
	}

	protected function parseRoles($rolesEntry) {
		if (empty($rolesEntry)) {
			return array();
		} elseif (is_array($rolesEntry)) {
			return $rolesEntry;
		} else {
			$entries = explode(',', $rolesEntry);
			array_walk($entries, 'array_walk_trim');
			return $entries;
		}
	}

	protected function getRoles() {
		$this->log()->debug(Hpls_Debug::print_r($this->getUsername() . ' - ' . $this->getSourceName(), __METHOD__));
		$roles = Hpls_Auth::i()->readRolesForLogingetUserRoles($this->getUsername(), $this->getSourceName());
		$roles[Hpls_Acl::ROLE_LOGGEDIN] = Hpls_Acl::ROLE_LOGGEDIN;
		return $roles;
	}

	protected function log() {
		return Hpls_Utils::log();
	}

	/**
	 * @todo comment
	 * @return Hpls_Var_Matcher
	 */
	protected function getLoginMapper() {
		return $this->_loginMapper;
	}

	protected function getSubstituted($text) {
		if (!empty($text))
			return $this->getLoginMapper()->replace($this->getOption(self::OPT_USERNAME, ''), $text);
		else
			return '';
	}

}

