<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created        : 15.10.2010
 * created by     : author
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : author
 */
/***/
require_once 'Hpls/Auth/Adapter/Abstract.php';

class Hpls_Auth_Adapter_Ini extends Hpls_Auth_Adapter_Abstract {
	const OPT_FILE = 'file';
	/**
	 * @todo comment
	 * @var unknown_type
	 */
	const OPT_CRYPTMETH = 'cryptmethod';
	const OPT_INI_PASSWORT_KEY = 'passwordkey';
	const OPT_INI_USERNAME_KEY = 'usernamekey';
	
	/**
	 * @var Zend_Config_Ini
	 */
	protected $_userIni = null;

	public function __construct($sourcename, array $options = array(), array $roleOptions = array()) {
		parent::__construct($sourcename, $options, $roleOptions);
	}

	function setOptions(array $options) {
		$options = array_change_key_case($options,CASE_LOWER);
		if (empty($options[self::OPT_FILE])) {
			require_once 'Hpls/Exception/Configuration.php';
			throw new Hpls_Exception_Configuration('In auth.xml for auth-source "'.$this->getSourceName().'" a "file" name is missing.',Hpls_Consts0::ERR_AUTH_INI_NOFILE);
		}
		if (empty($options[self::OPT_CRYPTMETH])) {
			$options[self::OPT_CRYPTMETH] = self::METHOD_MD5;
		}
		if (empty($options[self::OPT_INI_PASSWORT_KEY])) {
			$options[self::OPT_INI_PASSWORT_KEY] = 'password';
		}
		if (empty($options[self::OPT_INI_USERNAME_KEY])) {
			$options[self::OPT_INI_USERNAME_KEY] = '[['.Hpls_Consts0::AUTH_VAR_LOGIN.']]';
		}
		parent::setOptions($options);
		$this->setFile($options[self::OPT_FILE]);
	}

	/* (non-PHPdoc)
	 * @see Zend/Auth/Adapter/Zend_Auth_Adapter_Interface#authenticate()
	 */
	function authenticate() {
		$this->log()->debug(__METHOD__ . ': ' . Hpls_Debug::print_r($this->getUsername(), 'USER') . Hpls_Debug::print_r($this->getPassword(), 'PASSWORD').Hpls_Debug::print_r($this->_options,' OPTIONS') . Hpls_Debug::print_r($this->_userIni, 'USERINI'));
		$code = Zend_Auth_Result::FAILURE;
		$identity = null;
		$msgs = array();
		/* @var Zend_Config_Ini */
		$userEntry = Hpls_Utils::array_deep_value($this->_userIni, $this->_loginMapper->getSubstituted($this->getOption(self::OPT_INI_USERNAME_KEY)));;
		$this->log()->debug(__METHOD__ . ': ' . Hpls_Debug::print_r($userEntry, 'USER ENTRY'));
		if (!is_null($userEntry) && is_array($userEntry)) { //found entry for username -> check password
			if (isset($userEntry[$this->getOption(self::OPT_INI_PASSWORT_KEY)])) {
				$pass = trim($userEntry[$this->getOption(self::OPT_INI_PASSWORT_KEY)]);
				$givenPass = $this->getPassword();
				$this->log()->debug(__METHOD__ . ": ORIGINAL '$pass' WITH '$givenPass'");
				$res = Hpls_Utils::comparePassword($givenPass, $pass, $this->getOption(self::OPT_CRYPTMETH));
				if ($res) {
					$this->log()->debug(__METHOD__ . ': PASSWORD PASSED');
					//password equal
					$code = Zend_Auth_Result::SUCCESS;
					//					$roles = isset($userEntry[$this->iniRolesKey]) ? $this->parseRoles($userEntry[$this->iniRolesKey]) : array();
					$roles = $this->getRoles();
					$identity = array(
							'source' => $this->_sourceName,
							'username' => $this->getUsername(),
							'roles' => $roles );
				} elseif (is_null($res)) {
					$this->log()->err(__METHOD__ . ' Error while comparing passwords.');
					$code = Zend_Auth_Result::FAILURE_UNCATEGORIZED;
				} else {
					$code = Zend_Auth_Result::FAILURE_CREDENTIAL_INVALID;
				}
			} else {
				// no password entry
			}
		} else {
			$code = Zend_Auth_Result::FAILURE_IDENTITY_NOT_FOUND;
		}
		isDebug('auth') && $this->log()->debug(__METHOD__ . ': ' . Hpls_Debug::print_r($identity, 'IDENTITY'));
		return new Zend_Auth_Result($code, $identity, $msgs);
	}

	function setFile($filename) {
		$this->setOption(self::OPT_FILE, $filename);
		$cd = stripos($filename, DIRECTORY_SEPARATOR);
		if ($filename instanceof Zend_Config) {
			$this->_userIni = $filename->toArray();
		} elseif (is_array($filename)) {
			$this->_userIni = $filename;
		} elseif (0 === $cd) {
			$ini = new Zend_Config_Ini($filename);
			$this->_userIni = $ini->toArray();
		} else if (false === $cd) { //das ist gefährlich
			$ini = Hpls_Utils::readConfigIniCascade($filename);
			$this->_userIni = $ini->toArray();
		} else {
			//relative file
		

		}
	}

	function setMethod($method) {
		$this->setOption(self::OPT_CRYPTMETH, strtolower($method));
	}

}