<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 13.09.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Zend/Auth/Adapter/Ldap.php';
require_once 'Hpls/Auth/Adapter/Abstract.php';
require_once 'Zend/Ldap.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hpls_Auth_Adapter_Ldap extends Hpls_Auth_Adapter_Abstract {
	
	const OPT_IFMATCHED = 'ifMatched';
	const OPT_PROXYBIND = 'proxybind';
	const OPT_BIND_DN = 'dn';
	/**
	 * @todo comment
	 * @var Zend_Auth_Adapter_Ldap
	 */
	protected $_zendLdapAuth = null;
	protected $_ldap = null;

	public function setOptions($options) {
		if (!isset($options['bind'])) {
			//@todo entweder fehler oder fallback auf proxy
		}
		if (isset($options[self::OPT_PROXYBIND])) {
			
			if (!is_array($options[self::OPT_PROXYBIND]) || !isset($options[self::OPT_PROXYBIND][self::OPT_BIND_DN])) {
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration(" 'dn' is missing in 'proxybind' in source '" . $this->getSourceName() . "'", Hpls_Err::ERR_AUTH_CONFIGURATION_DN_MISSING);
			}
			if (!isset($options[self::OPT_PROXYBIND][self::OPT_PASSWORD])) {
				$options[self::OPT_PROXYBIND][self::OPT_PASSWORD] = '';
			}
		}
		parent::setOptions($options);
	}

	/* (non-PHPdoc)
	 * @see Zend_Auth_Adapter_Interface::authenticate()
	 */
	public function authenticate() {
		// TODO Auto-generated method stub
		$code = Zend_Auth_Result::FAILURE;
		$identity = null;
		$msgs = array();
		
		$this->log()->debug(__METHOD__);
		if ($this->isProxyUser()) {
			$code = $this->authByProxyuser();
			return $this->authByProxyuser();
		} else {
			if (is_null($this->_zendLdapAuth)) {
				$this->_zendLdapAuth = new Zend_Auth_Adapter_Ldap($this->getZendLdapOptions(), $this->getUsername(), $this->getPassword());
			}
			$identity = $this->generateIdentity();
			$result = $this->_zendLdapAuth->authenticate();
			
			$messages = $result->getMessages();
			foreach ($messages as $i => $message) {
				if ($i-- > 1) { // $messages[2] und höher sind Log Nachrichten
					$message = str_replace("\n", "\n  ", $message);
					$this->log()->debug("Ldap: $i: $message");
				}
			}
			if ($result->getCode() === Zend_Auth_Result::SUCCESS) {
				return new Zend_Auth_Result(Zend_Auth_Result::SUCCESS, $identity, $messages);
			} else {
				return new Zend_Auth_Result($code, $identity, $messages);
			}
		}
		
		
	}

	/**
	 * @todo comment
	 * @throws Hpls_Exception_Configuration
	 * @return Ambigous <string, number>|number|string
	 */
	protected function authByProxyuser() {
		$this->log()->debug(__METHOD__);
		
		//BindDn mit Proxy
		$res = $this->bindByProxy();
		if ($res !== Zend_Auth_Result::SUCCESS) {
			return $res;
		
		//			return $res;
		}
		//User-BaseDn aufspüren
		$ldap = $this->getLdap();
		$mode = 'or';
		$list = null;
		$bindOpts = $this->getOption(self::OPT_PROXYBIND, null);
		if (isset($bindOpts['or'])) {
			$list = $bindOpts['or'];
		} elseif (isset($bindOpts['and'])) {
			$list = $bindOpts['and'];
		} else {
			$list = $bindOpts;
		}
		$userBaseDn = null;
		if (isset($list[self::OPT_IFMATCHED])) {
			if (!is_array($list[self::OPT_IFMATCHED]))
				$list[self::OPT_IFMATCHED] = array(
						$list[self::OPT_IFMATCHED] );
			foreach ($list[self::OPT_IFMATCHED] as $listItem) {
				//suchen
				if (empty($listItem['filter'])) {
					$this->log()->warn(Hpls_Debug::print_r($listItem,"Empty LDAP filter not allowed"));
					continue;
				}
				try {
					$this->log()->debug(Hpls_Debug::print_r($listItem,__METHOD__.' ifMatched'));
					$filter = $this->getSubstituted($listItem['filter']);
					$basedn = $this->getSubstituted($listItem['basedn']);
					$scope = isset($listItem['scope']) ? $listItem['scope'] : null;
					$this->log()->debug(Hpls_Debug::print_r($filter,__METHOD__.' FILTER'));
					$this->log()->debug(Hpls_Debug::print_r($basedn,__METHOD__.' BASEDN'));
					$this->log()->debug(Hpls_Debug::print_r($scope,__METHOD__.' SCOPE'));
					//					echo "FILTER ";
					//					var_dump($filter);
					//					echo "BASEDN ";
					//					var_dump($basedn);
					//					echo "SCOPE ";
					//					var_dump($scope);
					$res = $ldap->search($filter, $basedn, $this->getSearchScope($scope));
					$this->log()->debug(Hpls_Debug::print_r($res, __METHOD__ . " RES"));
					$identity = $this->generateIdentity();
					
					if ($res->count() === 1) { //es dürfte nur ein Entry sein
						foreach ($res as $item) {
							$this->log()->debug(__METHOD__ . " USR: " . $item['dn'] . ":" . $this->getPassword());
							$ldap->bind($item['dn'], $this->getPassword()); //wenn's nicht klappt -> exception
							return new Zend_Auth_Result(Zend_Auth_Result::SUCCESS, $identity);
						}
					} elseif ($res->count() > 1) {
						//mehr als ein Entry -> LDAP-Filter nicht eindeutig genug
						return new Zend_Auth_Result(Zend_Auth_Result::FAILURE_IDENTITY_AMBIGUOUS, null);
					}
				} catch (Zend_Ldap_Exception $ex) {
					switch ($ex->getCode()) {
						//böse exceptions: ins log schreiben, vielleicht mehr
						case Zend_Ldap_Exception::LDAP_AUTH_METHOD_NOT_SUPPORTED :
						case Zend_Ldap_Exception::LDAP_CONNECT_ERROR :
						case Zend_Ldap_Exception::LDAP_INVALID_DN_SYNTAX :
						case Zend_Ldap_Exception::LDAP_NO_MEMORY :
						case Zend_Ldap_Exception::LDAP_NO_SUCH_OBJECT :
						case Zend_Ldap_Exception::LDAP_NOT_SUPPORTED :
						case Zend_Ldap_Exception::LDAP_SERVER_DOWN :
						case Zend_Ldap_Exception::LDAP_PROTOCOL_ERROR :
							$this->log()->err(__METHOD__ . ": $ex");
							return new Zend_Auth_Result(Zend_Auth_Result::FAILURE, null, array(
									$ex->getMessage() ));
						default :
					
		//nichts ausgeben, da ganz normale fehler und 
					}
				}
			}
		} else {
			require_once 'Hpls/Exception/Configuration.php';
			throw new Hpls_Exception_Configuration('At least one "<ifMatched>" is missing in //server/proxybind/* of auth source: ' . $this->getSourceName() . ' in "auth.xml".');
		}
		
		//mit user-dn binden
		if (empty($userBaseDn)) {
			return new Zend_Auth_Result(Zend_Auth_Result::FAILURE_IDENTITY_NOT_FOUND, array());
		}
		return new Zend_Auth_Result(Zend_Auth_Result::FAILURE_UNCATEGORIZED, array());
	}

	function generateIdentity(array $optional = array()) {
		$roles = $this->getRoles();
		$ret = array(
				'username' => $this->getUsername(),
				'source' => $this->getSourceName(),
				'roles' => $roles );
		return array_merge($ret, $optional);
	}
	
	/**
	 * @todo comment
	 * @return number|Zend_Auth_Result
	 */
	protected function bindByProxy() {
		try {
			$ldap = $this->getLdap();
			$ldap->bind($this->getAuthUsername(), $this->getAuthPassword());
			return Zend_Auth_Result::SUCCESS;
		
		} catch (Zend_Ldap_Exception $ex) {
			echo $ex;
			//			var_dump($ex);
			$code = Zend_Auth_Result::FAILURE_UNCATEGORIZED;
			switch ($ex->getCode()) {
				case Zend_Ldap_Exception::LDAP_INVALID_CREDENTIALS :
					$code = Zend_Auth_Result::FAILURE_CREDENTIAL_INVALID;
					break;
				case Zend_Ldap_Exception::LDAP_AUTH_UNKNOWN :
					$code = Zend_Auth_Result::FAILURE_IDENTITY_NOT_FOUND;
					break;
			
		//				case Zend_Ldap_Exception::LDAP_STRONG_AUTH_REQUIRED :
			//				case Zend_Ldap_Exception::LDAP_AUTH_METHOD_NOT_SUPPORTED :
			//					return Zend_Auth_Result::FAILURE_UNCATEGORIZED;
			}
			return new Zend_Auth_Result($code, null, array(
					$ex->getMessage() ));
		}
	}

	/**
	 * @todo comment
	 * @return array
	 */
	protected function getZendLdapOptions() {
		
		$ret = array(
				'host' => $this->getOption('host'),
				'port' => $this->getOption('port', 389)
				);
		if ($this->getOption('type') == 'ad') {
			switch ($this->getOption('ssl', 'starttls')) {
				case 'starttls' :
					$ret['useStartTls'] = true;
					break;
				case 'ssl' :
					$ret['useSsl'] = true;
					break;
				default :
					break;
			}
		
		//			$ret['baseDn'] =
		/*
; Typical options for Active Directory
ldap.server2.host = dc1.w.net
ldap.server2.useStartTls = true
ldap.server2.accountDomainName = w.net
ldap.server2.accountDomainNameShort = W
ldap.server2.accountCanonicalForm = 3
ldap.server2.baseDn = "CN=Users,DC=w,DC=net" */
		} else { //allgemeiner LDAP
			switch ($this->getOption('ssl', 'no')) {
				case 'starttls' :
					$ret['useStartTls'] = true;
					break;
				case 'ssl' :
					$ret['useSsl'] = true;
					break;
				default :
					break;
			}
			$ret['accountCanonicalForm'] = 1;
			$ret['bindRequiresDn'] = true;
			// $ret['baseDn'] = 'uid=c.bauer@jpberlin.de,dc=jpberlin.de,uid=jpberlin,ou=user,o=jpberlin,c=de';
			
		/*
	 ; Typical options for OpenLDAP
ldap.server1.host = s0.foo.net
ldap.server1.accountDomainName = foo.net
ldap.server1.accountDomainNameShort = FOO
ldap.server1.accountCanonicalForm = 3
ldap.server1.username = "CN=user1,DC=foo,DC=net"
ldap.server1.password = pass1
ldap.server1.baseDn = "OU=Sales,DC=foo,DC=net"
ldap.server1.bindRequiresDn = true
 */
		}
		$this->log()->info(Hpls_Debug::print_r($ret, __METHOD__ . ' ZFLdap Options'));
		return array(
				$this->getSourceName() => $ret );
	}

	public function getUsername() {
		//		$this->getLoginMapper()->getSubstituted($this->getOption('bind',''),parent::getUsername());
		

		$bind = $this->getOption('bind', null);
		return $this->getLoginMapper()->getSubstituted($bind[self::OPT_BIND_DN], parent::getUsername());
	}

	public function getAuthUsername() {
		if ($this->isProxyUser()) {
			$proxy = $this->getOption(self::OPT_PROXYBIND, null);
			return $this->getLoginMapper()->getSubstituted($proxy[self::OPT_BIND_DN], parent::getUsername());
		} else {
			return parent::getAuthUsername();
		}
	}

	public function getAuthPassword() {
		if ($this->isProxyUser()) {
			$proxy = $this->getOption(self::OPT_PROXYBIND, null);
			return Hpls_Utils::getClearPassword($proxy[self::OPT_PASSWORD]);
		} else {
			return parent::getAuthPassword();
		}
	}

	protected function isProxyUser() {
		$bind = $this->getOption(self::OPT_PROXYBIND, null);
		return ($bind !== null);
	}

	/**
	 * @todo comment
	 * @return Zend_Ldap
	 */
	protected function getLdap() {
		if (is_null($this->_ldap)) {
			$this->_ldap = new Zend_Ldap(reset($this->getZendLdapOptions()));
		}
		return $this->_ldap;
	}

	protected function getSearchScope($text) {
		switch (strtolower($text)) {
			case 'one' :
				return Zend_Ldap::SEARCH_SCOPE_ONE;
			case 'sub' :
				return Zend_Ldap::SEARCH_SCOPE_SUB;
			
			case 'base' :
			default :
				return Zend_Ldap::SEARCH_SCOPE_BASE;
		}
	}
}