<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 26.05.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
abstract class Hpls_Auth_Reader_RolesAbstract {
	protected $_source;
	protected $_options;
	/**
	 * @todo comment
	 * @var Hpls_Var_Matcher
	 */
	protected $_loginMapper = null;

	public function __construct($source, array $options) {
		$this->_source = $source;
		$this->setOptions($options);
	}

	/**
	 * @todo comment
	 * @param string $username
	 * @return array Wenn kein Ergebnis, dann leeres Array. <- Ergebnis ist also in jedem Fall ein Array.
	 */
	abstract function readRoles($username);

	function setPassword($password) {
		$this->_options[Hpls_Consts0::OPT_PASSWORD] = trim($password);
	}

	function setUsername($username) {
		$this->_options[Hpls_Consts0::OPT_USERNAME] = trim($username);
		$this->_loginMapper->setVariable('LOGINNAME', $this->_options[Hpls_Consts0::OPT_USERNAME]);
	}

	function getUsername() {
		return isset($this->_options[Hpls_Consts0::OPT_USERNAME]) ? $this->_options[Hpls_Consts0::OPT_USERNAME] : NULL;
	}

	function getPassword() {
		return isset($this->_options[Hpls_Consts0::OPT_PASSWORD]) ? $this->_options[Hpls_Consts0::OPT_PASSWORD] : NULL;
	}

	public function addFilter($filterString) {
		if (!isset($this->_options['filters']) || !is_array($this->_options['filters'])) {
			if (!is_array($this->_options['filters'])) {
				$this->_options['filters'] = array(
						$this->_options['filters'] );
			} else {
				$this->_options['filters'] = array();
			}
		
		}
		$this->_options['filters'][] = $filterString;
	}

	public function setSearchBase($searchBase) {
		$this->_options['searchbase'] = $searchBase;
	}

	public function setOptions(array $options) {
		$options = array_change_key_case($options, CASE_LOWER);
		if (!empty($options[Hpls_Consts0::OPT_LOGINMAPPING])) {
			$t = $options[Hpls_Consts0::OPT_LOGINMAPPING];
			if ($t instanceof Hpls_Var_Matcher) {
				$this->_loginMapper = $t;
			} elseif (is_array($t)) {
				$this->_loginMapper = new Hpls_Var_Matcher(array(
						'match' => $t ));
			}
		} else {
			//default mapper erzeugen
			$this->_loginMapper = Hpls_Var_Matcher::createLoginMapper();
		}
		
		$this->_options = $options;
	}

	public function getOption($name) {
		return isset($this->_options[$name]) ? $this->_options[$name] : null;
	}
	
	protected function log() {
		return Hpls_Utils::log();
	}
}