<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 19.05.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-08-14 21:14:28 +0200 (Sun, 14 Aug 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 8635 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/branches/for_20110120_performance/hpls.ac.core2.gui.common/Hpls/User/Persistence.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Utils/Logusing.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 8635 $
 * @since 1.0.9
 * @deprecated wir über Hpls_Auth_Source erledigt
 */
class Hpls_Auth_Reader_Userdata extends Hpls_Utils_Logusing {
	const TYPE_LDAP = 'ldap';
	const TYPE_INI = 'ini';
	const TYPE_DB = 'db';
	const TYPE_LOGIN = 'login';
	
	protected $SUPPORTED_TYPES = array(
			self::TYPE_LDAP,
			self::TYPE_INI,
			self::TYPE_DB );
	
	protected static $_instance = null;
	
	protected $_config = null;

	/**
	 * @return Hpls_Auth_Reader_Userdata
	 */
	public static function i() {
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	protected function __construct() {
	}

	/**
	 * Liest aus der in der Auth-Config (auth.xml) definierten E-Mail-Quellen zum aktuellenen Authentifizierungssource die E-Mail-Adressen
	 * zum übergebenen User-Objekt.
	 * @param Hpls_User_Object $userObj
	 * @throws Hpls_Exception_NotSupported
	 */
	public function readInEmails(Hpls_User_Object $userObj) {
		isDebug('user_email_read') && $this->log()->info(__METHOD__ . "(" . $userObj->getName() . ") START");
		$cfgs = $this->getConfigEmailsRead();
		isDebug('user_email_read') && $this->log()->debug(Hpls_Debug::print_r($cfgs, __METHOD__ . ' CFGS'));
		
		$lasttype = null;
		foreach ($this->getActiveSourcesSpecs() as $cfg) {
			//			$this->log()->debug(Hpls_Debug::print_r($cfg, __METHOD__ . ' CFG'));
			$type = ucfirst(strtolower($cfg->type));
			$classname = 'Hpls_Auth_Reader_Userdata_' . $type;
			$classfilename = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'Userdata' . DIRECTORY_SEPARATOR . $type . '.php';
			if (!file_exists($classfilename)) {
				require_once 'Hpls/Exception/NotSupported.php';
				throw new Hpls_Exception_NotSupported('Storage type ' . $type . ' in config file "' . Hpls_Consts0::FILE_CONFIG_AUTH . '" is unknown under "' . $classfilename . '".', -1);
			}
			require_once $classfilename;
			
			/**
			 * @var Hpls_Auth_Reader_UserdataAbstract
			 */
			$reader = new $classname($cfg);
			isDebug('user_email_read') && $this->log()->debug(Hpls_Debug::print_r($reader, __METHOD__ . ' READER'));
			$ownEmails = $reader->getManagedEmails($userObj->getName());
			isDebug('user_email_read') && $this->log()->debug(Hpls_Debug::print_r($ownEmails, __METHOD__ . ' READ EMAILS'));
			if (!empty($ownEmails)) {
				$userObj->addEmail($ownEmails);
			}
			if (!empty($ownEmails) && isset($cfg['continue']) && !Hpls_Utils::toBool($cfg['continue'])) {
				break;
			}
		}
		isDebug('method_trace', 'user_email_read') && $this->log()->info(__METHOD__ . "(" . $userObj->getName() . ") END");
	}
	
	public function readInRoles(Hpls_User_Object $usrObj) {
		$ini = $this->getAuthConfig();
		$roles = array();
		foreach ($this->getActiveSourcesSpecs() as $src) {
			isDebug('auth', 'auth_roles') && $this->log()->debug(__METHOD__ . ": source: $src" . Hpls_Debug::print_r($ini));
			if (isset($ini->{$src})) {
				if (isset($ini->{$src}->roles)) {
					//hinzufügen
					//entfernen
					//setzen
					if (isset($ini->{$src}->roles->add)) {
						$rolesAdd = explode(',', $ini->{$src}->roles->add);
						isDebug('auth', 'auth_roles') && $this->log()->debug(__METHOD__ . ": source set: $src" . Hpls_Debug::print_r($rolesAdd, 'rolesAdd'));
						foreach ($rolesAdd as $rAdd) {
							$rAdd = strtolower(trim($rAdd));
							$roles[$rAdd] = $rAdd;
						}
					} // if ->roles->add?
					//@todo reading and mapping roles from source
					if (isset($ini->{$src}->roles->source)) {
						/**
						 * @var $reader Hpls_Auth_Reader_RolesAbstract
						 */
						$readers = $this->getSourceReader($ini->{$src});
						//es kann mehrere RoleSources geben...
						foreach ($readers as $reader) {
							foreach ($reader->readRoles() as $rAdd) {
								$roles[$rAdd] = $rAdd; //jede Rolle nur 1x
							}
						}
					} // if ->roles->source?
				} //if ->roles?
			} //if ->{$src}?
		}
		return $roles;
	}
	
	public function readInDomains(Hpls_User_Object $usrObj) {
		
	}

	protected function getConfigEmailsRead() {
		if (is_null($this->_config)) {
			$ini = Zend_Registry::get(Hpls_Consts0::REGK_AUTH);
			isDebug('user_email_read') && $this->log()->debug(Hpls_Debug::print_r($ini->toArray(), __METHOD__ . ' CFGS'));
			if (isset($ini->{Hpls_Auth::i()->getAuthSource()})) {
				$this->_config = $ini->{Hpls_Auth::i()->getAuthSource()}->toArray();
			} else {
				$this->_config = null;
			}
		
		}
		return is_null($this->_config) ? null : $this->_config['email'];
	}
	
	protected function getActiveSourcesSpecs() {
		
	}
}