<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 27.04.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-06-20 16:48:04 +0200 (Mo, 20 Jun 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 8510 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/branches/for_20110120_performance/hpls.ac.core2.gui.common/Hpls/User/PersistenceInterface.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Storage/ReaderAbstract.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 8510 $
 * @since 1.0.9
 */
abstract class Hpls_Auth_Reader_UserdataAbstract extends Hpls_Utils_Logusing implements 
	Hpls_Storage_ReaderInterface {
	protected $_options = null;
	protected $_filter = array();
	protected $_name = null;
	/**
	 * @todo comment
	 * @var Hpls_Var_Matcher
	 */
	protected $_userMapper = null;
	protected $_authCredentials = array();

	function __construct($name, $options = array(), $userMapper = null) {
		$this->_name = $name;
		$this->_userMapper = $userMapper; //unbedingt vor setOptions(), da er dort überschrieben werden darf und kann
		$this->setOptions($options);
	}

	/**
	 * @todo comment
	 * @param string $username
	 * @throws Hpls_Exception_Configuration
	 */
	abstract public function getManagedDomains($username = null);

	/**
	 * @todo comment
	 * @param string $username
	 * @return array|array(0)|null <ul><li>null, wenn Fehler beim Auslesen entstanden,<li>leeres array(0), wenn keine E-Mails gefunden wurden, <li>sonst, Liste von E-Mail-Adressen</ul>
	 * @throws Hpls_Exception_Configuration
	 */
	abstract public function getManagedEmails($username = null);

	/**
	 * @todo comment
	 * @return mixed
	 */
	abstract public function getData($username = null);

	/**
	 * Checkt die per setOptions() gesetzten Optionen auf Gültigkeit und fügt evtl. Default-Werte ein.
	 * @throws Hpls_Exception_Configuration
	 */
	abstract public function checkOptions();

	public function setOptions($options) {
		$this->_options = array_change_key_case($options, CASE_LOWER);
		if (!empty($options[Hpls_Consts0::OPT_LOGINMAPPING])) {
			$t = $options[Hpls_Consts0::OPT_LOGINMAPPING];
			if ($t instanceof Hpls_Var_Matcher) {
				$this->_userMapper = $t;
			} elseif (is_array($t)) {
				$this->_userMapper = new Hpls_Var_Matcher(array(
						'match' => $t ));
			}
		} else {
			//default mapper belassen
		}
		
		$this->checkOptions();
	}

	function getOption($key, $default = null) {
		if (isset($this->_options[$key])) {
			return $this->_options[$key];
		}
		return $default;
	}

	function addFilter($filter) {
		$this->_filter[] = $filter;
	}

	function setAuth($user, $pass) {
		$this->_authCredentials['username'] = $user;
		$this->_authCredentials['password'] = $pass;
	}

	function getFilter($type = null) {
		if (empty($type)) {
			return $this->_filter;
		}
		$ret = array();
		foreach ($this->_filter as $filter) {
			if (strcasecmp($filter['type'], $type) == 0) {
				$ret[] = $filter;
			}
		}
		return $ret;
	}

	/**
	 * Der Name mit dem sich an der Quelle angemeldet wird.
	 * @return Ambigous <multitype:, NULL>
	 */
	function getAuthUsername() {
		return isset($this->_authCredentials[Hpls_Consts0::OPT_USERNAME]) ? $this->_authCredentials[Hpls_Consts0::OPT_USERNAME] : (isset($this->_options[Hpls_Consts0::OPT_USERNAME]) ? $this->_options[Hpls_Consts0::OPT_USERNAME] : null);
	}

	function getAuthPassword() {
		return isset($this->_authCredentials['password']) ? $this->_authCredentials['password'] : (isset($this->_options['password']) ? $this->_options['password'] : null);
	}

	function setAuthPassword($pass) {
		$this->_authCredentials['password'] = $pass;
	}

	/**
	 * Setzt den Usernamen, nach dem gesucht wird
	 * @param unknown_type $username
	 */
	function setUsername($username) {
		$this->_options[Hpls_Consts0::OPT_USERNAME] = $username;
		$this->getUsernameMapper()->setVariable(Hpls_Consts0::AUTH_VAR_LOGIN, $username);
	}
	
	function setPassword($password) {
		$this->_options[Hpls_Consts0::OPT_PASSWORD] = $password;
		$this->getUsernameMapper()->setVariable(Hpls_Consts0::AUTH_VAR_PASSWORD, $password);
	}

	function getFilterOption($filterIdx, $optionKey, $default = null) {
		if (isset($this->_filter[$filterIdx]) && isset($this->_filter[$filterIdx][$optionKey])) {
			return $this->_filter[$filterIdx][$optionKey];
		}
		
		return $default;
	}

	protected function getConfigPathString($path) {
		return (string) $path;
	}

	/**
	 * @todo IMPLEMENT
	 */
	protected function applyValueFilter($searchFilter, &$value, $key = null) {
		return $value;
	}

	/**
	 * @todo comment
	 * @return Hpls_Var_Matcher
	 */
	protected function getUsernameMapper() {
		if (is_null($this->_userMapper)) {
			$usermap = $this->getOption('usermap', null);
			if (!empty($usermap) && isset($usermap['match'])) {
				$this->_userMapper = new Hpls_Var_Matcher($usermap);
			} else {
				$this->_userMapper = new Hpls_Var_Matcher(null);
			}
		}
		return $this->_userMapper;
	}

}