<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 01.09.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Auth/Source.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Zend/Validate/EmailAddress.php';
require_once 'Hpls/Utils/Logusing.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.9
 */
class Hpls_Auth_Source extends Hpls_Utils_Logusing {
	protected $_config = null;
	protected $_name = null;
	/**
	 * @todo comment
	 * @var Hpls_Var_Matcher
	 */
	protected $_loginmapping = null;
	
	protected $_emailReader = null;
	protected $_roleReader = null;
	protected $_authAdapter = null;

	public function __construct($name, $config, $loginmapping) {
		$this->setConfig($config);
		$this->_name = $name;
		$this->_loginmapping = $loginmapping;
	}

	protected function setConfig($config) {
		
		if ($config instanceof Zend_Config) {
			$config = $config->toArray();
		} elseif (!is_array($config)) {
			//			require_once 'Hpls/Exception/Configuration.php';
			throw new InvalidArgumentException('$config is not a configuration.');
		}
		
		//check required
		if (!isset($config['type'])) {
			require_once 'Hpls/Exception/Configuration.php';
			throw new Hpls_Exception_Configuration('No source type specified', Hpls_Consts0::ERR_AUTH_NOSOURCETYPE);
		}
		if (!isset($config['server'])) {
			require_once 'Hpls/Exception/Configuration.php';
			throw new Hpls_Exception_Configuration('No server specified', Hpls_Consts0::ERR_AUTH_NOSERVER);
		}
		if (!isset($config['continue'])) {
			$config['continue'] = true;
		}
		if (isset($config['loginmapping'])) {
			//source-lokales loginmapping, überschreibt das, welches im konstruktor übergeben wurde
		//@todo als Hpls_Var_Matcher umsetzen
		}
		//		if (!isset($config['loginmapping'])) {
		//			require_once 'Hpls/Exception/Authentication.php';
		//			throw new Hpls_Exception_Authentication('No loginmapping provided',Hpls_Consts0::ERR_AUTH_NOSERVER);
		//			
		//		}
		$this->_config = $config;
	}

	public function getType() {
		return $this->_config['type'];
	}

	public function getName() {
		return $this->_name;
	}

	/**
	 * @todo comment
	 * @throws Hpls_Exception_NotSupported
	 * @return NULL|Zend_Auth_Adapter_Interface 
	 */
	public function getAuthAdapter() {
		if (is_null($this->_authAdapter)) {
			$type = ucfirst(strtolower(trim($this->_config['type'])));
			$filename = "Hpls/Auth/Adapter/$type.php";
			$classname = "Hpls_Auth_Adapter_$type";
			$filepath = Hpls_Utils::file_exists_searchpath($filename);
			if ($filepath === false) {
				require_once 'Hpls/Exception/NotSupported.php';
				throw new Hpls_Exception_NotSupported("There is no support for type '" . $this->_config['type'] . "': no file '$filename' found.", Hpls_Consts0::ERR_AUTH_NOTSUPPORTED);
			}
			require_once $filepath . DIRECTORY_SEPARATOR . $filename;
			
			$this->_config['server']['type'] = $this->_config['type'];
			/**
			 * @var  Hpls_Auth_Adapter_Abstract
			 */
			$this->_authAdapter = new $classname($this->getName(), $this->_config['server']);
		
		}
		return $this->_authAdapter;
	}

	public function isContinue() {
		return $this->_config['continue'];
	}

	public function readRoles($username, $currentRoles = array()) {
		$ini = $this->_config;
		
		isDebug('auth', 'auth_roles') && $this->log()->debug(__METHOD__ . ": source: " . $this->getName() . Hpls_Debug::print_r($ini));
		
		if (isset($ini['roles'])) {
			//hinzufügen
			if (isset($ini['roles']['add'])) {
				
				$rolesAdd = $ini['roles']['add'];
				if (!is_array($rolesAdd)) {
					$rolesAdd = preg_split('/\s*,\s*/', $rolesAdd);
				}
				isDebug('auth', 'auth_roles') && $this->log()->debug(__METHOD__ . ": source set: " . $this->getName() . Hpls_Debug::print_r($rolesAdd, 'rolesAdd'));
				foreach ($rolesAdd as $rAdd) {
					$rAdd = strtolower(trim($rAdd));
					$currentRoles[$rAdd] = $rAdd;
				}
			} // if ->roles->add?
			//entfernen
			if (isset($ini['roles']['remove'])) {
				$rolesAdd = $ini['roles']['remove'];
				if (!is_array($rolesAdd)) {
					$rolesAdd = preg_split('/\s*,\s*/', $rolesAdd);
				}
				isDebug('auth', 'auth_roles') && $this->log()->debug(__METHOD__ . ": source set: " . $this->getName() . Hpls_Debug::print_r($rolesAdd, 'rolesRemove'));
				foreach ($rolesAdd as $rAdd) {
					$rAdd = strtolower(trim($rAdd));
					unset($currentRoles[$rAdd]);
				}
			} // if ->roles->remove?
			//setzen
			if (isset($ini['roles']['set'])) {
				$currentRoles = array();
				$rolesAdd = $ini['roles']['set'];
				if (!is_array($rolesAdd)) {
					$rolesAdd = preg_split('/\s*,\s*/', $rolesAdd);
				}
				isDebug('auth', 'auth_roles') && $this->log()->debug(__METHOD__ . ": source set: " . $this->getName() . Hpls_Debug::print_r($rolesAdd, 'rolesSet'));
				foreach ($rolesAdd as $rAdd) {
					$rAdd = strtolower(trim($rAdd));
					$currentRoles[$rAdd] = $rAdd;
				}
			} // if ->roles->set?
			//@todo reading and mapping roles from source
			if (isset($ini['roles']['source'])) {
				isDebug('auth', 'auth_roles') && $this->log()->debug(Hpls_Debug::print_r($ini['roles']['source'], __METHOD__ . " ROLE READERS cfg"));
				/**
				 * @var $reader Hpls_Auth_Reader_RolesAbstract
				 */
				$readers = $this->getRoleReaders($ini['roles']['source']);
				isDebug('auth', 'auth_roles') && $this->log()->debug(Hpls_Debug::print_r($readers, __METHOD__ . " ROLE READERS"));
				//es kann mehrere RoleSources geben...
				foreach ($readers as $reader) {
					foreach ($reader->readRoles($username) as $rAdd) {
						$currentRoles[$rAdd] = $rAdd; //jede Rolle nur 1x
					}
				}
			} // if ->roles->source?
		} //if ->roles?
		

		return $currentRoles;
	
	}

	/**
	 * @todo comment
	 * @param Hpls_User_Object $userObj
	 * @throws Hpls_Exception_Configuration
	 * @throws Hpls_Exception_NotSupported
	 * @return array:string|NULL
	 */
	public function readEmails(Hpls_User_Object $userObj) {
		isDebug('user_email_read') && $this->log()->info(__METHOD__ . "(" . $userObj->getName() . ") START");
		$cfgs = $this->_config;
		
		isDebug('user_email_read') && $this->log()->debug(Hpls_Debug::print_r($cfgs, __METHOD__ . ' CFGS'));
		
		//emails überhaupt konfiguriert?
		if (!isset($cfgs['emails']['source']) || !is_array($cfgs['emails']['source'])) {
			//nein
			//loginname == email -> dann nur diese als email und fertig
			$val = new Zend_Validate_EmailAddress();
			if ($val->isValid($userObj->getName())) {
				isDebug('user_email_read') && $this->log()->info("Using username as email: " . $userObj->getName());
				return array(
						$userObj->getName() => $userObj->getName() );
			}
			$this->log()->warn('No email-source for auth-source "' . $this->getName() . '" defined.');
			return null;
		}
		
		$lasttype = null;
		//			$this->log()->debug(Hpls_Debug::print_r($cfg, __METHOD__ . ' CFG'));
		$ret = array();
		$this->log()->debug(Hpls_Debug::print_r($cfgs, __METHOD__));
		if (isset($cfgs['emails']['source']['type'])) { //wenn nur 1 source gegeben ist
			$cfgs['emails']['source'] = array(
					$cfgs['emails']['source'] );
		}
		foreach ($cfgs['emails']['source'] as $cfg) {
			$this->log()->debug(Hpls_Debug::print_r($cfg, __METHOD__));
			if (!isset($cfg['type'])) {
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration('No type is set for email-source in auth-source "' . $this->getName() . '" in config file "' . Hpls_Consts0::FILE_CONFIG_AUTH . '"', Hpls_Consts0::ERR_AUTH_EMAILS_NOSOURCETYPE);
			}
			if (!isset($cfg['server']) || !is_array($cfg['server'])) {
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration('No server is set for email-source in auth-source "' . $this->getName() . '" in config file "' . Hpls_Consts0::FILE_CONFIG_AUTH . '"', Hpls_Consts0::ERR_AUTH_EMAILS_NOSOURCETYPE);
			
			}
			$type = ucfirst(strtolower($cfg['type']));
			$classname = 'Hpls_Auth_Reader_Userdata_' . $type;
			$classfilename = 'Hpls/Auth/Reader/Userdata/' . $type . '.php';
			if (!Hpls_Utils::file_exists_searchpath($classfilename)) {
				require_once 'Hpls/Exception/NotSupported.php';
				throw new Hpls_Exception_NotSupported('Storage type ' . $type . ' in config file "' . Hpls_Consts0::FILE_CONFIG_AUTH . '" is unknown under "' . $classfilename . '".', -1);
			}
			require_once $classfilename;
			
			/**
			 * @var Hpls_Auth_Reader_UserdataAbstract
			 */
			$reader = new $classname($this->getName(), $cfg['server'], $this->getLoginMapper());
			isDebug('user_email_read') && $this->log()->debug(Hpls_Debug::print_r($reader, __METHOD__ . ' READER'));
			$ownEmails = $reader->getManagedEmails($userObj->getName());
			isDebug('user_email_read') && $this->log()->debug(Hpls_Debug::print_r($ownEmails, __METHOD__ . ' READ EMAILS'));
			if (!empty($ownEmails)) {
				$ret = array_merge($ret, $ownEmails);
				if (!isset($cfg['continue']) || !Hpls_Utils::toBool($cfg['continue'])) {
					break;
				}
			}
			isDebug('method_trace', 'user_email_read') && $this->log()->info(__METHOD__ . "(" . $userObj->getName() . ") END");
		}
		return $ret;
	}

	/**
	 * @todo comment
	 * @param unknown_type $srcCfg
	 * @throws Hpls_Exception_Configuration
	 * @throws Hpls_Exception_NotSupported
	 * @return multitype:unknown 
	 */
	protected function getRoleReaders($srcCfg) {
		$ret = array();
		if (is_array($srcCfg) && isset($srcCfg['type'])) {
			$srcCfg = array(
					$srcCfg );
		}
		foreach ($srcCfg as $roleSrc) {
			//falsche config?
			if (!isset($roleSrc['type']) || !isset($roleSrc['server'])) {
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration(__METHOD__);
			}
			//laden der PHP datei
			$type = ucfirst(strtolower(trim($roleSrc['type'])));
			isDebug('auth', 'auth_roles') && $this->log()->debug(__METHOD__ . " READER type: $type:" . $roleSrc['type']);
			$classname = "Hpls_Auth_Reader_Roles_$type";
			$filename = "Hpls/Auth/Reader/Roles/$type.php";
			if (Hpls_Utils::file_exists_searchpath($filename)) {
				isDebug('auth', 'auth_roles') && $this->log()->debug(__METHOD__ . " READER found: $filename:$classname");
				require_once $filename;
				$class = new $classname($this->getName(), $roleSrc['server']);
				$ret[] = $class;
			} else {
				require_once 'Hpls/Exception/NotSupported.php';
				throw new Hpls_Exception_NotSupported(" READER *NOT* found: $filename:$classname", Hpls_Consts0::ERR_NOTSUPPORTED);
			}
		}
		return $ret;
	}

	protected function getLoginMapper() {
		if (is_null($this->_loginmapping)) {
			$this->_loginmapping = Hpls_Var_Matcher::createLoginMapper();
		}
		return $this->_loginmapping;
	}

}