<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 20.08.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Config/Writer/Db.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Zend/Config/Writer.php';
require_once 'Zend/Db.php';
require_once 'Hpls/Utils/Db.php';

/**
 * @todo comment
 * Options:
 * <ul>
 * <li><b>dbname</b>
 * <li><b>type</b>
 * <li><b>host</b>
 * <li><b>port</b>
 * <li><b>username</b>
 * <li><b>password</b>
 * <li><b>keycol</b>
 * <li><b>valuecol</b>
 * <li><b>separator</b>
 * </ul>
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.9
 */
class Hpls_Config_Writer_Db extends Zend_Config_Writer {
	/**
	 * @todo comment
	 * @var Zend_Db_Adapter_Abstract
	 */
	protected $_db = null;
	protected $_options = null;
	protected $_toWrite = array();

	/* (non-PHPdoc)
	 * @see Zend_Config_Writer::write()
	 */
	public function write(Zend_Config $zconfig = null) {
		// TODO Auto-generated method stub
		if (!is_null($zconfig)) {
			$this->setConfig($zconfig);
		}
		
		$toWrite = $this->getToWrite();
		Hpls_Utils::log()->debug(__METHOD__ . Hpls_Debug::print_r($toWrite, " TOWRITE "));
		try {
			$this->getDb()->beginTransaction();
			$already = $this->getDbContents();
			$toDelete = array_diff_key($already, $toWrite);
			$toUpdate = array_intersect_key($toWrite, $already);
			$toInsert = array_diff_key($toWrite, $already);
			Hpls_Utils::log()->debug(__METHOD__ . Hpls_Debug::print_r($toDelete, " TODELETE ") . Hpls_Debug::print_r($toUpdate, " TOUPdate ") . Hpls_Debug::print_r($toInsert, " TOINSERT "));
			$this->update($toUpdate, $already);
			$this->insert($toInsert);
			$this->delete($toDelete);
			$this->getDb()->commit();
		} catch (Exception $ex) {
			Hpls_Utils::log()->err($ex);
			$this->getDb()->rollBack();
		}
	}

	public function setOptions($options) {
		if (!isset($options['nestSeparator'])) {
			$options['nestSeparator'] = '.';
		}
		if (!isset($options['keycol'])) {
			$options['keycol'] = 'key';
		}
		if (!isset($options['valuecol'])) {
			$options['valuecol'] = 'value';
		}
		if (!isset($options['tablename'])) {
			$options['tablename'] = 'config';
		}
		$this->_options = $options;
	}

	/**
	 * Set the nest separator
	 *
	 * @param  string $filename
	 * @return Zend_Config_Writer_Ini
	 */
	public function setNestSeparator($separator) {
		$this->_options['nestSeparator'] = $separator;
		
		return $this;
	}

	public function getNestSeparator() {
		return $this->_options['nestSeparator'];
	}

	protected function getDbContents() {
		$db = $this->getDb();
		$select = $db->select();
		$select->from(array(
				'c' => $this->getTablename() ), array(
				'k' => $this->_options['keycol'],
				'v' => $this->_options['valuecol'] ));
		$stm = $select->query(PDO::FETCH_ASSOC);
		$res = $stm->fetchAll();
		$ret = array();
		foreach ($res as $r) {
			$ret[$r['k']] = $r['v'];
		}
		return $ret;
	}

	protected function delete($toDelete) {
		if (empty($toDelete))
			return;
		$db = $this->getDb();
		$keys = array_keys($toDelete);
		$binds = array();
		$c = 0;
		$sql = 'DELETE FROM ' . $db->quoteIdentifier($this->getTablename()) . ' WHERE ' . $db->quoteIdentifier($this->_options['keycol']) . ' IN (';
		
		foreach ($keys as $k) {
			if ($c > 0) {
				$sql .= ',';
			}
			$name = 'n' . $c++;
			$sql .= $name;
			$binds[$name] = $k;
		}
		$sql .= ')';
		Hpls_Utils::log()->debug(__METHOD__ . Hpls_Debug::print_r($binds, " SQL: $sql"));
		$db->query($sql, $binds);
	}

	protected function update($toUpdate, $already) {
		if (empty($toUpdate))
			return;
		$db = $this->getDb();
		foreach ($toUpdate as $k => $v) {
			if (strcmp($v, $already[$k]) !== 0) {
				$res = $db->update($this->getTablename(), array(
						$this->_options['valuecol'] => $v ), array(
						$this->_options['keycol'] . ' = ?' => $k ));
				if ($res !== 1) {
					require_once 'Zend/Config/Exception.php';
					throw new Zend_Config_Exception("Could not update key '$k' to database.");
				}
			}
		}
	
	}

	protected function insert($toInsert) {
		if (empty($toInsert))
			return;
		$db = $this->getDb();
		
		foreach ($toInsert as $k => $v) {
			$res = $db->insert($this->getTablename(), array(
					$this->_options['keycol'] => $k,
					$this->_options['valuecol'] => $v ));
			if ($res !== 1) {
				require_once 'Zend/Config/Exception.php';
				throw new Zend_Config_Exception("Could not write (new) key '$k' to database.");
			}
		}
	}

	protected function getToWrite() {
		$extends = $this->_config->getExtends();
		$sectionName = $this->_config->getSectionName();
		return $this->_addBranch($this->_config);
	}

	/**
	 * Add a branch to an INI string recursively
	 *
	 * @param  Zend_Config $config
	 * @return void
	 */
	protected function _addBranch(Zend_Config $config, $parents = array()) {
		$ret = array();
		
		foreach ($config as $key => $value) {
			$group = array_merge($parents, array(
					$key ));
			
			if ($value instanceof Zend_Config) {
				$ret = array_merge($ret, $this->_addBranch($value, $group));
			} else {
				$ret[implode($this->getNestSeparator(), $group)] = $this->_prepareValue($value);
			}
		}
		
		return $ret;
	}

	/**
	 * Prepare a value for INI
	 *
	 * @param  mixed $value
	 * @return string
	 */
	protected function _prepareValue($value) {
		if (is_integer($value) || is_float($value)) {
			return $value;
		} elseif (is_bool($value)) {
			return ($value ? 'true' : 'false');
		} else
			return $value;
	}

	/**
	 * Root elements that are not assigned to any section needs to be
	 * on the top of config.
	 *
	 * @see    http://framework.zend.com/issues/browse/ZF-6289
	 * @param  Zend_Config
	 * @return Zend_Config
	 */
	protected function _sortRootElements(Zend_Config $config) {
		$configArray = $config->toArray();
		$sections = array();
		
		// remove sections from config array
		foreach ($configArray as $key => $value) {
			if (is_array($value)) {
				$sections[$key] = $value;
				unset($configArray[$key]);
			}
		}
		
		// readd sections to the end
		foreach ($sections as $key => $value) {
			$configArray[$key] = $value;
		}
		
		return new Zend_Config($configArray);
	}

	protected function getTablename() {
		return $this->_options['tablename'];
	}

	/**
	 * @todo comment
	 * @return Zend_Db_Adapter_Abstract
	 */
	protected function getDb() {
		if (is_null($this->_db)) {
			$this->_db = Hpls_Utils_Db::mapDriverType2ZendDb($this->_options);
		}
		return $this->_db;
	}
}