<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 12.05.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Constraint/Interface.php';
require_once 'Hpls/Utils/Logusing.php';
require_once 'Hpls/Msgs.php';

/**
 * Basisklasse für alle Constraint-Implementierungen
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
abstract class Hpls_Constraint_CommonAbstract extends Hpls_Utils_Logusing implements 
	Hpls_Constraint_Interface {
	
	const OPT_FIELDDEPENDENCY = 'field_dependency';
	const OPTVAL_DEPENDENCY_ONEOF = 'one_of';
	const OPTVAL_DEPENDENCY_ALL = 'all';
	
	protected $_options = null;
	/**
	 * @todo comment
	 * @var Zend_Controller_Request_Abstract 
	 */
	protected $_request = null;

	/* (non-PHPdoc)
	 * @see Hpls_Constraint_Interface::setOptions()
	 */
	public function setOptions($options) {
		$this->_options = $options;
	}

	/**
	 * @return the $_options
	 */
	protected function getOptions() {
		return $this->_options;
	}

	protected function getOption($key, $default = null) {
		if (isset($this->_options[$key]))
			return $this->_options[$key];
		return $default;
	}

	/**
	 * @return the $_request
	 */
	protected function getRequest() {
		return $this->_request;
	}

	/**
	 * @param Zend_Controller_Request_Abstract $_request
	 */
	protected function setRequest($_request) {
		$this->_request = $_request;
	}

	/**
	 * @todo comment
	 * @return NULL|string
	 */
	protected function getDepKind() {
		return $this->_options[self::OPT_FIELDDEPENDENCY];
	}

	/* (non-PHPdoc)
	 * @see Hpls_Constraint_Interface::getCheckerID()
	 */
	public function getCheckerID() {
		// TODO Auto-generated method stub
		return get_class($this);
	}

	protected function addFieldError($field, $errMsg, $errCode = null, $category = null) {
		if (empty($category)) {
			$category = get_class($this);
		}
		$this->addFieldMsg(Hpls_Msgs::MSG_ERROR, $field, $errMsg, $errCode, $category);
	}

	/**
	 * @todo comment
	 * <p>Um Speicher bei der Verarbeitung zu sprachen, werden Objekte statt Arrays eingesetzt, 
	 * da bei Array immer neue Kopien angelegt würden
	 * @param string $type
	 * @param string $field
	 * @param string $msg
	 * @param int|string $code
	 * @param string $category
	 */
	protected function addFieldMsg($level, $field, $msg, $code = null, $category = null) {
		if (empty($category)) {
			$category = get_class($this);
		}
		
		Hpls_Msgs::i()->addFieldMsg($level, $category, $field, $msg, $code);
		if (is_object($this->_request)) {
			$rmsgObj = $this->_request->getParam(Hpls_Constraint_Consts::REQ_KEY_RESULTS, null);
			if (is_null($rmsgObj)) {
				$rmsgObj = new stdClass();
				$rmsgObj->{__count__} = new stdClass();
			}
			
			$fieldObj = null;
			$level = strtoupper($level);
			if (isset($rmsgObj->{$field})) {
				$fieldObj = $rmsgObj->{$field};
			} else {
				$fieldObj = new stdClass();
				$rmsgObj->{$field} = $fieldObj;
			}
			
			if (!isset($rmsgObj->__count__->{$level})) {
				$rmsgObj->__count__->{$level} = 1;
			} else {
				$count = $rmsgObj->__count__->{$level};
				$rmsgObj->__count__->{$level} = $count + 1;
			}
			
			$typeObj = null;
			if (isset($fieldObj->{$level})) {
				$typeObj = $fieldObj->{$level};
			} else {
				$typeObj = new stdClass();
				$fieldObj->{$level} = $typeObj;
			}
			
			if (!isset($typeObj->{$category})) {
				$typeObj->{$category} = array();
			}
			$typeObj->{$category}[] = array(
					'msg' => $msg,
					'code' => $code );
			$this->_request->setParam(Hpls_Constraint_Consts::REQ_KEY_RESULTS, $rmsgObj);
		}
	}

	/**
	 * @todo comment
	 * @return boolean
	 */
	protected function isOneOf() {
		return $this->getOption(self::OPT_FIELDDEPENDENCY, self::OPTVAL_DEPENDENCY_ONEOF) == self::OPTVAL_DEPENDENCY_ONEOF;
	}
}
