<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : 14.06.2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.libs/Hpls/Constraint/Unifier/Date.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 */
/***/
require_once 'Hpls/Constraint/Unifier/Interface.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package HPLS
 * @subpackage subpackage
 * @version $LastChangedRevision: 9196 $
 * @since alpha.1
 *
 */
class Hpls_Constraint_Unifier_Date implements 
	Hpls_Constraint_Unifier_Interface {
	
	public static $_formatHandler = array(
			'/^\d{4}-\d{2}-\d{2}$/' => 'yyyy-MM-dd' /* ISO */
	,'/^\d{4}-\d{2}-\d{2}(T|\ )\d{2}:\d{2}:\d{2}$/' => 'yyyy-MM-dd\TH:i:s',
			'/^\d{4}-\d{2}-\d{2}(T|\ )\d{2}:\d{2}$/' => 'yyyy-MM-dd\TH:i:00',
			//			'/^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}$/' => 'yyyy-MM-dd\TH:i:s',
			'/^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\+\d{4}$/' => 'yyyy-MM-dd\TH:i:sO',
			'/^\d+$/' => Zend_Date::TIMESTAMP );
	protected static $_instance = null;

	public static function i() {
		if (is_null(self::$_instance)) {
			self::$_instance = new Hpls_Constraint_Unifier_Date();
		}
		return self::$_instance;
	}
	
	protected $_formats;

	public function __construct(array $formats = array('YYYY-MM-DD')) {
		$this->_formats = $formats;
	}

	public function setFormats(array $formats) {
		$this->_formats = $formats;
	}

	public function getFormat($date) {
		foreach (self::$_formatHandler as $match => $format) {
			if (preg_match($match, $date)) {
				return $format;
			}
		}
		return false;
	}

	/**
	 * @todo comment
	 * @param int|null $value the unix timestamp in milliseconds
	 * @return int|null the unix timestamp in milliseconds or <b>null</b> if unification not possible.
	 */
	public function filter($value) {
		Hpls_Utils::log(__METHOD__ . Hpls_Debug::print_r($value));
		if (is_numeric($value))
			return $value;
		$timestamp = null;
		if (($timestamp = strtotime($value)) === false) {
			$timestamp = null;
		}
		return $timestamp;
	}

	protected function handleISO($dateString) {
	
	}

	/* (non-PHPdoc)
	 * @see Hpls_Constraint_Interface::apply()
	 */
	public function apply($fields, Zend_Controller_Request_Abstract $request) {
		// TODO Auto-generated method stub
		if (!is_array($fields)) {
			$fields = array(
					$fields );
		}
		foreach ($fields as $field) {
			if ($request->getParam($field, null) !== null) {
				$request->setParam($field, $this->filter($request->getParam($field, null)));
			}
		}
		return true;
	}

	/* (non-PHPdoc)
	 * @see Hpls_Constraint_Interface::setOptions()
	 */
	public function setOptions($options) {
		// TODO Auto-generated method stub
		if (is_array($options)) {
			if (isset($options['formats']) && is_array($options['formats'])) {
				$this->_formats = $options['formats'];
			}
		}
	}

	/* (non-PHPdoc)
	 * @see Hpls_Constraint_Interface::getCheckerID()
	 */
	public function getCheckerID() {
		// TODO Auto-generated method stub
		return __CLASS__;
	}

}