<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 03.05.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Utils/Logusing.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
abstract class Hpls_ConstraintAbstract extends Hpls_Utils_Logusing {
	const ALL_KEY = '__all__';
	/**
	 * @todo comment
	 * @var Zend_Controller_Request_Abstract
	 */
	protected $_request = null;
	protected $_config = null;
	protected $_doneFields = array();
	protected static $NOTOUCH = array(
			'module',
			'controller',
			'action' );

	protected function __construct($regConfigKey) {
		if (is_array($regConfigKey)) {
			$this->_config = $regConfigKey;
		} else {
			$this->_config = Zend_Registry::get($regConfigKey);
		}
	}

	/**
	 * @todo comment
	 * @param Zend_Controller_Request_Abstract $request
	 * @throws Hpls_Exception_NotSupported
	 */
	public function apply(Zend_Controller_Request_Abstract $request) {
		isDebug('constraints') && $this->log()->debug(get_class($this) . '::' . __METHOD__ . ' START');
		$this->_request = $request;
		$constraintList = $this->getConstraintsToApply($request->getModuleName(), $request->getControllerName(), $request->getActionName());
		isDebug('constraints') && $this->log()->debug(Hpls_Debug::print_r($constraintList, get_class($this) . '::' . __METHOD__ . " ToDo:"));
		foreach ($constraintList as $contraints) {
			
			foreach ($contraints as $constraintClass => $specs) {
				
				$class = null;
				if (!isset($specs['instance'])) {
					//Zend-Klassen wrappen
					if (strncasecmp($constraintClass, 'Zend_', 5) == 0) {
						$path = explode('_', $constraintClass);
						//zu wrappende Klasse als option übergeben
						switch (strtolower($path[1])) {
							case 'filter' :
								require_once 'Hpls/Constraint/ZendFilterWrapper.php';
								$class = new Hpls_Constraint_ZendFilterWrapper();
								$class->setOptions(array(
										'towrap' => $constraintClass,
										'file' => $specs['file'] ));
								break;
							case 'validate' :
								require_once 'Hpls/Constraint/ZendValidateWrapper.php';
								$class = new Hpls_Constraint_ZendValidateWrapper();
								$class->setOptions(array(
										'towrap' => $constraintClass,
										'file' => $specs['file'] ));
								break;
							default :
								require_once 'Hpls/Exception/NotSupported.php';
								throw new Hpls_Exception_NotSupported("'$class' from Zend is not supported.", -1);
						}
					} else {
						//keine Zend-Klasse
						require_once $specs['file'];
						$class = new $constraintClass();
						$class->setOptions($specs['options']);
					}
					$specs['instance'] = $class;
				} else {
					$class = $specs['instance'];
				}
				
				if (isset($specs['each'])) {
					foreach ($specs['each'] as $fields) {
						$this->applyConstraintEach($class, $fields, $request);
					}
				}
				if (isset($specs['all'])) {
					foreach ($specs['all'] as $fields) {
						$this->applyConstraintAll($class, $fields, $request);
					}
				}
			
			} // for $checkers
		} // for $checkerList
		

		isDebug('constraints') && $this->log()->debug(get_class($this) . '::' . __METHOD__ . ' END');
	}

	/**
	 * @todo comment
	 * @param Hpls_Constraint_Interface $class
	 * @param unknown_type $fields
	 * @param Zend_Controller_Request_Abstract $request
	 * @return multitype:Ambigous <void, boolean> 
	 */
	protected function applyConstraintEach(Hpls_Constraint_Interface $class, $fields, Zend_Controller_Request_Abstract $request) {
		isDebug('constraints') && $this->log()->debug(get_class($this) . '::' . __METHOD__ . ' START - apply: ' . get_class($class));
		$ret = array();
		foreach ($fields as $field) {
			if (!in_array($field, self::$NOTOUCH)) {
				$ret[$field] = $class->apply($field, $request);
				if (!isset($this->_doneFields[$field]) || !is_array($this->_doneFields[$field])) {
					$this->_doneFields[$field] = array();
				}
				$this->_doneFields[$field][] = $class->getCheckerID();
			}
		}
		isDebug('constraints') && $this->log()->debug(get_class($this) . '::' . __METHOD__ . ' END - apply: ' . get_class($class) . Hpls_Debug::dump($ret, 'RETURN'));
		return $ret;
	}

	/**
	 * @todo comment
	 * @param Hpls_Constraint_Interface $class
	 * @param array $fields
	 * @param Zend_Controller_Request_Abstract $request
	 * @return Ambigous <void, boolean>
	 */
	protected function applyConstraintAll(Hpls_Constraint_Interface $class, array $fields, Zend_Controller_Request_Abstract $request) {
		isDebug('constraints') && $this->log()->debug(get_class($this) . '::' . __METHOD__ . ' START - apply: ' . get_class($class));
		$ret = $class->apply($fields, $request);
		foreach ($fields as $field) {
			$this->_doneFields[$field][] = $class->getCheckerID();
		}
		isDebug('constraints') && $this->log()->debug(get_class($this) . '::' . __METHOD__ . ' END - ' . Hpls_Debug::print_r($ret, 'RETURN'));
		return $ret;
	}

	/**
	 * @todo comment
	 * @param unknown_type $curModule
	 * @param unknown_type $curController
	 * @param unknown_type $curAction
	 * @param unknown_type $params
	 * @return multitype:unknown 
	 */
	protected function getConstraintsToApply($curModule, $curController, $curAction, $params = null) {
		$ret = array();
		isDebug('constraints') && $this->log()->debug(get_class($this) . '::' . __METHOD__ . "($curModule,$curController,$curAction)" . Hpls_Debug::print_r($this->_config, ' config '));
		$tmpControllers = array_intersect_key($this->_config, array(
				self::ALL_KEY => 1,
				$curModule => 1 ));
		isDebug('constraints') && $this->log()->debug(get_class($this) . '::' . __METHOD__ . "($curModule,$curController,$curAction)" . Hpls_Debug::print_r($tmpControllers, ' CONTROLLERS '));
		if (!empty($tmpControllers)) {
			foreach ($tmpControllers as $controllers) {
				$tmpActions = array_intersect_key($controllers, array(
						self::ALL_KEY => 1,
						$curController => 1 ));
				//				$this->log()->debug(Hpls_Debug::print_r($tmpActions, get_class($this) . '::' . __METHOD__ . ' ACTIONS '));
				if (!empty($tmpActions)) {
					foreach ($tmpActions as $actions) {
						$tmpCheckers = array_intersect_key($actions, array(
								self::ALL_KEY => 1,
								$curAction => 1 ));
						//						$this->log()->debug(Hpls_Debug::print_r($tmpCheckers, get_class($this) . '::' . __METHOD__ . ' CONSTRAINEVALUER '));
						//alle checker anhängen, auch wenn sie dadurch doppelt vorkommen
						

						foreach ($tmpCheckers as $checkClass) {
							//@todo zusammenfassen, wenn die options gleich sind
							$ret[] = $checkClass;
						}
					
					}
				}
			}
		}
		return $ret;
	}

	protected function addError($fields, $message, $exception = null) {
	
	}

	protected function load($source) {
	
	}

	protected function save($source) {
	
	}

}