<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created   : Sep 19, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Executable/ServiceLSB.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Executable/Service.php';
require_once 'Hpls/Executable/Wrapper.php';

/**
 * @todo comment
 *
 * @author author
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9196 $
 * @since Magdeburg:Produktiv1
 */
class Hpls_Executable_ServiceLSB extends Hpls_Executable_Service {
	
	const SERVICE_EXECUTOR = '/usr/sbin/service';
	
	const RET_CODE_SERVICE_UNKNOWN = 100;
	const RET_CODE_ACTION_NOT_ALLOWED = 101;
	/*
STATUS CODES
Should an init script be executed, invoke-rc.d always returns the status code returned by the init script. Init scripts should not return status codes in the 100+ range (which is reserved in Debian and by the LSB). The status codes returned by invoke-rc.d proper are:

0
    Success. Either the init script was run and returned exit status 0 (note that a fallback action may have been run instead of the one given in the command line), or it was not run because of runlevel/local policy constrains and --disclose-deny is not in effect. 
1 - 99
    Reserved for init.d script, usually indicates a failure. 
100
    Init script ID (name) unknown. This means the init script was not registered successfully through update-rc.d or that the init script does not exist. 
101
    Action not allowed. The requested action will not be performed because of runlevel or local policy constraints. 
102
    Subsystem error. Init script (or policy layer) subsystem malfunction. Also, forced init script execution due to --try-anyway or --force failed. 
103
    Syntax error. 
104
    Action allowed. Init script would be run, but --query is in effect. 
105
    Behavior uncertain. It cannot be determined if action should be carried out or not, and --query is in effect. 
106
    Fallback action requested. The policy layer denied the requested action, and supplied an allowed fallback action to be used instead. 	
	*/
	/**
	 * @var Hpls_Executable_Wrapper
	 */
	protected $_executor = null;

	/**
	 * Enter description here ...
	 * @param string $serviceName /etc/init.d/<serviceName>
	 */
	public function __construct($serviceName) {
		parent::__construct($serviceName);
		$this->_executor = new Hpls_Executable_Wrapper(self::SERVICE_EXECUTOR, array(), true);
		$this->_executor->setExecWithSudo(true);
	}

	public function exists($host = null) {
		// TODO Auto-generated method stub
		return file_exists('/etc/init.d/' . $this->getService());
	}

	public function status($host = null) {
		$this->setCommandOptions('status');
		return $this->runService($host);
	}

	/* (non-PHPdoc)
	 * @see Hpls_Executable_Service::start()
	 */
	public function start($host = null) {
		$this->setCommandOptions('start');
		return $this->runService($host);
	}

	/* (non-PHPdoc)
	 * @see Hpls_Executable_Service::stop()
	 */
	public function stop($host = null) {
		$this->setCommandOptions('stop');
		return $this->runService($host);
	
	}

	public function reload($host = null) {
		$this->setCommandOptions('reload');
		return $this->runService($host);
	}

	public function restart($host = null) {
		$this->setCommandOptions('restart');
		return $this->runService($host);
	
	}

	public function otherAction($initScriptAction, $host = null) {
		$this->setCommandOptions($initScriptAction);
		return $this->runService($host);
	}

	public function setStopByOneError($value = true) {
		$this->_executor->setStopByOneError($value);
	}

	public function isStopByOneError() {
		return $this->_executor->isStopByOneError();
	}

	public function getOutput($host = null) {
		return $this->_executor->getOutput($host);
	}

	public function getReturnCode($host = null) {
		return $this->_executor->getReturnCode($host);
	}

	public function setHosts($hosts) {
		$this->_executor->setHosts($hosts);
	}

	public function getHosts() {
		return $this->_executor->getHosts();
	}

	public function setRemoteUser($user = null) {
		$this->_executor->setRemoteUser($user);
	}

	public function getRemoteUser() {
		return $this->_executor->getRemoteUser();
	}

	public function setFireAndForget($value = true) {
		$this->_executor->setFireAndForget($value);
	}

	public function isFireAndForget() {
		return $this->_executor->isFireAndForget();
	}

	protected function setCommandOptions($options) {
		if (FALSE === is_array($options)) {
			$options = array(
					$options );
		}
		array_unshift($options, $this->getService());
		$this->_executor->setCommandOptions($options);
	}

	protected function setCommand($commandName, $commandOptions) {
		$this->_serviceName = $commandName;
		$this->_executor->setCommand(self::SERVICE_EXECUTOR, $commandOptions);
	}

	/**
	 * @todo comment
	 * 
	 * @param string|NULL $host
	 * @return Ambigous <string, boolean>
	 */
	protected function runService($host = null) {
		isDebug() && $this->log()->debug(__METHOD__);
		$ret = false;
		if (is_null($host) === FALSE) {
			$ret = $this->_executor->execOnHost($host);
		} else {
			$ret = $this->_executor->exec();
		}
		return $ret;
	}

}