<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 26.07.2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Executable/WrapperAbstract.php $
 *
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Cli/Consts.php';
require_once 'Hpls/Utils/Logusing.php';
/**
 * @todo comment
 * <p>Stellt Basis-Funktionen zur Verfügung, die Ableitungen nutzten können um ihre Funktionalität zu realisieren. Sie sind protected um der
 * Ableitung die Wahl zu lassen, ob sie diese Funktionen direkt als API zur Verfügung stellt oder eigene public-Method implementiert.
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package AC
 * @subpackage ??
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.0.5
 */
abstract class Hpls_Executable_WrapperAbstract extends Hpls_Utils_Logusing {
	const REMOTE_EXECUTOR = '/usr/bin/ssh';
	
	protected $_cmd;
	protected $_cmdOptions;
	protected $_cmdLine;
	protected $_cmdOutput = array();
	protected $_cmdReturnCode = array();
	protected $_cmdOutputRedirect = null;
	protected $_cmdExecutor = null;
	protected $_hosts;
	protected $_stopByOneError;
	protected $_execWithSudo;
	protected $_fireAndForget = false;
	protected $_tmpDir = null;
	
	protected $_remoteUser;
	protected $_startAt = false;
	protected $DEBUG = true;

	protected function __construct($cmd, $cmdOptions = '', $stopByOneError = true) {
		$this->setStopByOneError($stopByOneError);
		$this->setCommand($cmd, $cmdOptions);
		$this->setHosts(Hpls_Cli_Consts::HOST_LOCALHOST);
	}

	/**
	 * @todo comment
	 *
	 * @return boolean true, wenn das Programm einen return-code von 0 geliefert hat. Für den eigentlichen Returncode getReturnCode() befragen.
	 *
	 * @throws Hpls_Exception_Executable_Wrapper Wenn etwas beim Aufruf des Programm schief ging.
	 */
	protected function exec() {
		$cmd = $this->getCmdLine();
		$this->log()->debug(__METHOD__ . "## $cmd");
		$success = true;
		foreach ($this->getHosts() as $host) {
			$ret = $this->execOnHost($host);
			$success = $success && $ret;
			if ($this->isStopByOneError() && $ret === FALSE) {
				$this->log()->info("Error in execution '$cmd' on '$host'");
				return false;
			}
		}
		return $success;
	}

	protected function sudoExec() {
		$this->setExecWithSudo();
		return $this->exec();
	}

	protected function execAtTimeLocalhost($time = 'now + 1 minute') {
		$cmd = $this->getCmdLine();
		$host = Hpls_Cli_Consts::HOST_LOCALHOST;
		
		$filename = sprintf("%s/start__%s-%s__.at", Hpls_Consts0::DIR_DATA_JOBS, strtr($this->_cmd, '/ ', '__'), strtr($time, '/ +', '___'));
		$atcmd = sprintf("at -f '%s' $time 2>&1", $filename);
		
		$output = array();
		$returnCode = 0;
		try {
			isDebug('print_exec_cmdline') && $this->log()->debug(__METHOD__ . ": Executing '$atcmd'");
			file_put_contents($filename, $cmd);
			exec($atcmd, $output, $returnCode);
			isDebug('print_exec_result') && $this->log()->debug(__METHOD__ . ": ExecResult '$atcmd': " . Hpls_Debug::print_r($returnCode, 'RETCODE') . Hpls_Debug::print_r($output, 'OUTPUT'));
		} catch (Exception $ex) {
			require_once 'Hpls/Exception/Executable/Wrapper.php';
			throw new Hpls_Exception_Executable_Wrapper($ex->getMessage(), $ex->getCode(), $this->getCommand(), $this->_cmdOptions, $host);
		}
		
		//herausfinden, wann der Job gestartet wird (steht im output)
		$timestr = '';
		if (count($output) > 1) {
			$timestr = $output[1];
		
		//[0] => warning: commands will be executed using /bin/sh\n    [1] => job 42 at Wed Dec 22 16:46:00 2010
		} else {
			$timestr = $output[0];
		}
		$match = array();
		if (preg_match('/.*job \d+ at (.*)$/', $timestr, $match) > 0) {
			$this->_startAt = strtotime($match[1]);
		}
		
		$this->_cmdOutput[$host] = $output;
		$this->_cmdReturnCode[$host] = $returnCode;
		return $returnCode === 0;
	}

	protected function getAtExecTime() {
		return $this->_startAt;
	}

	/**
	 * Führt den Befehl $this->getCmd() mit den gesetzten Optionen auf $host aus.
	 *
	 * @param string $host IP oder DNS-Hostname, auf der das Programm ausgeführt werden soll.
	 * Um es lokal auszuführen, leer lassen oder Hpls_Cli_Consts::HOST_LOCALHOST angeben
	 *
	 * @return boolean true, wenn das Programm einen return-code von 0 geliefert hat. Für den eigentlichen Returncode getReturnCode() befragen.
	 *
	 * @throws Hpls_Exception_Executable_Wrapper Wenn etwas beim Aufruf des Programm schief ging.
	 */
	protected function execOnHost($host) {
		$cmd = $this->getCmdLine();
		$sp = $this->splitHostUser($host);
		$host = $sp['host'];
		$user = isset($sp['user']) ? $sp['user'] : $this->getRemoteUser();
		
		$this->log()->debug(__METHOD__ . "## exec '$cmd' on host '$host' as '$user' ('' == apache user)");
		
		if (!(empty($host) || strcasecmp($host, Hpls_Cli_Consts::HOST_LOCALHOST) === 0 || strcasecmp($host, Hpls_Cli_Consts::HOST_LOCALHOST_IP) === 0)) {
			$cmd = $this->getRemoteExecutor($host, $user) . " '$cmd'";
		} else {
			$host = Hpls_Cli_Consts::HOST_LOCALHOST;
		}
		$output = array();
		$returnCode = 0;
		try {
			isDebug('print_exec_cmdline') && $this->log()->debug(__METHOD__ . ": Executing '$cmd'");
			exec($cmd, $output, $returnCode);
			isDebug('print_exec_result') && $this->log()->debug(__METHOD__ . ": ExecResult '$cmd': " . Hpls_Debug::print_r($returnCode, 'RETCODE') . Hpls_Debug::print_r($output, 'OUTPUT'));
		} catch (Exception $ex) {
			require_once 'Hpls/Exception/Executable/Wrapper.php';
			throw new Hpls_Exception_Executable_Wrapper($ex->getMessage(), $ex->getCode(), $this->getCommand(), $this->_cmdOptions, $host);
		}
		
		$this->_cmdOutput[$host] = $output;
		$this->_cmdReturnCode[$host] = $returnCode;
		return $returnCode === 0;
	}

	/**
	 * @param array:string|string $hosts
	 */
	protected function setHosts($hosts) {
		if (empty($hosts)) {
			$this->_hosts = array(
					Hpls_Cli_Consts::HOST_LOCALHOST );
		} else if (is_array($hosts) === FALSE) {
			$this->_hosts = array(
					$hosts );
		} else {
			$this->_hosts = $hosts;
		}
	}

	protected function getHosts() {
		return $this->_hosts;
	}

	protected function setExecWithSudo($value = true) {
		$this->_execWithSudo = $value;
	}

	protected function isExecWithSudo() {
		return $this->_execWithSudo;
	}

	protected function setCmdExecutor($value = null) {
		$this->_cmdExecutor = $value;
	}

	protected function isExecuteWithExecutor() {
		return is_null($this->_cmdExecutor);
	}

	protected function setRemoteUser($user = null) {
		$this->_remoteUser = $user;
	}

	protected function getRemoteUser() {
		return $this->_remoteUser;
	}

	protected function setCommand($commandName, $commandOptions) {
		$this->_cmd = $commandName;
		$this->setCommandOptions($commandOptions);
		$this->_cmdLine = null;
	}

	protected function getCommand() {
		return $this->_cmd;
	}

	protected function setCommandOptions($commandOptions) {
		if (!is_array($commandOptions)) {
			$commandOptions = array(
					$commandOptions );
		}
		$this->_cmdOptions = $commandOptions;
		$this->_cmdLine = null;
	}

	protected function getCommandOptions() {
		return $this->_cmdOptions;
	}

	protected function addParam($param, $value = null, $quote = true) {
		if (!is_null($value)) {
			if ($quote) {
				$param .= " '" . addslashes($value) . "'";
			} else {
				$param .= " $value";
			}
		}
		$this->_cmdOptions[] = $param;
	}

	/**
	 * @param boolean $value
	 */
	protected function setStopByOneError($value = true) {
		$this->_stopByOneError = $value;
	}

	/**
	 *
	 */
	protected function isStopByOneError() {
		return $this->_stopByOneError;
	}

	protected function setFireAndForget($value = true) {
		$this->_fireAndForget = $value;
	}

	protected function isFireAndForget() {
		return $this->_fireAndForget;
	}

	/**
	 * @todo comment
	 *
	 * @param string|NULL $host Der Hostname/IP, auf dem der Service manipuliert wurde, NULL => LOCALHOST
	 * @return multitype:multitype:string|null Die Ausgabe des Ausrufs auf Host $host.
	 */
	protected function getOutput($host = null) {
		if (is_null($host)) {
			return $this->_cmdOutput[Hpls_Cli_Consts::HOST_LOCALHOST];
		}
		return isset($this->_cmdOutput[$host]) ? $this->_cmdOutput[$host] : null;
	}

	/**
	 * @todo comment
	 *
	 * @param string|NULL $host Der Hostname/IP, null => LOCALHOST
	 * @return int|null Der Returncode des Ausrufs auf Host $host.
	 */
	protected function getReturnCode($host = null) {
		if (is_null($host)) {
			return $this->_cmdReturnCode[Hpls_Cli_Consts::HOST_LOCALHOST];
		}
		return isset($this->_cmdReturnCode[$host]) ? $this->_cmdReturnCode[$host] : null;
	}

	protected function getRemoteExecutor($host, $user = null) {
		$cmd = self::REMOTE_EXECUTOR . " ";
		if (!is_null($user)) {
			$cmd .= "$user@";
		}
		$cmd .= "$host ";
		return $cmd;
	}

	protected function getCmdLine() {
		if (is_null($this->_cmdLine)) {
			$this->_cmdLine = $this->getCommand();
			$cmdOpts = $this->getCommandOptions();
			if (empty($cmdOpts)) {
				//				no further arguments
			} else if (is_array($cmdOpts)) {
				$this->_cmdLine .= ' ' . implode(' ', $cmdOpts);
			} else {
				$this->_cmdLine .= " $cmdOpts";
			}
		}
		if (!is_null($this->_cmdOutputRedirect)) {
			$this->_cmdLine .= " " . $this->_cmdOutputRedirect;
		}
		$cmd = $this->_cmdLine;
		
		if ($this->isExecuteWithExecutor()) {
			$cmd = $this->_cmdExecutor . " $cmd";
		}
		
		if ($this->isExecWithSudo()) {
			$cmd = "sudo " . $cmd;
		}
		if ($this->isFireAndForget()) {
			$cmd .= ' > /dev/null 2>&1 &';
		}
		return $cmd;
	}

	protected function splitHostUser($host) {
		$split = explode('@', $host);
		if (count($split) === 1) {
			return array(
					'host' => $host );
		}
		return array(
				'host' => $split[0],
				'user' => $split[1] );
	}

	protected function getTmpDir() {
		if (is_null($this->_tmpDir)) {
			$cfg = Zend_Registry::get(MAConstsBack::REGBCK_CONFIG)->toArray();
			$this->_tmpDir = $cfg['cacheDir'];
		}
		return $this->_tmpDir;
	}

}