<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 21.12.2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 *
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Job/StatusAbstract.php';
require_once 'Hpls/Job/Status/Factory.php';

abstract class Hpls_Job_Abstract {
	protected $_jobName = null;
	protected $_jobID = null;
	protected $_jobHost = 'localhost';
	
	/**
	 * Enter description here ...
	 * @var Hpls_Job_StatusAbstract
	 */
	protected $_status = null;

	public function __construct($jobName, $jobID, $jobHost = 'localhost', Hpls_Job_StatusAbstract $status = null) {
		$this->setJobName($jobName);
		$this->setJobID($jobID);
		$this->setJobHost($jobHost);
		
		if (is_null($status)) {
			$this->_status = Hpls_Job_Status_Factory::i()->getStatus($jobName, $jobID, $jobHost);
			$this->initSteps();
		} else {
			$this->setStatus($status);
		}
	}

	/**
	 * @return the $_jobName
	 */
	public function getJobName() {
		return $this->_jobName;
	}

	/**
	 * @return the $_jobID
	 */
	public function getJobID() {
		return $this->_jobID;
	}

	/**
	 * @return the $_jobHost
	 */
	public function getJobHost() {
		return $this->_jobHost;
	}

	/**
	 * @param field_type $_jobName
	 */
	public function setJobName($_jobName) {
		$this->_jobName = $_jobName;
	}

	/**
	 * @param field_type $_jobID
	 */
	public function setJobID($_jobID) {
		$this->_jobID = $_jobID;
	}

	/**
	 * @param field_type $_jobHost
	 */
	public function setJobHost($_jobHost) {
		$this->_jobHost = $_jobHost;
	}

	public function isRunning() {
		if ($this->getStatus()->hasPersistedState()) {
			//			Hpls_Debug::log()->debug(__METHOD__ . '### persisted state');
			

			if (!$this->getStatus()->isLoaded()) {
				//				Hpls_Debug::log()->debug(__METHOD__ . '### loading persisted state');
				$this->getStatus()->load();
			}
		}
		//		Hpls_Debug::log()->debug(__METHOD__ . '### not RUNNING');
		return $this->getStatus()->isRunning();
	}

	public function isFinished() {
		if ($this->getStatus()->hasPersistedState()) {
			//			Hpls_Debug::log()->debug(__METHOD__ . '### persisted state');
			if (!$this->getStatus()->isLoaded()) {
				//				Hpls_Debug::log()->debug(__METHOD__ . '### loading persisted state');
				$this->getStatus()->load();
			}
		}
		//		Hpls_Debug::log()->debug(__METHOD__ . '### FINISHED');
		return $this->getStatus()->isFinished();
	}

	public function isStarted() {
		if ($this->getStatus()->hasPersistedState()) {
			//			Hpls_Debug::log()->debug(__METHOD__ . '### persisted state');
			if (!$this->getStatus()->isLoaded()) {
				//				Hpls_Debug::log()->debug(__METHOD__ . '### loading persisted state');
				$this->getStatus()->load();
			}
		}
		//		Hpls_Debug::log()->debug(__METHOD__ . '### FINISHED');
		return $this->getStatus()->isStarted();
	
	}

	public function clearStatus() {
		if ($this->getStatus()->hasPersistedState()) {
			if ($this->getStatus()->isLoaded()) {
				$this->_status = Hpls_Job_Status_Factory::i()->getStatus($this->_jobName, $this->_jobID, $this->_jobHost);
				$this->initSteps();
			} else {
				$this->getStatus()->clearPersistedState();
			}
		}
	}

	/**
	 * @return Hpls_Job_StatusAbstract $_status
	 */
	public function getStatus() {
		return $this->_status;
	}

	/**
	 * @param Hpls_Job_StatusAbstract $status
	 * @throws Hpls_Exception_Job Wenn der Status nicht zu diesem Job gehört.
	 */
	public function setStatus(Hpls_Job_StatusAbstract $status) {
		if (!$status->isStatusForJob($this)) {
			require_once 'Hpls/Exception/Job.php';
			throw new Hpls_Exception_Job("The given Job_Status ($status) is not valid one for this Hpls_Job ($this).", 100);
		}
		
		$this->_status = $status;
	}

	public function __toString() {
		return $this->_jobName . ':' . $this->_jobID . '@' . $this->_jobHost;
	}

	abstract protected function initSteps();

	public function persist() {
		$this->getStatus()->save();
	}

	protected function addStep($stepID) {
		$status = $this->getStatus();
		$status->addStep($stepID);
	}

}