<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 22.12.2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Job/Step.php $
 *
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
class Hpls_Job_Step {
	const STATE_UNDEFINED = -1;
	const STATE_NOTSTARTED = 0;
	const STATE_RUNNING = 1;
	const STATE_FINISHED = 2;
	const STATE_PAUSED = 3;
	const STATE_ABORTED = 10;
	const STATE_FAILED = 100;

	public static function stateIdToString($id) {
		switch ($id) {
			case self::STATE_FAILED :
				return 'failed';
			case self::STATE_FINISHED :
				return 'finished';
			case self::STATE_NOTSTARTED :
				return 'notstarted';
			case self::STATE_PAUSED :
				return 'paused';
			case self::STATE_ABORTED :
				return 'aborted';
			case self::STATE_RUNNING :
				return 'running';
			case self::STATE_UNDEFINED :
				return 'undefined';
			default :
				return 'unknown';
		}
	}
	
	/**
	 * Enter description here ...
	 * @var Hpls_Job_StatusAbstract
	 */
	protected $_owner = null;
	
	protected $_attributes = null;
	protected $_localIDcache = null;

	public function __construct($owner, $stepID, $order) {
		$this->_owner = $owner;
		if (is_array($stepID)) {
			$stepID['order'] = $order;
			$this->setValues($stepID);
		} else {
			$this->_attributes = array(
					'order' => (int) $order,
					'id' => $stepID,
					'state' => self::STATE_NOTSTARTED,
					'percent' => 0,
					'starttime' => -1,
					'stoptime' => -1 );
		}
	}

	public function __get($varname) {
		return $this->_attributes[$varname];
	}

	public function __set($varname, $value) {
		if (is_null($value)) {
			unset($this->_attributes[$varname]);
		} else {
			$this->_attributes[$varname] = $value;
		}
	}

	public function __isset($varname) {
		return isset($this->_attributes[$varname]);
	}

	public function __unset($varname) {
		if (isset($this->_attributes[$varname])) {
			unset($this->_attributes[$varname]);
		}
	}

	/**
	 * @return the $_owner
	 */
	public function getOwner() {
		return $this->_owner;
	}

	/**
	 * @param Hpls_Job_StatusAbstract $_owner
	 */
	public function setOwner($_owner) {
		$this->_owner = $_owner;
	}

	public function setValues(array $values) {
		if (!array_key_exists('order', $values) || !array_key_exists('id', $values)) {
			require_once 'Hpls/Exception/Job/Status.php';
			throw new Hpls_Exception_Job_Status("Step has no ID or Order");
		}
		$this->_attributes = $values;
	
	}

	public function getID() {
		return $this->_attributes['id'];
	}

	public function toArray() {
		return $this->_attributes;
	}

	public function hasError() {
		return $this->state === self::STATE_FAILED;
	}

	public function isFinished() {
		return $this->state == self::STATE_FINISHED;
	}

	public function isAborted() {
		return $this->state == self::STATE_ABORTED;
	}

	public function isRunning() {
		return $this->state == self::STATE_RUNNING || $this->state == self::STATE_PAUSED;
	}

	public function isStarted() {
		return $this->state != self::STATE_NOTSTARTED;
	}

	public function getLocaleID() {
		if (is_null($this->_localIDcache)) {
			$this->_localIDcache = strtr($this->_owner->getJobName(), '.', '_') . '_' . $this->_attributes['id'];
		}
		return $this->_localIDcache;
	}

	public function getLocaleStateID() {
		return 'job_step_' . self::stateIdToString($this->state);
	}

	public function __toString() {
		return $this->_attributes['id'];
	}

}