<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 14.09.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Zend/Mail/Protocol/Imap.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hpls_Mail_Protocol_Imap extends Zend_Mail_Protocol_Imap {
	const ANONYMOUS = 'anonymous';
	const LOGIN = 'login';
	const PLAIN = 'plain';
	const EXTERNAL = 'external';
	const CRAMMD5 = 'cram-md5';
	const DIGESTMD5 = 'digest-md5';
	
	protected $_saslMode = null;

	/**
	 * Public constructor
	 *
	 * @param  string   $host  hostname or IP address of IMAP server, if given connect() is called
	 * @param  int|null $port  port of IMAP server, null for default (143 or 993 for ssl)
	 * @param  bool     $ssl   use ssl? 'SSL', 'TLS' or false
	 * @throws Zend_Mail_Protocol_Exception
	 */
	function __construct($host = '', $port = null, $ssl = false, $saslMode = self::CRAMMD5) {
		$this->_saslMode = strtolower($saslMode);
		if ($host) {
			$this->connect($host, $port, $ssl);
		}
	}

	public function login($user, $password) {
		/**
		 * Auth_SASL_Common
		 */
		$class = null;
		switch ($this->_saslMode) {
			//			case self::ANONYMOUS :
			//				$filename = 'Auth/SASL/Anonymous.php';
			//				$class = new Auth_SASL_Anonymous();
			//				break;
			//			case self::LOGIN :
			//				$filename = 'Auth/SASL/Login.php';
			//				$class = new Auth_SASL_Login();
			//				break;
			//			case self::PLAIN :
			//				$filename = 'Auth/SASL/Plain.php';
			//				$class = new Auth_SASL_Plain();
			//				break;
			//			case self::EXTERNAL :
			//				$filename = 'Auth/SASL/External.php';
			//				$class = new Auth_SASL_External();
			//				break;
			case self::CRAMMD5 :
				return $this->loginWithCramMD5($user, $password);
		
		//			case self::DIGESTMD5 :
		//				$filename = 'Auth/SASL/DigestMD5.php';
		//				$class = new Auth_SASL_DigestMD5();
		//				break;
		//			
		//			default :
		//				$scram = '/^SCRAM-(.{1,9})$/i';
		//				if (preg_match($scram, $this->_saslMode, $matches)) {
		//					$hash = $matches[1];
		//					$filename = 'Auth/SASL/SCRAM.php';
		//					$class = new Auth_SASL_SCRAM($hash);
		//					break;
		//				}
		//				break;
		}
		
		return $this->requestAndResponse('LOGIN', $this->escapeString($user, $password), true);
	
	}

	protected function loginWithCramMD5($user, $password) {
		require_once 'Auth/SASL/CramMD5.php';
		$class = new Auth_SASL_CramMD5();
		
		$tag = null;
		$this->sendRequest("AUTHENTICATE CRAM-MD5", array(), $tag);
		$tokens = null;
		$this->readLine($tokens, $tag, true);
		$challenge = base64_decode(trim($tokens));
		$reply = $class->getResponse($user, $password, $challenge);
		
		// send result
		$this->putLine(base64_encode($reply));
		$this->readLine($tokens);
		
		return $tokens; //$this->requestAndResponse($reply);
	}

	/**
	 * Send simple (one line) command to the connection stream
	 *
	 * @param string $string Command string
	 * @param bool   $endln  True if CRLF need to be added at the end of command
	 *
	 * @param int Number of bytes sent, False on error
	 */
	function putLine($string, $endln = true) {
		if (!$this->_socket)
			return false;
		
		$res = @fwrite($this->_socket, $string . ($endln ? "\r\n" : ''));
		
		if ($res === false) {
			/**
			 * @see Zend_Mail_Protocol_Exception
			 */
			require_once 'Zend/Mail/Protocol/Exception.php';
			throw new Zend_Mail_Protocol_Exception('cannot write - connection closed?');
		}
		
		return $res;
	}
}