<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 23.05.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Msgs.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
require_once 'Hpls/Consts0.php';

class Hpls_Msg_Item {
	public $msg;
	public $code;
	public $params;
	public $level;
	public $display_count = 0;
	public $max_display_count;
	public $store;

	public function __construct($msg = null, $code = null, $params = array(), $level = null, $max_display_count = 1, $store = 'obj') {
		$this->msg = $msg;
		$this->code = $code;
		$this->params = $params;
		$this->level = $level;
		$this->max_display_count = $max_display_count;
		$this->store = $store;
	}

	public function incDisplayCount() {
		$this->display_count += 1;
	}

	/**
	 * Gibt zurück, ob die Nachricht oft genug angezeigt wurde.
	 *
	 * @return boolean
	 */
	public function isShown() {
		return $this->display_count >= $this->max_display_count;
	}
}

/**
 *
 * @todo comment
 *      
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *           
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.9
 */
class Hpls_Msgs {
	const MSG_ERROR = 'error';
	const MSG_INFO = 'info';
	const MSG_WARN = 'warn';
	
	const NS_GLOBAL = '__GLOBAL__';
	const NS_GROUP = '__GROUP__';
	const NS_FIELD = '__FIELD__';
	
	protected static $_instance = null;

	/**
	 *
	 * @return Hpls_Msgs
	 */
	public static function i() {
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function __construct() {
		$this->_msgs = new stdClass();
	}
	
	protected $_msgs = null;

	/**
	 *
	 * @todo comment
	 * @param $level string
	 *       	 DEBUG|INFO|WARN|ERR
	 * @param $category string       	
	 * @param $msg string       	
	 * @param $code string       	
	 */
	public function addGlobalMsg($level, $category, $msg, $code = null, $translationParams = null, $distinct = false, $persist = false) {
		$this->log()->debug(Hpls_Debug::print_r($msg, __METHOD__));
		if ($persist !== false) {
			$this->storeGlobal($this->getPersistStore(), $level, $category, $msg, $code, $distinct, $translationParams);
		} else {
			$this->storeGlobal($this->_msgs, $level, $category, $msg, $code, $distinct, $translationParams);
		}
	}

	public function addGroupMsg($level, $category, $group, $msg, $code = null, $translationParams = null, $distinct = false, $persist = false) {
		$this->log()->debug(Hpls_Debug::print_r($msg, __METHOD__));
		if ($persist !== false) {
			$sess = Hpls_Application0::i()->getSession(Hpls_Consts0::SESSION_NS_MSGS);
			$this->storeGroup($this->getPersistStore(), $group, $level, $category, $msg, $code, $distinct, $translationParams);
		} else {
			$this->storeGroup($this->_msgs, $group, $level, $category, $msg, $code, $distinct, $translationParams);
		}
	
	}

	/**
	 *
	 * @todo comment
	 * @param $category string       	
	 * @param $field string       	
	 * @param $errMsg string|array       	
	 * @param $errCode int       	
	 * @param $translationParams array       	
	 * @param $persist boolean       	
	 */
	public function addFieldError($category, $field, $errMsg, $errCode = null, $translationParams = null, $persist = false) {
		$this->addFieldMsg(self::MSG_ERROR, $category, $field, $errMsg, $errCode);
	}

	/**
	 *
	 * @todo comment
	 *       <p>Um Speicher bei der Verarbeitung zu sprachen, werden Objekte
	 *       statt Arrays eingesetzt,
	 *       da bei Array immer neue Kopien angelegt würden
	 * @param $level string
	 *       	 DEBUG|INFO|WARN|ERR
	 * @param $category string       	
	 * @param $field string       	
	 * @param $msg string       	
	 * @param $code int|string       	
	 */
	public function addFieldMsg($level, $category, $field, $msg, $code = null, $translationParams = null, $persist = false) {
		$rmsgObj = $this->_msgs;
		if ($persist !== false) {
			$this->storeFieldMsg($this->getPersistStore(), $level, $category, $field, $msg, $code, $translationParams);
		} else {
			$this->storeFieldMsg($this->_msgs, $level, $category, $field, $msg, $code, $translationParams);
		}
	}

	public function hasGlobalMsg() {
		$sess = $this->getPersistStore();
		return (isset($this->_msgs->{self::NS_GLOBAL}) && count($this->_msgs->{self::NS_GLOBAL}) > 0) || (isset($sess->{self::NS_GLOBAL}) && count($sess->{self::NS_GLOBAL}) > 0);
	}

	public function hasFieldMsg($fieldName) {
		$sess = $this->getPersistStore();
		return (isset($this->_msgs->{self::NS_FIELD}) && isset($this->_msgs->{self::NS_FIELD}->{$fieldName}) && count($this->_msgs->{self::NS_GROUP}->{$fieldName}) > 0) || (isset($sess->{self::NS_FIELD}) && isset($sess->{self::NS_FIELD}->{$fieldName}) && count($sess->{self::NS_FIELD}->{$fieldName}) > 0);
	}

	public function hasGroupMsg($groupName) {
		$sess = $this->getPersistStore();
		return (isset($this->_msgs->{self::NS_GROUP}) && isset($this->_msgs->{self::NS_GROUP}->{$groupName}) && count($this->_msgs->{self::NS_GROUP}->{$groupName}) > 0) || (isset($sess->{self::NS_GROUP}) && isset($sess->{self::NS_GROUP}->{$groupName}) && count($sess->{self::NS_GROUP}->{$groupName}) > 0);
	}

	public function getGlobalMsgs() {
		if (false === isset($this->_msgs->{self::NS_GLOBAL})) {
			if (false === isset($this->getPersistStore()->{self::NS_GLOBAL})) {
				// weder in local noch persist store nachrichten
				return array();
			} else
				return Hpls_Utils::toArray($this->getPersistStore()->{self::NS_GLOBAL});
		} elseif (false === isset($this->getPersistStore()->{self::NS_GLOBAL})) {
			return Hpls_Utils::toArray($this->_msgs->{self::NS_GLOBAL});
		}
		// in beiden sind nachrichten
		return array_merge_recursive(Hpls_Utils::toArray($this->_msgs->{self::NS_GLOBAL}), Hpls_Utils::toArray($this->getPersistStore()->{self::NS_GLOBAL}));
	}

	public function getGroupMsgs($groupName) {
		$sess = $this->getPersistStore();
		if (false === isset($this->_msgs->{self::NS_GROUP}) || false === isset($this->_msgs->{self::NS_GROUP}->{$groupName})) {
			if (false === isset($sess->{self::NS_GROUP}) || false === isset($sess->{self::NS_GROUP}->{$groupName})) {
				return array();
			} else {
				return Hpls_Utils::toArray($sess->{self::NS_GROUP}->{$groupName});
			}
		} elseif (false === isset($sess->{self::NS_GROUP}) || false === isset($sess->{self::NS_GROUP}->{$groupName})) {
			return Hpls_Utils::toArray($this->_msgs->{self::NS_GROUP}->{$groupName});
		}
		return array_merge_recursive(Hpls_Utils::toArray($this->_msgs->{self::NS_GROUP}->{$groupName}), Hpls_Utils::toArray($sess->{self::NS_GROUP}->{$groupName}));
	}

	public function getFieldMsgs($fieldName) {
		$sess = $this->getPersistStore();
		if (false === isset($this->_msgs->{self::NS_FIELD}) || false === isset($this->_msgs->{self::NS_FIELD}->{$fieldName})) {
			if (false === isset($sess->{self::NS_FIELD}) || false === isset($sess->{self::NS_FIELD}->{$fieldName})) {
				return array();
			} else {
				return Hpls_Utils::toArray($sess->{self::NS_FIELD}->{$fieldName});
			}
		} elseif (false === isset($sess->{self::NS_FIELD}) || false === isset($sess->{self::NS_FIELD}->{$fieldName})) {
			return Hpls_Utils::toArray($this->_msgs->{self::NS_FIELD}->{$fieldName});
		}
		return array_merge_recursive(Hpls_Utils::toArray($this->_msgs->{self::NS_FIELD}->{$fieldName}), Hpls_Utils::toArray($sess->{self::NS_FIELD}->{$fieldName}));
	}

	/**
	 *
	 * @todo comment
	 * @return multitype:
	 */
	public function getFieldsWithMsgs() {
		if (!isset($this->_msgs->{self::NS_FIELD})) {
			return array();
		}
		$vars = get_object_vars($this->_msgs->{self::NS_FIELD});
		if (isset($vars['__OBJECT__']))
			unset($vars['__OBJECT__']);
		return array_keys($vars);
	}

	public function applyTo($object) {
		if ($object instanceof Zend_Controller_Request_Abstract) {
			$object->setParam(Hpls_Consts0::REQ_MSGS, $this->_msgs);
		} elseif ($object instanceof Zend_Controller_Response_Abstract) {
			$object->setBody($this->_msgs, Hpls_Consts0::REQ_MSGS);
		} elseif ($object instanceof Zend_View_Interface) {
			$object->assign(Hpls_Consts0::REQ_MSGS, $this->_msgs);
		}
	}

	/**
	 *
	 * @todo comment
	 * @todo implement
	 * @param $category unknown_type       	
	 * @param $level unknown_type       	
	 */
	public function clearGlobalMsgs($category = null, $level = null) {
		$sess = Hpls_Application0::i()->getSession(Hpls_Consts0::SESSION_NS_MSGS);
		if (is_null($level) && is_null($category)) {
			$this->_msgs = new stdClass();
		}
		
		// jetzt ist eins von beiden != null oder beide...
		if (is_null($level)) {
			// category != null
		
		} else if (is_null($category)) {
			// level != null
		
		} else {
		
		}
	
	}

	/**
	 * Löscht alle schon mehr als max_displaycount angezeigten Nachrichten.
	 */
	public function clearOldMsgs() {
		foreach (array(
				$this->_msgs,
				Hpls_Application0::i()->getSession(Hpls_Consts0::SESSION_NS_MSGS) ) as $store) {
			// global
			if (isset($store->{self::NS_GLOBAL})) {
				foreach ($store->{self::NS_GLOBAL} as $level => $categories) {
					// status
					foreach ($categories as $category => $msgs) {
						$remove = array();
						foreach ($msgs as $i => $msg) {
							if ($msg->isShown()) {
								unset($msgs[$i]);
							}
						}
						if (empty($msgs)) {
							unset($categories->{$category});
						} else {
							$categories->{$category} = $msgs;
						}
					}
					if (count((array) $categories) === 0) {
						unset($store->{self::NS_GLOBAL}->{$level});
					}
				}
				if (count((array) $store->{self::NS_GLOBAL}) === 0) {
					unset($store->{self::NS_GLOBAL});
				}
			}
			
			// group
			if (isset($store->{self::NS_GROUP})) {
				foreach ($store->{self::NS_GROUP} as $groupname => $groups) {
					foreach ($groups as $level => $categories) {
						// status
						foreach ($categories as $category => $msgs) {
							$remove = array();
							foreach ($msgs as $i => $msg) {
								if ($msg->isShown()) {
									unset($msgs[$i]);
								}
							}
						}
					}
				}
			}
			
			// field
			if (isset($store->{self::NS_FIELD})) {
				foreach ($store->{self::NS_FIELD} as $fieldname => $fields) {
					foreach ($fields as $level => $categories) {
						// status
						foreach ($categories as $category => $msgs) {
							$remove = array();
							foreach ($msgs as $i => $msg) {
								if ($msg->isShown()) {
									unset($msgs[$i]);
								}
							}
						}
					}
				}
			
			}
		
		}
	}

	public function incDisplayCountGlobal() {
		foreach (array(
				$this->_msgs,
				Hpls_Application0::i()->getSession(Hpls_Consts0::SESSION_NS_MSGS) ) as $store) {
			if (isset($store->{self::NS_GLOBAL})) {
				foreach ($store->{self::NS_GLOBAL} as $level => $categories) {
					// status
					foreach ($categories as $category => $msgs) {
						foreach ($msgs as $msg) {
							$msg->incDisplayCount();
						}
					}
				}
			}
		}
	}

	public function incDisplayCountField($fieldname) {
		foreach (array(
				$this->_msgs,
				Hpls_Application0::i()->getSession(Hpls_Consts0::SESSION_NS_MSGS) ) as $store) {
			if (isset($store->{self::NS_FIELD})) {
				if (isset($store->{self::NS_FIELD}->{$fieldname})) {
					foreach ($store->{self::NS_FIELD}->{$fieldname} as $level => $categories) {
						// status
						foreach ($categories as $category => $msgs) {
							foreach ($msgs as $msg) {
								$msg->incDisplayCount();
							}
						}
					}
				}
			}
		}
	
	}

	public function incDisplayCountGroup($groupname) {
		foreach (array(
				$this->_msgs,
				Hpls_Application0::i()->getSession(Hpls_Consts0::SESSION_NS_MSGS) ) as $store) {
			if (isset($store->{self::NS_GROUP})) {
				if (isset($store->{self::NS_GROUP}->{$groupname})) {
					foreach ($store->{self::NS_GROUP}->{$groupname} as $level => $categories) {
						// status
						foreach ($categories as $category => $msgs) {
							foreach ($msgs as $msg) {
								$msg->incDisplayCount();
							}
						}
					}
				}
			}
		}
	
	}

	protected function getPersistStore() {
		return Hpls_Application0::i()->getSession(Hpls_Consts0::SESSION_NS_MSGS);
	}

	protected function storeGlobal($rmsgObj, $level, $category, $msg, $code, $distinct, $translationParams) {
		$this->log()->debug(Hpls_Debug::print_r($rmsgObj, __METHOD__));
		assert(is_object($rmsgObj));
		$level = strtoupper($level);
		
		$destObj = $rmsgObj;
		foreach (array(
				self::NS_GLOBAL,
				$level ) as $next) {
			if (!isset($destObj->{$next})) {
				$destObj->{$next} = new stdClass();
			}
			$destObj = $destObj->{$next};
		}
		
		if (!isset($destObj->{$category})) {
			$destObj->{$category} = array();
		}
		
		if (!is_array($msg)) {
			$msg = array(
					$msg );
		}
		foreach ($msg as $m) {
			$add = true;
			if ($distinct) {
				foreach ($destObj->{$category} as $x) {
					if (($x->msg === $m) && ($x->code === $code)) {
						$add = false;
						break;
					}
				}
			}
			if ($add) {
				$destObj->{$category}[] = new Hpls_Msg_Item($m, $code, $translationParams, $level);
			}
		}
		
		$this->log()->debug(Hpls_Debug::print_r($rmsgObj, __METHOD__));
	}

	protected function storeGroup($rmsgObj, $groupname, $level, $category, $msg, $code, $distinct, $translationParams) {
		$this->log()->debug(Hpls_Debug::print_r($rmsgObj, __METHOD__));
		assert(is_object($rmsgObj));
		$level = strtoupper($level);
		
		$destObj = $rmsgObj;
		foreach (array(
				self::NS_GROUP,
				$groupname,
				$level ) as $next) {
			if (!isset($destObj->{$next})) {
				$destObj->{$next} = new stdClass();
			}
			$destObj = $destObj->{$next};
		}
		
		if (!isset($destObj->{$category})) {
			$destObj->{$category} = array();
		}
		
		if (!is_array($msg)) {
			$msg = array(
					$msg );
		}
		foreach ($msg as $m) {
			$add = true;
			if ($distinct) {
				foreach ($destObj->{$category} as $x) {
					if (($x->msg === $m) && ($x->code === $code)) {
						$add = false;
						break;
					}
				}
			}
			if ($add) {
				$destObj->{$category}[] = new Hpls_Msg_Item($m, $code, $translationParams, $level);
			}
		}
	
	}

	protected function storeFieldMsg($rmsgObj, $level, $category, $field, $msg, $code = null, $translationParams = null, $persist = false, $distinct = false) {
		assert(is_object($rmsgObj));
		
		$this->log()->debug(Hpls_Debug::print_r($rmsgObj, __METHOD__));
		assert(is_object($rmsgObj));
		$lvlObj = null;
		$glbObj = null;
		$grpObj = null;
		$level = strtoupper($level);
		
		$destObj = $rmsgObj;
		foreach (array(
				self::NS_FIELD,
				$field,
				$level ) as $next) {
			if (!isset($destObj->{$next})) {
				$destObj->{$next} = new stdClass();
			}
			$destObj = $destObj->{$next};
		}
		
		if (!isset($destObj->{$category})) {
			$destObj->{$category} = array();
		}
		
		if (!is_array($msg)) {
			$msg = array(
					$msg );
		}
		foreach ($msg as $m) {
			$add = true;
			if ($distinct) {
				foreach ($destObj->{$category} as $x) {
					if (($x->msg === $m) && ($x->code === $code)) {
						$add = false;
						break;
					}
				}
			}
			if ($add) {
				$destObj->{$category}[] = new Hpls_Msg_Item($m, $code, $translationParams, $level);
			}
		}
	}

	/**
	 *
	 * @todo comment
	 * @return Zend_Log
	 */
	protected function log() {
		return Hpls_Utils::log();
	}

}