<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 24.08.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Settings/HandlerAbstract.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Utils/Logusing.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.9
 */
abstract class Hpls_Settings_HandlerAbstract extends Hpls_Utils_Logusing {
	
	protected $_autoload = true;
	
	/**
	 * @todo comment
	 * @var Zend_Config
	 */
	protected $_configData = null;
	/**
	 * Konfiguration für den Handler selbst.
	 * 
	 * @var array:string
	 */
	protected $_options = null;
	/**
	 * @todo comment
	 * @var array:string
	 */
	private $_prefixKey = null;

	public function __construct($key = null) {
		$this->setPrefixKey($key);
	}

	public function getPrimaryKey() {
		return $this->_prefixKey;
	}

	/**
	 * @todo comment
	 * @param string $key
	 * @throws Zend_Config_Exception
	 */
	abstract public function save($key = null);

	/**
	 * @todo comment
	 * @param string $key
	 * @throws Zend_Config_Exception
	 */
	abstract public function load($key = null);

	/**
	 * @todo comment
	 * @param Zend_Config|null $subkey
	 * @param boolean $create Wenn der $subkey nicht existiert, soll er angelegt werden?
	 * @return Zend_Config|null <null>, wenn der $subkey nicht gefunden wurde
	 */
	public function getConfig($subkey, $autoload = true, $create = false, $uncached = false) {
		isDebug('settings','method_trace') && $this->log()->debug(Hpls_Debug::print_r($subkey, __METHOD__ . " SEARCH SUBKEY "));
		if ($uncached || (is_null($this->_configData) && $autoload)) {
			$this->load();
		}
		isDebug('settings_read') && $this->log()->debug(Hpls_Debug::print_r($this->_configData, __METHOD__ . " CONFIG "));
		if (FALSE === is_null($this->_configData)) {
			if (empty($subkey)) {
				$subkey = array();
			}
			if (FALSE === is_array($subkey)) {
				$subkey = explode(Hpls_Settings::KEY_DELIMITER, trim($subkey));
			}
			
			//prefix/namespace prependen
			if (!empty($this->_prefixKey)) {
				$subkey = array_merge($this->_prefixKey, $subkey);
			}
			
			if (empty($subkey)) {
				return $this->_configData;
			}
			
			isDebug('settings_read') && $this->log()->debug(Hpls_Debug::print_r($subkey, __METHOD__ . " SEARCH SUBKEY ARRAY "));
			$it = $this->_configData;
			
			foreach ($subkey as $keyPart) {
				if (isset($it->{$keyPart})) {
					//rekursiv absteigen
					isDebug('settings_read') && $this->log()->debug(Hpls_Debug::print_r($keyPart, __METHOD__ . " KEYPART EXISTS "));
					$it = $it->{$keyPart};
				} else {
					//der Key match nicht auf die gegebene Struktur	
					if ($create) {
						$it->{$keyPart} = new Zend_Config(array(), true);
						$it = $it->{$keyPart};
					} else {
						return null;
					}
				}
			}
			return $it;
		} else {
			return null;
		}
	}

	/**
	 * @todo comment
	 * @param string|array:string $subkey
	 * @param Zend_Config $config
	 */
	public function setConfig($subkey, Zend_Config $config) {
		
		if (empty($subkey)) {
			
			if (empty($this->_prefixKey)) {
				$this->_configData = $config;
				return;
			}
			$subkey = array();
		}
		
		if (!is_array($subkey)) {
			$subkey = explode(Hpls_Settings::KEY_DELIMITER, trim($subkey));
		}
		
		//prefix/namespace prependen
		if (!empty($this->_prefixKey)) {
			$subkey = array_merge($this->_prefixKey, $subkey);
		}
		
		isDebug('settings_write') && $this->log()->debug(Hpls_Debug::print_r($subkey, __METHOD__));
		
		if (is_null($this->_configData)) {
			$this->_configData = new Zend_Config(array(), true);
		}
		
		if (count($subkey) > 1) {
			$lastkey = array_pop($subkey); //dem letzten subkey wird $config zugewiesen und abgestiegen wird nur bis zu dem key davor 
			$data = $this->_configData;
			//iterativer abstieg in der keyhierarchie
			foreach ($subkey as $k) {
				//wenn der subkey nicht existier, oder etwas anderes als Zend_Config ist, dann überschreiben
				if ((FALSE === isset($data->{$k})) || (!$data->{$k} instanceof Zend_Config)) {
					$data->{$k} = new Zend_Config(array(), true);
				}
				$data = $data->{$k};
			}
			$data->{$lastkey} = $config;
		} else {
			$this->_configData->{$subkey[0]} = $config;
		}
	}

	public function clearCache() {
		$this->_configData = null;
	}
	
	protected function setPrefixKey($key) {
		if (is_array($key)) {
			$this->_prefixKey = $key;
		} elseif (empty($key)) {
			$this->_prefixKey = null;
		} else {
			$this->_prefixKey = explode(Hpls_Settings::KEY_DELIMITER, $key);
		}
	}

	protected function getWholeConfig() {
		return $this->_configData;
	}

	/**
	 * @return the $_options
	 */
	protected function getOptions() {
		return $this->_options;
	}

	/**
	 * @param array:string $_options
	 */
	protected function setOptions($_options) {
		$this->_options = $_options;
	}

}