<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 25.05.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Utils/Logusing.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
interface Hpls_Storage_ReaderInterface {

	function connect();

	function disconnect();

	function addFilter($filterString);

	function setAuth($user, $pass);

	function getValues();

	//	function setSearchParam($paramName, $paramValue);
}

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
abstract class Hpls_Storage_ReaderAbstract extends Hpls_Utils_Logusing implements 
	Hpls_Storage_ReaderInterface {
	
	protected $_options = null;
	protected $_filter = array();
	protected $_name = null;
	/**
	 * @todo comment
	 * @var Hpls_Var_Matcher
	 */
	protected $_userMapper = null;
	protected $_authCredentials = array();

	function __construct($name, $options = array(), $userMapper = null) {
		$this->_name = $name;
		$this->_userMapper = $userMapper; //unbedingt vor setOptions(), da er dort überschrieben werden darf und kann
		$this->setOptions($options);
	}

	function addFilter($filter) {
		$this->_filter[] = $filter;
	}

	function setAuth($user, $pass) {
		$this->_authCredentials['username'] = $user;
		$this->_authCredentials['password'] = $pass;
	}

	function getFilter($type = null) {
		if (empty($type)) {
			return $this->_filter;
		}
		$ret = array();
		foreach ($this->_filter as $filter) {
			if (strcasecmp($filter['type'], $type) == 0) {
				$ret[] = $filter;
			}
		}
		return $ret;
	}

	/**
	 * Der Name mit dem sich an der Quelle angemeldet wird.
	 * @return Ambigous <multitype:, NULL>
	 */
	function getAuthUsername() {
		return isset($this->_authCredentials[Hpls_Consts0::OPT_USERNAME]) ? $this->_authCredentials[Hpls_Consts0::OPT_USERNAME] : (isset($this->_options[Hpls_Consts0::OPT_USERNAME]) ? $this->_options[Hpls_Consts0::OPT_USERNAME] : null);
	}

	function getAuthPassword() {
		return isset($this->_authCredentials['password']) ? $this->_authCredentials['password'] : (isset($this->_options['password']) ? $this->_options['password'] : null);
	}

	function setAuthPassword($pass) {
		$this->_authCredentials['password'] = $pass;
	}

	/**
	 * Setzt den Usernamen, nach dem gesucht wird
	 * @param unknown_type $username
	 */
	function setUsername($username) {
		$this->_options[Hpls_Consts0::OPT_USERNAME] = $username;
		$this->getUsernameMapper()->setVariable(Hpls_Consts0::AUTH_VAR_LOGIN, $username);
	}

	function getOption($key, $default = null) {
		if (isset($this->_options[$key])) {
			return $this->_options[$key];
		}
		return $default;
	}

	function getFilterOption($filterIdx, $optionKey, $default = null) {
		if (isset($this->_filter[$filterIdx]) && isset($this->_filter[$filterIdx][$optionKey])) {
			return $this->_filter[$filterIdx][$optionKey];
		}
		
		return $default;
	}

	/**
	 * @param array|null $_options
	 */
	public function setOptions($options) {
		$this->_options = $options;
	}

	protected function getConfigPathString($path) {
		return (string) $path;
	}

	/**
	 * @todo IMPLEMENT
	 */
	protected function applyValueFilter($searchFilter, &$value, $key = null) {
		return $value;
	}

	/**
	 * @todo comment
	 * @return Hpls_Var_Matcher
	 */
	protected function getUsernameMapper() {
		if (is_null($this->_userMapper)) {
			$usermap = $this->getOption('usermap', null);
			if (!empty($usermap) && isset($usermap['match'])) {
				$this->_userMapper = new Hpls_Var_Matcher($usermap);
			} else {
				$this->_userMapper = new Hpls_Var_Matcher(null);
			}
		}
		return $this->_userMapper;
	}

}