<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created        : 28.09.2010
 * created by     : author
 *
 * last changed   : $LastChangedDate: 2012-03-25 18:46:03 +0200 (So, 25. Mär 2012) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9273 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Utils/Db.php $
 * 
 * author         : author
 */
/**
 */
require_once 'Zend/Db.php';

/**
 *
 * @todo comment
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9273 $
 * @since ??
 */
class Hpls_Utils_Db {
	
	const KEY_HOST = 'host';
	const KEY_PORT = 'port';
	const KEY_DBNAME = 'dbname';
	const KEY_TYPE = 'type';
	const KEY_USERNAME = 'username';
	const KEY_PASSWORD = 'password';
	const KEY_TABLEPREFIX = 'tableprefix';
	const KEY_VENDOR = 'vendor';
	const KEY_DESCRIPTION = 'description';
	protected static $ALLOWED_PROPERTIES = array(
			self::KEY_DBNAME,
			self::KEY_DESCRIPTION,
			self::KEY_HOST,
			self::KEY_PASSWORD,
			self::KEY_PORT,
			self::KEY_TABLEPREFIX,
			self::KEY_TYPE,
			self::KEY_USERNAME,
			self::KEY_VENDOR );
	
	const TYPE_DB_MYSQL = 'mysql';
	const TYPE_DB_PGSQL = 'pgsql';
	const TYPE_DB_SQLITE = 'sqlite';
	public static $MYSQL_ALIASES = array(
			'mysql' );
	public static $PGSQL_ALIASES = array(
			'pgsql' );
	public static $SQLITE_ALIASES = array(
			'sqlite' );
	
	const ERR_CAT_STRUCTURE = 1;
	const ERR_CAT_CONNECTION = 2;
	const ERR_CAT_UNKNOWN = -1;

	public static function map(array $fields, $resultSet, $target) {
	
	}

	/**
	 * Mapt eine Angabe in einer Config-Datei in eine Zend_Db-DBMS-Angabe.
	 * <pre>
	 * db.port=5432
	 * db.host = localhost
	 * db.dbname=mailtrace
	 * db.type = pgsql
	 * db.username=mailtrace
	 * db.password=iey2oN8i
	 * </pre>
	 * 
	 * @param $configstring array:string
	 */
	public static function mapDriverType2ZendDb(array $config, $directOptions = NULL) {
		$type = strtolower($config['type']);
		$params = array();
		$adapter = null;
		
		if (in_array($type, self::$MYSQL_ALIASES)) {
			$adapter = 'Pdo_Mysql';
			$params['host'] = empty($config['host']) ? 'localhost' : $config['host'];
			$params['username'] = $config['username'];
			$params['password'] = empty($config['password']) ? null : $config['password'];
			$params['dbname'] = $config['dbname'];
			if (!empty($config['port'])) {
				$params['port'] = $config['port'];
			}
			$params['driver_options'] = array(
					PDO::MYSQL_ATTR_USE_BUFFERED_QUERY => true );
		
		} elseif (in_array($type, self::$PGSQL_ALIASES)) {
			$adapter = 'Pdo_Pgsql';
			$params['host'] = empty($config['host']) ? 'localhost' : $config['host'];
			$params['username'] = $config['username'];
			$params['password'] = empty($config['password']) ? null : $config['password'];
			$params['dbname'] = $config['dbname'];
			if (!empty($config['port'])) {
				$params['port'] = $config['port'];
			}
		} elseif (in_array($type, self::$SQLITE_ALIASES)) {
			$adapter = 'Pdo_Sqlite';
			$params['dbname'] = $config['dbname'];
		} else {
			require_once 'Hpls/Exception/Configuration.php';
			throw new Hpls_Exception_Configuration("DB type '$type' is not recognized or supported.");
		}
		
		$dbObj = Zend_Db::factory($adapter, $params);
		return $dbObj;
	}

	public static function toPreparedValues(array $x) {
		$ret = array();
		foreach ($x as $k => $v) {
			if (stripos($k, ':') === FALSE) {
				$ret[":$k"] = $v;
			} else {
				$ret[$k] = $v;
			}
		}
		return $ret;
	}

	/**
	 *
	 * @todo comment
	 * @param $db Zend_Db_Adapter_Abstract
	 * @param $values array
	 * @param $logic unknown_type
	 * @return string
	 */
	public static function toWhere(Zend_Db_Adapter_Abstract $db, array $values, $logic = 'AND', $op = '=') {
		if (count($values) === 0)
			return '';
		$ret1 = array();
		$ret2 = array();
		foreach ($values as $k => $v) {
			$ret1[] = $db->quoteIdentifier($k) . " $op " . (stripos($k, ':') === FALSE ? ":col_$k" : $k);
			$ret2[(stripos($k, ':') === FALSE ? ":col_$k" : $k)] = $v;
		}
		return array(
				'sql' => implode(" $logic ", $ret1),
				'values' => $ret2 );
	}

	public static function getErrorCategory(Zend_Db_Adapter_Abstract $db, $code) {
		return self::ERR_CAT_UNKNOWN;
	}

	public static function dbExists(array $params) {
		$dsn = '';
		$ddl = '';
		$dbname = $params['dbname'];
		switch (strtolower($params['type'])) {
			case 'pgsql' :
				$dsn = "pgsql:dbname=$dbname;host=" . $params['host'] . ';';
				
				$ddl = 'SELECT 1;';
				$ddl .= ';';
				break;
			case 'mysql' :
				$dsn = "mysql:dbname=$dbname;host=" . $params['host'];
				$ddl = "SELECT 1;";
				break;
			case 'sqlite' :
				$dsn = "sqlite:" . $params['host'];
				$ddl = "SELECT 1;";
				break;
		}
		
		try {
			$db = new PDO($dsn, $params['username'], $params['password']);
			$db->exec($ddl);
			return true;
		} catch (Exception $ex) {
			return false;
		}
	}

	/**
	 *
	 * @todo comment
	 * @param $params array <ul>
	 *        <li> 'type': pgsql|mysql|sqlite
	 *        <li> 'host'
	 *        <li> 'port'
	 *        <li> 'username'
	 *        <li> 'password'
	 *        <li> 'dbname'
	 *        <li> 'owner': (optional) Der spätere Besitzer der Datenbank.
	 *        </ul>
	 */
	public static function createDB(array $params) {
		$dsn = '';
		$ddl = '';
		switch (strtolower($params['type'])) {
			case 'pgsql' :
				$dsn = "pgsql:dbname=postgres;host=" . $params['host'] . ';';
				$ddl = 'CREATE DATABASE "' . $params['dbname'] . '"';
				if (isset($params['owner'])) {
					$ddl .= ' OWNER = "' . $params['owner'] . '"';
				}
				$ddl .= ';';
				break;
			case 'mysql' :
				$dsn = "mysql:host=" . $params['host'];
				$ddl = "CREATE DATABASE `" . $params['dbname'] . "`;";
				break;
			case 'sqlite' :
				$dsn = "sqlite:" . $params['host'];
				$ddl = "CREATE DATABASE `" . $params['dbname'] . "`;";
				break;
		}
		$db = new PDO($dsn, $params['username'], $params['password']);
		$db->exec($ddl);
	}

	/**
	 *
	 * @todo comment
	 * @param $params array <ul>
	 *        <li> 'type': pgsql|mysql|sqlite
	 *        <li> 'host'
	 *        <li> 'port'
	 *        <li> 'username'
	 *        <li> 'password'
	 *        <li> 'dbname'
	 *        </ul>
	 */
	public static function dropDB(array $params) {
		$dsn = '';
		$ddl = '';
		switch (strtolower($params['type'])) {
			case 'pgsql' :
				$dsn = "pgsql:dbname=postgres;host=" . $params['host'] . ';';
				$ddl = 'DROP DATABASE "' . $params['dbname'] . '";';
				break;
			case 'mysql' :
				$dsn = "mysql:host=" . $params['host'];
				$ddl = "DROP DATABASE `" . $params['dbname'] . "`;";
				break;
			case 'sqlite' :
				$dsn = "sqlite:" . $params['host'];
				$ddl = "DROP DATABASE `" . $params['dbname'] . "`;";
				break;
		}
		$db = new PDO($dsn, $params['username'], $params['password']);
		$db->exec($ddl);
	}
}