<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 12.09.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hpls_Utils_Mail {

	/**
	 * @todo comment
	 * @param string|array $to
	 * @param string $subject
	 * @param string $text
	 * @param string $from
	 * @param string $html
	 * @param array $config
	 * @throws Hpls_Exception_Configuration
	 * @throws Hpls_Exception_Utils
	 * @throws Hpls_Exception_Mail Wenn irgendwas beim senden schief ging
	 */
	public static function sendMail($to, $subject, $text, $from, $html = false, array $config = array()) {
		require_once 'Zend/Mail.php';
		if (empty($config)) {
			require_once 'Hpls/Settings.php';
			$cfg = Hpls_Settings::i()->getConfig('ac.mail.smtp');
			if (is_null($cfg)) {
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration('ac.mail.smtp is not set.');
			}
			if ($cfg instanceof Zend_Config) {
				$cfg = $cfg->toArray();
			}
		} else {
			$cfg = $config;
		}
		
		if (isset($cfg['auth_required']) && Hpls_Utils::toBool($cfg['auth_required']) && is_array($cfg['auth'])) {
			$cfg['username'] = $cfg['auth']['username'];
			$cfg['password'] = Hpls_Utils::getClearPassword($cfg['auth']['password']);
			$cfg['auth'] = $cfg['auth']['method'];
		} else {
			//auth-config löschen, damit Zend das als unauthentifiziert akzeptiert
			unset($cfg['auth']);
			unset($cfg['username']);
			unset($cfg['password']);
		}
		if (!isset($cfg['type'])) {
			$cfg['type'] = 'smtp';
		}
		
		if (isset($cfg['port']) && empty($cfg['port'])) {
			unset($cfg['port']);
		}
		
		$mail = new Zend_Mail();
		
		if (!is_array($to)) {
			$to = explode(',', $to);
		}
		if (isset($to['mail'])) {
			//only 1 mail address
			$mail->addTo($to['mail'], isset($to['name']) ? $to['name'] : '');
		} else {
			foreach ($to as $mailTo) {
				if (is_array($mailTo)) {
					$mail->addTo($mailTo['mail'], isset($mailTo['name']) ? $mailTo['name'] : '');
				} else {
					$mail->addTo($mailTo);
				}
			}
		}
		$mail->setFrom($from);
		
		$mail->setSubject($subject);
		if ($html) {
			$mail->setBodyHtml($text);
		} else {
			$mail->setBodyText($text);
		}
		
		isDebug('send_mail') && Hpls_Utils::log()->debug(Hpls_Debug::print_r($cfg, __METHOD__));
		
		$mta = null;
		switch (strtolower($cfg['type'])) {
			case 'smtp' :
				require_once 'Zend/Mail/Transport/Smtp.php';
				//				$config = array(...)
				$mta = new Zend_Mail_Transport_Smtp($cfg['host'], $cfg);
				
				break;
			case 'php' :
			default :
				require_once 'Zend/Mail/Transport/Sendmail.php';
				$mta = new Zend_Mail_Transport_Sendmail();
				break;
		}
		if (is_null($mta)) {
			require_once 'Hpls/Exception/Utils.php';
			throw new Hpls_Exception_Utils('Could not create MTA', Hpls_Consts0::ERR_UTILS_MAIL_NOMTA);
		}
		try {
			if (isDebug('send_mail')) {
				Hpls_Utils::log()->debug(Hpls_Debug::print_r($mail, __METHOD__ . ' MAIL '));
				Hpls_Utils::log()->debug(Hpls_Debug::print_r($mta, __METHOD__ . ' MTA: '));
			}
			$mail->send($mta);
		} catch (Exception $e1) {
			Hpls_Utils::log()->err($e1);
			require_once 'Hpls/Exception/Mail.php';
			throw new Hpls_Exception_Mail('E-Mail konnte nicht gesendet werden: ' . $e1->getMessage(), Hpls_Consts0::ERR_UTILS_MAIL_SEND, $e1);
		}
	}
}