<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 23.06.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Var/Subster.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
class Hpls_Var_Subster {
	protected $_variableMap = null;
	/**
	 * Case-Insenstive variablen
	 * @var array:string:string
	 */
	protected $_variableMapI = null;
	/**
	 * @todo comment
	 * @var false|CASE_UPPER|CASE_LOWER
	 */
	protected $_changeCase = false;

	/**
	 * @return the $_targetPattern
	 */
	public function getTargetPattern() {
		return $this->_targetPattern;
	}

	/**
	 * @param field_type $_targetPattern
	 */
	public function setTargetPattern($vars = array()) {
		$this->setVariables($vars);
	}

	/**
	 * @todo comment
	 * @param string $name
	 * @param mixed $value
	 * @param boolean $caseSensitive
	 */
	public function setVariable($name, $value, $caseSensitive = false) {
		if ($caseSensitive) {
			$this->_variableMap[$name] = $value;
		} else {
			$this->_variableMapI[strtoupper($name)] = $value;
		}
	}

	public function unsetVariable($name) {
		unset($this->_variableMap[$name]);
		unset($this->_variableMapI[$name]);
	}

	public function setVariables($vars, $caseSensitive = false) {
		if ($caseSensitive) {
			$this->_variableMap = $vars;
			$this->_variableMapI = array();
		} else {
			$this->_variableMapI = array_change_key_case($vars, CASE_UPPER);
			$this->_variableMap = array();
		}
	}

	public function getVariable($name) {
		if (isset($this->_variableMap[$name])) {
			return $this->_variableMap[$name];
		} elseif ($this->_variableMapI[strtoupper($name)]) {
			return $this->_variableMapI[strtoupper($name)];
		}
		return null;
	}

	/**
	 * @todo comment
	 * @param string|NULL $targetPattern
	 * @param string|NULL $text
	 * @return string
	 */
	public function getSubstituted($targetPattern = null, $text = null) {
		isDebug('var_matcher') && $this->log()->debug(__METHOD__ . "($targetPattern,$text)  - START");
		isDebug('var_matcher') && $this->log()->debug(Hpls_Debug::print_r($this->_variableMap, 'VarMap'));
		isDebug('var_matcher') && $this->log()->debug(Hpls_Debug::print_r($this->_variableMapI, 'VarMapI'));
		if (!empty($this->_variableMap)) {
			$replace = $this->generateReplace(true);
			$targetPattern = str_replace($replace['search'], $replace['replace'], $targetPattern);
		}
		if (!empty($this->_variableMapI)) {
			$replace = $this->generateReplace(false);
			$targetPattern = str_ireplace($replace['search'], $replace['replace'], $targetPattern);
		}
		return $this->getChangeCase() !== false ? ($this->getChangeCase() === CASE_UPPER ? strtoupper($targetPattern) : strtolower($targetPattern)) : $targetPattern;
	}

	public function generateReplace($caseSensitive = false) {
		isDebug('var_matcher') && $this->log()->debug(Hpls_Debug::print_r($this->_variableMapI, __METHOD__ . " - START"));
		$search = array();
		$replace = array();
		if ($caseSensitive) {
			foreach ($this->_variableMap as $name => $value) {
				$search[] = "[[$name]]";
				$replace[] = $value; //$this->getChangeCase() !== false ? ($this->getChangeCase() === CASE_UPPER ? strtoupper($value) : strtolower($value)) : $value;
			}
		} else {
			foreach ($this->_variableMapI as $name => $value) {
				$search[] = "[[$name]]";
				$replace[] = $value; //$this->getChangeCase() !== false ? ($this->getChangeCase() === CASE_UPPER ? strtoupper($value) : strtolower($value)) : $value;
			}
		}
		return array(
				'search' => $search,
				'replace' => $replace );
	}

	/**
	 * @todo comment
	 * @return Zend_Log
	 */
	protected function log() {
		return Hpls_Utils::log();
	}

	/**
	 * @return the $_changeCase
	 */
	public function getChangeCase() {
		return $this->_changeCase;
	}

	/**
	 * @param false $_changeCase
	 */
	public function setChangeCase($_changeCase) {
		$this->_changeCase = $_changeCase;
	}

}